<?php
/**
 * @name Model_adm
 * @description Classe modèle de la partie d'administration d'Esia
 *		Toutes les méthodes sont déclarée en 'static' ainsi on garde les Objets PDO pour les programmeurs extérieurs et pour faire des modifications, ils ne sont pas /
 *		obligé de connaitre la hierachie de classe d'ESIA
 * @author Nicolas Biersart <n.biersart@esia-network.com>
 * @contributor
 * @copyright Nicolas Biersart 2012
 * @license
 * @esia 1.00
 * @version 1.0.0
 * @date 10/04/2013
 */
include_once DEVICE_CLASS;
include_once GESA_CLASS;
include_once MVCMODEL;
//include_once $path.'/plugin/gesa/libs/gesa.class.php';
class Model_adm extends Model_esia
{
	/**
	 * retourne un tableau associatif contenant les informations sur les droits d'accès.
	 * @name getAccess
	 * @param PDO $dbconnection
	 * @param integer $id (optionnelle) id du droit d'utilisation
	 * @retval un tableau associatif organiser comme suit:
	 * 		$arr[<privilege_id>][<privilege_id>]=identifiant du privilège
	 *		$arr[<privilege_id>][<privilege_name>]=nom du privilège
	 * 		$arr[<privilege_id>][<privilege_site>]=si il se rapporte à un droit sur un noeud ou un droit sur le site
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getAccess($dbconnection,$id = null)
	{
		if(isset($id))
		{
			$query =  $dbconnection->prepare("SELECT privilege_id,privilege_name,privilege_site,privilege_description FROM esia_privileges WHERE privilege_id =?;");
			parent::executeStatement($query,array($id));
		}
		else
		{
			$query =  $dbconnection->prepare("SELECT privilege_id,privilege_name,privilege_site,privilege_description FROM esia_privileges;");
			parent::executeStatement($query);
		}
		$list_access=array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$list_access[$row->privilege_id] = array('id'   => $row->privilege_id,
					'name' => $row->privilege_name,
					'site' => $row->privilege_site,
					'description' => $row->privilege_description);
		}
		return $list_access;
	}

	/**
	 * retourne un tableau associatif contenant les dernières lignes des fichiers de logs
	 * @name getDaemonLog
	 * @param PDO $dbconnection
	 * @param integer $type si Model_esia::LOG_DAEMON_CHECK log du démon de check
	 *	      				si model_esia::LOG_DAEMON_TRAP log du démon de trap
	 * @retval $arr[]=array('date'=>$in[0],'state'=>$in[1],'message'=>$in[2]); 
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getDaemonLog($dbconnection,$type=Model_esia::LOG_DAEMON_CHECK)
	{
		if($type == model_adm::LOG_DAEMON_TRAP)
		{
			$query = $dbconnection->prepare("SELECT config_key, config_value
					FROM esia_configuration
					WHERE config_key = 'SYS_ESIATRAPD_LOG';");
		}
		else
		{
			$query = $dbconnection->prepare("SELECT config_key, config_value
					FROM esia_configuration
					WHERE config_key = 'SYS_ESIADAEMON_LOG';");
		}
		$query = parent::executeStatement($query);
		$row=$query->fetch(PDO::FETCH_OBJ);
		$lines = Model_adm::tail($row->config_value, 10);
		$arr= array();
		foreach($lines as $line)
		{
			$in = preg_split('/#/',$line);
			if($in[0]!=$line)
			{
				$arr[]=array('date'=>$in[0],'state'=>$in[1],'message'=>$in[2]);
			}
		}
		return $arr;
	}

	/**
	 * retourne un tableau associatif contenant les dernières lignes des fichiers de logs
	 * @name countServiceByPlugin
	 * @param PDO $dbconnection
	 * @param integer $id id du plugin
	 * @retval le nom de service exécutant se plugin
	 * @exception EsiaDbException si une erreur survient
	 */
	static function countServiceByPlugin($dbconnection,$id)
	{
		$query= $dbconnection->prepare("SELECT count(*) as nbr
				FROM esia_services
				WHERE service_plugin_id=?;");
		$query = parent::executeStatement($query,array($id));
		$row=$query->fetch(PDO::FETCH_OBJ);
		return $row->nbr;
	}
	
	/**
	 * équivalent de la fonction tail en shell retourne les X dernière ligne d'un fichier
	 * @name tail
	 * @param string $file chemin d'accès au fichier de log
	 * @param integer $lines nombre de lignes à retourner
	 * @retval un tableau contenant les X dernières lignes du fichier
	 * @exception /
	 */
	private static function tail($file, $lines)
	{
		//global $fsize;
		$handle = @fopen($file, "r");
		if($handle!==FALSE)
		{
			$linecounter = $lines;
			$pos = -2;
			$beginning = false;
			$text = array();
			while ($linecounter > 0)
			{
				$t = " ";
				$buff="";
				while ($t != "\n") {
					if(fseek($handle, $pos, SEEK_END) == -1) {
						$beginning = true;
						break;
					}
					$t = fgetc($handle);
					$buff.=$t;
					$pos --;
				}
				$linecounter --;
				$text[] = strrev($buff);
				if ($beginning) break;
			}
			fclose ($handle);
			return array_reverse($text);
		}
		else
		{
			return array();
		}
	}
		
	/**
	 * retourne un tableau associatif contenant les 5 derniers plugins exécutés par esiaDaemon
	 * @name getLastExecCheckPlugin
	 * @param PDO $dbconnection
	 * @retval $arr[]= array('device'=>$row->device_name,
 	 *						 'name'=>$row->service_name,
	 *						 'page_control'=>$page_control,
	 *						 'state' =>$row->service_previous_state,
	 *						 'message'=>$row->service_last_stdout,
	 *						 'date'=>$row->timeexec);
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getLastExecCheckPlugin($dbconnection)
	{
		$query = $dbconnection->prepare("SELECT device_name,device_type,service_previous_state, to_char(service_last_exec_time,'DD/MM/YYYY HH24:MI:SS') as timeexec, service_last_stdout, service_name
				FROM esia_services
				JOIN esia_device ON service_device_id = device_id
				ORDER BY service_last_exec_time
				LIMIT 5;");
		$query = parent::executeStatement($query);
		$arr= array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$page_control = Model_esia::getPageControl($dbconnection, $row->device_type);
			if($page_control!=null)
			{
				$page_control = preg_replace('#:name#',$row->device_name,$page_control);
			}
			$arr[]= array('device'=>$row->device_name,
					'name'=>$row->service_name,
					'page_control'=>$page_control,
					'state' =>$row->service_previous_state,
					'message'=>$row->service_last_stdout,
					'date'=>$row->timeexec
			);
		}
		return $arr;

	}

	/**
	 * récupère les groupes d'utilisateurs
	 * @name getGroup
	 * @param PDO $dbconnection
	 * @param integer|string $id (optionnel) id ou nom du group a récupérer
	 * @retval un tableau contenant
	 *	array[<id du group>]['id']	id du groupe
	 *	array[<id du group>]['name'] nom du groupe
	 *	array[<id du group>]['description'] description du groupe
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getGroup($dbconnection,$id=null)
	{
		if(isset($id)&&is_numeric($id))
		{
			$query = $dbconnection->prepare("SELECT group_id,group_name,group_description FROM esia_groups WHERE group_id=?");
			$query = parent::executeStatement($query,array($id));
		}
		elseif(isset($id))
		{
			$query = $dbconnection->prepare("SELECT group_id,group_name,group_description FROM esia_groups WHERE group_name LIKE ?");
			$query = parent::executeStatement($query,array('%'.$id.'%'));
		}
		else
		{
			$query = $dbconnection->prepare("SELECT group_id,group_name,group_description FROM esia_groups;");
			$query = parent::executeStatement($query);
		}
		$list_grp=array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$list_grp[$row->group_id]=array('id'=>$row->group_id,
					'name'=> $row->group_name,
					'description'=>$row->group_description);
		}
		return $list_grp;

	}
	
	/**
	 * récupère les privilèges alloués au groupe donné
	 * @name getGroupPrivilege
	 * @param PDO $dbconnection
	 * @param integer|string $id id ou nom du groupe
	 * @retval un tableau contenant
	 *	array[<privilege_id>]['id']	id du droit
	 *	array[<privilege_id>]['name'] nom du droit
	 *	array[<privilege_id>]['description'] description du droit
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getGroupPrivilege($dbconnection,$id=null)
	{
		$sql = "SELECT privilege_id,privilege_name,privilege_description
		FROM esia_privileges
		LEFT JOIN esia_privilege_grantedto ON privilege_id = grant_privilege_id
		LEFT JOIN esia_groups ON group_id = grant_group_id ";
		if(isset($id)&&is_numeric($id))
		{
			$query = $dbconnection->prepare($sql.'WHERE group_id = ?;');
		}
		else
		{
			$query = $dbconnection->prepare($sql.'WHERE group_name= ?;');
		}
		$query = parent::executeStatement($query,array($id));
		$list_privi=array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$list_privi[$row->privilege_id]=array('id'=>$row->privilege_id,
					'name'=> $row->privilege_name,
					'description'=>$row->privilege_description);
		}
		return $list_privi;

	}

	/**
	 * récupère les privilèges non alloués au groupe donné
	 * @name getGroupNotPrivilege
	 * @param PDO $dbconnection
	 * @param string|integer $id (optionnel) id ou nom du groupe
	 * @retval un tableau contenant
	 *	array[<privilege_id>]['id']	id du droit
	 *	array[<privilege_id>]['name'] nom du droit
	 *	array[<privilege_id>]['description'] description du droit
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getGroupNotPrivilege($dbconnection,$id=null)
	{
		$sql = "SELECT privilege_id,privilege_name,privilege_description
		FROM esia_privileges
		EXCEPT
		SELECT privilege_id,privilege_name,privilege_description
		FROM esia_privileges
		LEFT JOIN esia_privilege_grantedto ON privilege_id = grant_privilege_id
		LEFT JOIN esia_groups ON group_id = grant_group_id ";
		if(isset($id)&&is_numeric($id))
		{
			$query = $dbconnection->prepare($sql.'WHERE group_id = ?;');
		}
		else
		{
			$query = $dbconnection->prepare($sql.'WHERE group_name= ?;');
		}
		$query = parent::executeStatement($query,array($id));
		$list_privi=array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$list_privi[$row->privilege_id]=array('id'=>$row->privilege_id,
					'name'=> $row->privilege_name,
					'description'=>$row->privilege_description);
		}
		return $list_privi;

	}

	/**
	 * retourne groupe de noeud
	 * @param PDO $dbconnection
	 * @param integer $id du group node
	 * @param string $name du group node
	 * @retval retourne list des groupes de noeud
	 * @exception EsiaDbException si une erreur survient
	 */
	
	static function getGroupNode($dbconnection,$id=null,$name=null)
	{
		$sql = "SELECT device_group_id,device_group_name,device_group_parent
		FROM esia_device_group ";
		
		$param=array();
		if(isset($id)&&is_numeric($id))
		{
			$query = $dbconnection->prepare($sql.'WHERE device_group_id = :id;');
			$param[]=$id;
		}		
		else
		{
			$query = $dbconnection->prepare($sql.'WHERE device_group_name= :name;');
			$param[]=$name;
		}
		$query = parent::executeStatement($query, $param);
		$list_node=array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$list_node[]=array('id'=>$row->device_group_id,
					'name'=> $row->device_group_name,
					'parent'=> $row->device_group_parent);
		}

		return $list_node;
	}
	
	
	/**
	 * Retourne les parents possibles
	 * @param PDO $dbconnection
	 * @param integer $id du groupe de device
	 * @retval array contenant les parents possible
	 * @exception 	EsiaDbException si une erreur survient
	 */
	static function getGroupDeviceParent($dbconnection,$id)
	{
		$query = $dbconnection->prepare("SELECT device_group_id,device_group_name,device_group_parent
		      FROM esia_device_group
			  WHERE device_group_id != :id
			  AND device_group_id NOT IN (SELECT group_id FROM getallchilddevicegroup(:id))
			  ORDER BY device_group_order;");
		$query = parent::executeStatement($query, array(':id'=> $id));
		$list_node=array();
		
		$req = $dbconnection->prepare("WITH RECURSIVE q AS (SELECT device_group_id, device_group_name,device_group_parent,device_group_order
	                      									FROM esia_device_group
	 														WHERE device_group_id = :id
	                    									UNION ALL
	                      									SELECT m.device_group_id, m.device_group_name,m.device_group_parent,m.device_group_order
	                        								FROM esia_device_group m
	                        								JOIN q ON q.device_group_parent = m.device_group_id)
	 									SELECT * FROM q ");
		
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{			
			$req = parent::executeStatement($req, array(':id'=> $row->device_group_id));
			$tmpname="";
			while($val=$req->fetch(PDO::FETCH_OBJ))
			{
				if($tmpname=="")
				{
					$tmpname = $val->device_group_name;
				}
				else
				{
					$tmpname = $val->device_group_name.">".$tmpname;
				}				
			}		
			$row->device_group_name=$tmpname;
			$list_node[]=array('id'=>$row->device_group_id ,'name'=>$row->device_group_name,'parent'=>$row->device_group_parent);
		}
		return $list_node;
	}
	
	/**
	 * récupère les noeuds auquelle le groupe a le droit d'accès
	 * @name getGroupAccessNode
	 * @param PDO $dbconnection
	 * @param string|integer $id id ou nom du groupe
	 * @retval un tableau contenant
	 *	array[<node id>]['id']	id du noeud
	 *	array[<node id>]['name'] nom du noeud
	 *	array[<node id>]['group'] nom du groupe de noeud
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getGroupAccessNode($dbconnection,$id=null)
	{
		$sql = "SELECT device_id,device_name,g2.device_group_name
		FROM esia_device
		LEFT JOIN esia_device_group_in ON device_id = group_in_device_id
		LEFT JOIN esia_device_group  as g2 ON device_group_id= group_in_device_group_id
		JOIN esia_group_device_access ON access_device_id= device_id
		JOIN esia_groups ON group_id = access_group_id ";
		if(isset($id)&&is_numeric($id))
		{
			$query = $dbconnection->prepare($sql.'WHERE group_id = ?;');
		}
		else
		{
			$query = $dbconnection->prepare($sql.'WHERE group_name= ?;');
		}
		$query = parent::executeStatement($query,array($id));
		$list_node=array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$list_node[$row->device_group_name][]=array('id'=>$row->device_id,
					'name'=> $row->device_name);
		}
		return $list_node;
	}
	
	/**
	 * récupère les noeuds auquelle le groupe n'a pas le droit d'accès
	 * @name getGroupNoAccessNode
	 * @param PDO $dbconnection
	 * @param integer|string $id id ou nom du groupe
	 * @retval un tableau contenant
	 *	array[<node group>]['id']	id du noeud
	 *	array[<node group>]['name'] nom du noeud
	 *	array[<node group>]['group'] nom du groupe de noeud
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getGroupNoAccessNode($dbconnection,$id=null)
	{
		$sql = "SELECT device_id,device_name,g2.device_group_name
		FROM esia_device
		LEFT JOIN esia_device_group_in ON device_id = group_in_device_id
		LEFT JOIN esia_device_group  as g2 ON device_group_id= group_in_device_group_id
		EXCEPT
		SELECT device_id,device_name,g2.device_group_name
		FROM esia_device
		LEFT JOIN esia_device_group_in ON device_id = group_in_device_id
		LEFT JOIN esia_device_group  as g2 ON device_group_id= group_in_device_group_id
		JOIN esia_group_device_access ON access_device_id= device_id
		JOIN esia_groups ON group_id = access_group_id ";
		if(isset($id)&&is_numeric($id))
		{
			$query = $dbconnection->prepare($sql.'WHERE group_id = ?;');
		}
		else
		{
			$query = $dbconnection->prepare($sql.'WHERE g2.group_name= ?;');
		}
		$query = parent::executeStatement($query,array($id));
		$list_node=array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$list_node[$row->device_group_name][]=array('id'=>$row->device_id,
					'name'=> $row->device_name);
		}
		return $list_node;

	}
	
	/**
	 * retourne les patterns de service si rien n'est spécifié => tout les patterns
	 *										si id est spécifié => le pattern demandé
	 *										si node est spécifié => les patterns associés au noeud
	 * @name getPatternService
	 * @param PDO $dbconnection
	 * @param integer $id (optionnelle) identifiant du pattern
	 * @param integer $node (optionnelle) identifant d'un noeud
	 * @retval un tableau contenant
	 *	array[<pattern_id>]['id']	identifiant du pattern
	 *	array[<pattern_id>]['name'] nom du pattern	
 	 *	array[<pattern_id>]['description'] sa description
	 *	array[<pattern_id>]['plugins']	tableau des plugins composant le pattern
 	 *	array[<pattern_id>]['plugins']['id'] id du plugin au sein du pattern (esia_service_pattern_included)
	 *	array[<pattern_id>]['plugins']['name'] nom du service
 	 *	array[<pattern_id>]['plugins']['parameter'] ses paramètres
	 *	array[<pattern_id>]['plugins']['interval'] interval d'exécution
 	 *	array[<pattern_id>]['plugins']['plugin_type'] type (management,default,trap,etc)
	 *	array[<pattern_id>]['plugins']['plugin_id'] id du plugin 
 	 *	array[<pattern_id>]['plugins']['alerting'] id du programme d'alerte
	 *	array[<pattern_id>]['plugins']['alerting_parameter'] paramètre du programme d'alerte
 	 *	array[<pattern_id>]['plugins']['unknown'] si il est exécuté en cas d'erreur inconnue
	 *	array[<pattern_id>]['plugins']['warning'] si il est exécuté en cas d'erreur warning
     *	array[<pattern_id>]['plugins']['critical'] si il est exécuté en cas d'erreur critique
	 *	array[<pattern_id>]['plugins']['priority'] priorité du service
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getPatternService($dbconnection,$id=null,$node=null)
	{
		$param=array();
		if(isset($node)&&is_numeric($node))
		{
			$sql = 'SELECT pattern_id,pattern_name,pattern_description,included_id,
							included_name,included_parameter,included_interval,plugin_id,plugin_type,plugin_name,
							included_program_id,included_program_parameter,included_unknown,included_warning,included_critical,included_priority
					FROM esia_service_patterns
					JOIN esia_service_pattern_included ON pattern_id = included_pattern_id
					JOIN esia_plugins ON plugin_id = included_plugin_id
					JOIN esia_services ON service_included_id = included_id
					WHERE service_device_id = ? AND service_delete=FALSE ';
			$param[]=$node;
		}
		else 
		{
			$sql = 'SELECT pattern_id,pattern_name,pattern_description,included_id,included_name,included_parameter,included_interval,plugin_id,plugin_type,plugin_name,
						included_program_id,included_program_parameter,included_unknown,included_warning,included_critical,included_priority,included_ok
					FROM esia_service_patterns
					LEFT JOIN esia_service_pattern_included ON pattern_id = included_pattern_id
					LEFT JOIN esia_plugins ON plugin_id = included_plugin_id';
			if(isset($id))
			{
				if(is_numeric($id))
				{
					$sql.=' WHERE pattern_id = ?;';
				}
				else
				{
					$sql.=' WHERE pattern_name= ?;';
				}
				$param[]=$id;
			}
		}
		$query = $dbconnection->prepare($sql);
		$query = parent::executeStatement($query,$param);
		$list=array();
		$plugins = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$list[$row->pattern_id]=array('id'=>$row->pattern_id,'name'=>$row->pattern_name,'description'=>$row->pattern_description);
			if(!empty($row->included_id))
			{
				$plugins[]=array('id'=>$row->included_id,
						'name'=>$row->included_name,
						'parameter'=>$row->included_parameter,
						'interval'=>$row->included_interval,
						'plugin_type'=>$row->plugin_type,
						'plugin_id'=>$row->plugin_id,
						'plugin_name'=>$row->plugin_name,
						'alerting'=>$row->included_program_id,
						'alerting_parameter'=> $row->included_program_parameter,
						'unknown'=> $row->included_unknown,
						'warning'=> $row->included_warning,
						'critical'=> $row->included_critical,
						'ok'=> $row->included_ok,
						'priority'=>$row->included_priority
						);
			}
			$list[$row->pattern_id]['plugins'] = $plugins;
			
		}
		return $list;
	
	}

	/**
	 * retourne les noeud associé à un pattern donné
	 * @name getPatternService
	 * @param PDO $dbconnection
	 * @param integer $id identifiant du pattern
	 * @retval un tableau contenant
	 * 	array[]['id'] id du noeud
	 * 	array[]['name'] nom du noeud
	 * 	array[]['description'] description du noeud
	 * 	array[]['type'] type du noeud
	 * 	array[]['group'] group du noeud
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getPatternServiceNodeAssign($dbconnection,$id)
	{
		$list=array();
		if(isset($id)&&is_numeric($id))
		{
			$query = $dbconnection->prepare("SELECT DISTINCT device_id,device_name,device_description,device_group_name,device_type
											FROM esia_device
											JOIN esia_group_device_access ON access_device_id = device_id
											JOIN esia_services ON service_device_id = device_id
											JOIN esia_service_pattern_included ON service_included_id= included_id
											WHERE device_delete = 'false'
											AND included_pattern_id = ?");
			$query = parent::executeStatement($query,array($id));
			while($row=$query->fetch(PDO::FETCH_OBJ))
			{
				$list[]=array('id'=>$row->device_id,
							  'name'=>$row->device_name,
							  'description'=>$row->device_description,
							  'type'=>$row->device_type,
							  'group'=>$row->device_group_name);				
			}
		}
		return $list;
	}
	
	/**
	 * Insère l'objet device dans la base de donnée.
	 * @name addNode
	 * @param PDO $dbconnection
	 * @param Device $dev L'objet Device à insérer dans la base de donnée
	 * @retval l'id du nouveau device
	 * @exception EsiaDbException si une erreur survient
	 */
	static function addNode($dbconnection, Device $dev )
	{
		$query = $dbconnection->prepare("INSERT INTO esia_device (
				device_id, device_name, device_type,device_ip_addr,device_protocol_connection, device_username, device_passwd,
				device_snmp_version,device_snmp_timer,device_snmp_community,device_snmp3_user,device_snmp3_sec_lvl,device_snmp3_auth_protocole,device_snmp3_auth_key,device_snmp3_priv_protocol,device_snmp3_priv_passphrase,device_group_name,device_description)
				VALUES(DEFAULT,:name,:type,:ip,:protocol,:login,:pwd,:snmp_ver,:snmp_timer,:community,:snmp3_user,:snmp3_seclvl,:snmp3_authprot,:snmp3_authkey,:snmp3_privprot,:snmp3_privpass,:grp,:description) RETURNING device_id;");
		$pass = $dev->getPassword();
		if(!empty($pass))
		{
			//chiffrement du mot de passe
			$td = mcrypt_module_open('rijndael-256', '', 'ecb', '');
			$iv = mcrypt_create_iv(mcrypt_enc_get_iv_size($td), MCRYPT_RAND);
			//on récupère la clé de chiffrement
			mcrypt_generic_init($td, DB_KEY_CRYPT, $iv);
			$pass = mcrypt_generic($td, $dev->getPassword());
			mcrypt_generic_deinit($td);
			mcrypt_module_close($td);
		}
		//$query->bindParam(":pwd", $pass, PDO::PARAM_LOB);
		$query = parent::executeStatement($query, array(':name' => $dev->getName(),
				':type'=>$dev->getType() ,
				':ip'=>$dev->getIPv4DecimalAdress(),
				':login'=>$dev->getLogin(),
				':pwd'=>pg_escape_bytea($pass) ,
				':snmp_ver'=>$dev->getSNMPObject()->getVersion(),
				':snmp_timer'=>$dev->getSNMPObject()->getTimeout(),
				':community'=>$dev->getSNMPObject()->getCommunity(),
				':snmp3_user'=>$dev->getSNMPObject()->getSNMP3User(),
				':snmp3_seclvl'=>$dev->getSNMPObject()->getSNMP3SecLevel(),
				':snmp3_authprot'=>$dev->getSNMPObject()->getSNMP3AuthProtocol(),
				':snmp3_authkey'=>$dev->getSNMPObject()->getSNMP3AuthKey(),
				':snmp3_privprot'=>$dev->getSNMPObject()->getPrivacyProtocol(),
				':snmp3_privpass'=>$dev->getSNMPObject()->getPrivacyKey(),
				':protocol'=>$dev->getTypeConnection(),
				':grp'=>$dev->getGroup(),
				':description'=>$dev->getDescription() ));
		$row=$query->fetch(PDO::FETCH_OBJ);
		$query = $dbconnection->prepare("INSERT INTO esia_device_group_in(group_in_device_id, group_in_device_group_id)VALUES (:id, :group);");
		$grps =  $dev->getGroupList();
		foreach($grps as $grp)
		{
			$query = parent::executeStatement($query,array(':id' => $row->device_id,':group'=>$grp));
		}
		global $lang;
		model_adm::addGdprRegister($dbconnection,'ADD','NODE',$lang['GDPR_ADD_NODE']. $dev->getName() ,$lang['GDPR_ADD_NODE_FINALITY']);
		return $row->device_id;
	}

	/**
	 * autorise le group 'root' a accéder à se device
	 * @name addAddDeviceInRootGroup
	 * @param PDO $dbconnection
	 * @param integer|Device $dev L'objet Device à ajouter au groupe root ou l'id du device
	 * @throws EsiaDbException
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function addAddDeviceInRootGroup($dbconnection,$dev )
	{
		if(@get_class($dev)=='Device')
		{
			$id =  $dev->getId();
		}
		elseif(@get_class($dev)=='RemoteDevice')
		{
			$id =  $dev->getId();
		}
		else
		{
			$id =  $dev;
		}
		$id =  $dev->getId();
		if(!empty( $id ))
		{
			$result = $dbconnection->prepare("SELECT group_id FROM esia_groups WHERE group_name='root'");
			$result = parent::executeStatement($result);
			$row=$result->fetch(PDO::FETCH_OBJ);
			$query = $dbconnection->prepare("INSERT INTO esia_group_device_access VALUES (:group_id,:dev_id);");
			$query = parent::executeStatement($query,array(':group_id'=>$row->group_id,
					':dev_id' =>$dev->getId() ));
		}
		else
		{
			throw new EsiaDbException("Id is NULL");
		}
	}
	
	/**
	 * Ajoute un nouveau plugin de test à esia
	 * @name addCheckPlugin
	 * @param PDO $dbconnection
	 * @param string $name Nom du plugin
	 * @param string $path Chemin d'accès à l'exécutable
	 * @param string $parameter paramètre par défaut
	 * @param integer $interval Interval de test
	 * @param string $description Description du plugin
	 * @param integer $priority  priorité du plugin (default 7)
	 * @retval id du plugin nouvellement inséré
	 * @exception EsiaDbException si une erreur survient
	 */
	static function addCheckPlugin($dbconnection,$name,$path,$parameter,$interval,$description,$priority=7)
	{
		$query= $dbconnection->prepare("INSERT INTO esia_plugins(plugin_id, plugin_name, plugin_path, plugin_type,
				plugin_default_parameter, plugin_default_interval_time, plugin_description,plugin_priority)
				VALUES (DEFAULT, :name, :path, 'default', :param,:interval,:description,:priority) RETURNING plugin_id;");
		$query = parent::executeStatement($query,array(':name' =>$name,
				':path'=>$path,
				':param'=>$parameter,
				':interval'=>$interval,
				':description'=>$description,
				':priority'=>$priority));
		$row=$query->fetch(PDO::FETCH_OBJ);
		return $row->plugin_id;
	}
	
	/**
	 * Ajoute un nouveau plugin de traitement de trap à esia
	 * @name addTrapPlugin
	 * @param PDO $dbconnection
	 * @param string $name Nom du plugin
	 * @param string $path Chemin d'accès à l'exécutable
	 * @param string $parameter aramètre par défaut
	 * @param string $oid oid de la trap SNMP
	 * @param string $description Description du plugin
	 * @param integer $priority priorité du plugin
	 * @retval id du plugin nouvellement inséré
	 * @exception EsiaDbException si une erreur survient
	 */
	static function addTrapPlugin($dbconnection,$name,$path,$parameter,$oid,$description,$priority=7)
	{
		$query= $dbconnection->prepare("INSERT INTO esia_plugins(plugin_id, plugin_name, plugin_path, plugin_type,plugin_trap_oid,plugin_default_parameter,plugin_default_interval_time, plugin_description,plugin_priority)
				VALUES (DEFAULT,:name,:path,'trap',:oid,:param,0,:description,:priority) RETURNING plugin_id;");
		$query = parent::executeStatement($query,array(':name' =>$name,
				':path'=>$path,
				':param'=>$parameter,
				':description'=>$description,
				':oid'=>$oid,
				':priority'=>$priority));
		$row=$query->fetch(PDO::FETCH_OBJ);
		return $row->plugin_id;
	}

	/**
	 * Ajoute un service à la base de donnée
	 * @name addService
	 * @param PDO $dbconnection
	 * @param string $name Nom du service
	 * @param integer $plugin id du plugin
	 * @param integer $node id du noeud
	 * @param integer $interval Interval de test
	 * @param string $parameter paramètre du service
	 * @param integer $priority priorité du service
	 * @param integer $alert id du programme a exécuter en cas d'erreur
	 * @param string $alert_param  = paramètre à utiliser
	 * @param bool $unknown si = 1 exécuter le programme d'alerte en cas d'erreur de type inconnu
	 * @param bool $warning si = 1 exécuter le programme d'alerte en cas d'erreur de type warning
	 * @param bool $critical si = 1 exécuter le programme d'alerte en cas d'erreur de type critical
	 * @throws EsiaDbException
	 * @retval l'identifiant du nouveau service
	 * @exception EsiaDbException si une erreur survient
	 */
	static function addService($dbconnection,$name,$plugin,$node,$interval,$parameter,$priority,$alert,$alert_param,$unknown=null,$warning=null,$critical=null,$ok=null)
	{
		if(is_numeric($node)&&is_numeric($plugin)&&is_numeric($interval))
		{
			//DEBUT GESA
			$gesa = Model_adm::getGesaLink($dbconnection,$node);
			if($gesa!=null)
			{
				if(gethostbyname($gesa->getName())==$gesa->getName())
				{
					$parameter = preg_replace('#\$NRPE_HOST#',$gesa->getIPv4Adress(),$parameter);
				}
				else
				{	
					$parameter = preg_replace('#\$NRPE_HOST#',$gesa->getName(),$parameter);
				}
			}
			//FIN GESA
			$query= $dbconnection->prepare("INSERT INTO esia_services(service_id,service_name,service_plugin_id, service_device_id, service_interval_time,service_parameter,service_priority, service_previous_state)
					VALUES (DEFAULT,:name,:plugin_id,:device_id,:interval,:parameter,:priority,4) RETURNING service_id;");
			$query = parent::executeStatement($query,array(':name'=>$name,
					':plugin_id'=>$plugin,
					':device_id'=>$node,
					':interval'=>$interval,
					':parameter'=>$parameter,
					':priority'=>$priority));
			$row=$query->fetch(PDO::FETCH_OBJ);
			//si on a spécifier un programme d'alerte
			if($unknown!=NULL && $ok!=null && $warning!=null && $critical!=null && (!empty($alert)||$alert!=0) )
			{
				$query= $dbconnection->prepare("INSERT INTO esia_service_response_program(
						response_service_id, response_program_id, response_parameter, response_warning, response_critical, response_unknown,response_ok)
						VALUES (:service, :program, :parameter,:warning, :critical, :unknown,:ok);");
				$query = parent::executeStatement($query,array(':service'=>$row->service_id,
						':program'=>$alert,
						':parameter'=>$alert_param,
						':warning'=>$warning,
						':critical'=>$critical,
						':ok'=>$ok,
						':unknown'=>$unknown));
			}
			$query= $dbconnection->prepare("INSERT INTO esia_service_pending_execute(pending_service_id)VALUES (:id);");
			$query = parent::executeStatement($query,array(':id'=>$row->service_id));
			global $lang;
			model_adm::addGDPRRegister($dbconnection,'ADD','SERVICE',$lang['GDPR_ADD_SERVICE']. $name,$lang['GDPR_ADD_SERVICE_FINALITY']);
			
			return $row->service_id;
		}
		else
		{
			throw new EsiaDbException('bad parameter: $plugin,$interval and $node must be numeric');
		}
	}

	/**
	 * ajoute les informations des graphiques
	 * @name addGraph
	 * @param PDO $dbconnection
	 * @param object|integer $node id du device ou un objet device
	 * @param string $url url d'accès au serveur munin/cacti
	 * @param integer $tree si cacti id de la tree au se trouve les graphs
	 * @param integer $leaf si cacti numéros de la variable leaf
	 * @throws EsiaDbException
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function addGraph($dbconnection, $node,$url,$tree=null,$leaf=null)
	{
		if(@get_class($node)=='Device')
		{
			$node =  $node->getId();
		}
		elseif(@get_class($node)=='RemoteDevice')
		{
			$node =  $node->getId();
		}
		else
		{
			$node =  $node;
		}
		if(!is_numeric($node))
		{
			throw new EsiaDbException('$node is not numeric or not object device');
		}
		if(!empty($tree) && !empty($leaf))
		{
			$query = $dbconnection->prepare("INSERT INTO esia_device_graph(graph_device_id, graph_url, graph_cacti_tree_id, graph_cacti_leaf_id, graph_software)
					VALUES (:node, :url, :tree, :leaf,'cacti');");
			$query = parent::executeStatement($query,array(':node'=>$node,
					':url'=>$url,
					':tree'=>$tree,
					':leaf'=>$leaf));

		}
		else
		{
			$query = $dbconnection->prepare("INSERT INTO esia_device_graph(graph_device_id, graph_url,graph_software)
					VALUES (:node, :url,'munin');");
			$query = parent::executeStatement($query,array(':node'=>$node,':url'=>$url));
		}

	}

	/**
	 * ajoute un nouvel utilisateur à l'application
	 * @name addUser
	 * @param PDO $dbconnection
	 * @param User $usermod un objet User à ajouter à la base de donnée
	 * @throws EsiaDbException
	 * @retval l'id de l'utilisateur
	 * @exception EsiaDbException si une erreur survient
	 */
	static function addUser($dbconnection,User $usermod)
	{
		if(! @get_class($usermod)=='User')
		{
			throw new EsiaDbException('$user is not a user\'s object');
		}
		$query=$dbconnection->prepare('INSERT INTO esia_users (user_id,user_name,user_password,user_change_password,user_email,user_lang,user_inscription_time,home_page)
				VALUES (default,:name,:password,1,:email,:lang,:now,:home_page) RETURNING user_id;');
		$query = parent::executeStatement($query,array(':name'=>$usermod->getName(),
				':password'=>$usermod->getHashPassword(),
				':email'=>$usermod->getEmail(),
				':lang'=>$usermod->getLang(),
				':now'=> $usermod->getInscriptionTime(),
				':home_page'=>$usermod->getHomePage()));
		$row=$query->fetch(PDO::FETCH_OBJ);
		global $lang;
		model_adm::addGDPRRegister($dbconnection,'ADD','USER',$lang['GDPR_ADD_USER']. $usermod->getName() ,$lang['GDPR_ADD_USER_FINALITY']);
		return $row->user_id;
	}
	
	/**
	 * ajoute l'utilisateur dans le groupe donné
	 * @name addUserInGroup
	 * @param PDO $dbconnection
	 * @param integer $user_id id de l'utilisateur
	 * @param integer $group_id id du group
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	
	static function addUserInGroup($dbconnection,$user_id,$group_id)
	{
		$query = $dbconnection->prepare("INSERT INTO esia_memberof VALUES (:group_id,:user_id);");
		$query = parent::executeStatement($query,array(':group_id'=>$group_id,
				':user_id'=>$user_id));
	}
	
	/**
	 * ajoute un groupe d'utilisateur à l'application
	 * @name addGroup
	 * @param PDO $dbconnection
	 * @param string $group nom du groupe
	 * @param string $description description du groupe
	 * @retval identifiant du groupe ajouté
	 * @exception EsiaDbException si une erreur survient
	 */
	static function addGroup($dbconnection,$group,$description)
	{
		$query = $dbconnection->prepare('INSERT INTO esia_groups VALUES (default,?,?) RETURNING group_id;');
		$query = parent::executeStatement($query,array($group,$description));
		$row=$query->fetch(PDO::FETCH_OBJ);
		return $row->group_id;
	}
	
	/**
	 * insére le pattern service dans la base de donnée
	 * @name addPatternService
	 * @param PDO $dbconnection
	 * @param string $name Nom du paternService
	 * @param string $description description du pattern service
	 * @param array $services tableau contenant les informations de base des plugins du pattern
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function addPatternService($dbconnection,$name,$description,$services=null)
	{
		$query = $dbconnection->prepare("INSERT INTO esia_service_patterns(pattern_id, pattern_name, pattern_description) VALUES (default, :name, :description) RETURNING pattern_id;");
		$query = parent::executeStatement($query,array(':name'=>$name,
				':description'=>$description));
		$row=$query->fetch(PDO::FETCH_OBJ);
		$pattern = $row->pattern_id;
		if(isset($services)&& is_array($services))
		{
			$query = $dbconnection->prepare("INSERT INTO esia_service_pattern_included(included_id, 
																		included_name, 
																		included_pattern_id, 
																		included_plugin_id, 
																		included_parameter, 
																		included_interval,
																		included_priority )
					VALUES (DEFAULT, :name, :pattern,:plugin,:parameter,:interval,:priority) RETURNING included_id ;");
			$queryUpdate = $dbconnection->prepare("UPDATE esia_service_pattern_included 
													SET included_program_id=:program, 
  														included_program_parameter=:alertparameter ,
  														included_unknown=:unkwnon,
														included_warning=:warning,
														included_critical=:critical,
														included_ok=:ok,
													WHERE included_id = :id;");
			foreach($services as $ser)
			{
				$query = parent::executeStatement($query,array(':name'=>$ser['name'],
						':pattern'=>$pattern,
						':plugin'=>$ser['plugin'],
						':parameter'=>$ser['parameter'],
						':interval'=>$ser['interval'],
						':priority'=>$ser['priority']));
				if(isset($ser['alerting_software']) && !empty($ser['alerting_software']))
				{
					$row=$query->fetch(PDO::FETCH_OBJ);
					$queryUpdate = parent::executeStatement($queryUpdate,array(':program'=>$ser['alerting_software'],
																			':alertparameter'=>	$ser['alerting_parameter'],
																			':unkwnon'=>$ser['unkwnon'],
																			':warning'=> $ser['warning'],
																			':critical'=>$ser['critical'],':ok'=>$ser['ok'],
																			':id'=>$row->included_id));
				}
			}
		}
		return $pattern;
	}

	/**
	 * ajoute un groupe de noeud  à l'application
	 * @name addGroupNode
	 * @param PDO $dbconnection
	 * @param string $group nom du groupe
	 * @param integer $parent groupe parent
	 * @retval identifiant du groupe de noeud ajouté
	 * @exception EsiaDbException si une erreur survient
	 */
	static function addGroupNode($dbconnection,$group,$parent=null)
	{
		if(isset($parent))
		{
			$query = $dbconnection->prepare('INSERT INTO esia_device_group(device_group_id, device_group_name, device_group_parent) 
											 VALUES (DEFAULT, ?, ?) RETURNING device_group_id;');
			$query = parent::executeStatement($query,array($group,$parent));
		}
		else
		{
			$query = $dbconnection->prepare('INSERT INTO esia_device_group(device_group_id, device_group_name, device_group_parent)
					VALUES (DEFAULT, ?, NULL) RETURNING device_group_id;');
			$query = parent::executeStatement($query,array($group));			
		}
		$row=$query->fetch(PDO::FETCH_OBJ);
		return $row->device_group_id;
	}

	/**
	 * Assigne un pattern de service à un noeud 
	 * @name assignPatternServiceNode
	 * @param PDO $dbconnection
	 * @param integer $patternid identifiant du pattern de service
	 * @param integer $node identifiant du noeud
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function assignPatternServiceNode($dbconnection,$patternid,$node)
	{
		$pattern = Model_adm::getPatternService($dbconnection,$patternid);
		$pattern = $pattern[$patternid];
		$query = $dbconnection->prepare("UPDATE esia_services SET service_included_id=:patternid WHERE service_id = :serid");
		foreach($pattern['plugins'] as $plugin)
		{
			//on ajoutou modifie un service
			$warning='FALSE';
			$critical='FALSE';
			$unknown='FALSE';
			$ok='FALSE';
			if(!empty($plugin['unknwon']))
				$unknown='TRUE';
			if(!empty($plugin['warning']))
				$warning='TRUE';
			if(!empty($plugin['critical']))
				$critical='TRUE';
			if(!empty($plugin['ok']))
				$ok='TRUE';
			$serid = model_adm::addService($dbconnection,$plugin['name'],$plugin['plugin_id'],$node,
								$plugin['interval'],$plugin['parameter'],$plugin['priority'],$plugin['alerting'],$plugin['alerting_parameter'],
								$unknown,$warning,$critical,$ok);
			$query = parent::executeStatement($query,array(':serid'=>$serid,':patternid'=>$plugin['id']));
		}		
	}

	/**
	 * met à jour l'objet device
	 * @name updateDevice
	 * @param PDO $dbconnection
	 * @param object Device $dev
	 * @throws Exception
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function updateDevice($dbconnection,Device $dev)
	{
		$id =  $dev->getId();
		if(!empty( $id ))
		{
			$pass = $dev->getPassword();
			if(!empty($pass))
			{
				//chiffrement du mot de passe
				$td = mcrypt_module_open('rijndael-256', '', 'ecb', '');
				$iv = mcrypt_create_iv(mcrypt_enc_get_iv_size($td), MCRYPT_RAND);
				//on récupère la clé de chiffrement
				mcrypt_generic_init($td, DB_KEY_CRYPT, $iv);
				$pass = mcrypt_generic($td, $dev->getPassword());
				mcrypt_generic_deinit($td);
				mcrypt_module_close($td);
			}
			$query = $dbconnection->prepare("UPDATE esia_device SET device_name=:name ,
					device_type=:type ,
					device_ip_addr=:ip,
					device_protocol_connection=:protocol,
					device_username=:login,
					device_passwd=:pwd,
					device_description =:description,
					device_snmp_version=:snmp_ver,
					device_snmp_timer=:snmp_timer,
					device_snmp_community=:community,
					device_snmp3_user=:snmp3_user,
					device_snmp3_sec_lvl=:snmp3_seclvl,
					device_snmp3_auth_protocole=:snmp3_authprot,
					device_snmp3_auth_key=:snmp3_authkey,
					device_snmp3_priv_protocol=:snmp3_privprot,
					device_snmp3_priv_passphrase=:snmp3_privpass,
					device_group_name=:grp
					WHERE device_id=:id;");
			$query = parent::executeStatement($query,array(':id' => $dev->getId(),
					':name' => $dev->getName(),
					':type'=>$dev->getType() ,
					':ip'=>$dev->getIPv4DecimalAdress(),
					':login'=>$dev->getLogin(),
					':pwd'=>pg_escape_bytea($pass) ,
					':snmp_ver'=>$dev->getSNMPObject()->getVersion(),
					':snmp_timer'=>$dev->getSNMPObject()->getTimeout(),
					':community'=>$dev->getSNMPObject()->getCommunity(),
					':snmp3_user'=>$dev->getSNMPObject()->getSNMP3User(),
					':snmp3_seclvl'=>$dev->getSNMPObject()->getSNMP3SecLevel(),
					':snmp3_authprot'=>$dev->getSNMPObject()->getSNMP3AuthProtocol(),
					':snmp3_authkey'=>$dev->getSNMPObject()->getSNMP3AuthKey(),
					':snmp3_privprot'=>$dev->getSNMPObject()->getPrivacyProtocol(),
					':snmp3_privpass'=>$dev->getSNMPObject()->getPrivacyKey(),
					':protocol'=>$dev->getTypeConnection(),
					':grp'=>$dev->getGroup(),
					':description'=>$dev->getDescription() ));
			//suppression des anciens groupes
			$query = $dbconnection->prepare("DELETE FROM esia_device_group_in WHERE group_in_device_id=:id");
			$query = parent::executeStatement($query,array(':id' => $dev->getId()));
			//ajout dans les nouveaux groupes
			$query = $dbconnection->prepare("INSERT INTO esia_device_group_in(group_in_device_id, group_in_device_group_id)VALUES (:id, :group);");
			$grps =  $dev->getGroupList();
			foreach($grps as $grp)
			{
				$query = parent::executeStatement($query,array(':id' => $dev->getId(),':group'=>$grp));
			}
		}
		else
		{
			throw new Exception("Id is NULL");
		}
	}
		
	/**
	 * met à jours les informations des graphiques
	 * @name updateGraph
	 * @param PDO $dbconnection
	 * @param object|integer $node id du device ou un objet device
	 * @param string $url url d'accès au serveur munin/cacti
	 * @param integer $tree si cacti id de la tree au se trouve les graphs
	 * @param integer $leaf si cacti numéros de la variable leaf
	 * @throws EsiaDbException
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function updateGraph($dbconnection, $node,$url,$tree=null,$leaf=null)
	{
		if(@get_class($node)=='Device')
		{
			$node =  $node->getId();
		}
		elseif(@get_class($node)=='RemoteDevice')
		{
			$node =  $node->getId();
		}
		else
		{
			$node =  $node;
		}
		if(!is_numeric($node))
		{
			throw new EsiaDbException('$node is not numeric or not object device');
		}
		if(!empty($tree) && !empty($leaf))
		{
			$query = $dbconnection->prepare("UPDATE esia_device_graph
					SET graph_url=:url, graph_cacti_tree_id=:tree, graph_cacti_leaf_id=:leaf, graph_software='cacti'
					WHERE graph_device_id=:node;");
			$query = parent::executeStatement($query,array(':node'=>$node,
					':url'=>$url,
					':tree'=>$tree,
					':leaf'=>$leaf));
		}
		else
		{
			$query = $dbconnection->prepare("UPDATE esia_device_graph
					SET graph_url=:url, graph_software='munin'
					WHERE graph_device_id=:node;");
			$query = parent::executeStatement($query,array(':node'=>$node,':url'=>$url));
		}
	}

	/**
	 * met à jour un plugin de test
	 * @name UpdateCheckPlugin
	 * @param PDO $dbconnection
	 * @param integer $id id du plugin
	 * @param string $name Nom du plugin
	 * @param string $path Chemin d'accès à l'exécutable
	 * @param string $parameter paramètre par défaut
	 * @param integer $interval Interval de test
	 * @param string $description Description du plugin
	 * @param integer $priority priorité du plugin
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function UpdateCheckPlugin($dbconnection,$id,$name,$path,$parameter,$interval,$description,$priority)
	{
		$query= $dbconnection->prepare("UPDATE esia_plugins SET plugin_name=:name,plugin_path=:path,plugin_type='default',
				plugin_default_parameter=:param, plugin_default_interval_time=:interval, plugin_description=:description,plugin_priority=:priority
				WHERE plugin_id=:id;");
		parent::executeStatement($query,array(':name' =>$name,
				':path'=>$path,
				':param'=>$parameter,
				':interval'=>$interval,
				':description'=>$description,
				':id'=>$id,
				':priority'=>$priority));
	}

	/**
	 * met à jour un plugin de traitement de trap
	 * @name UpdateTrapPlugin
	 * @param PDO $dbconnection
	 * @param integer $id id du plugin
	 * @param string $name Nom du plugin
	 * @param string $path Chemin d'accès à l'exécutable
	 * @param string $parameter paramètre par défaut
	 * @param string $oid oid de la trap SNMP
	 * @param string $description Description du plugin
	 * @param integer $priority priorité du plugin
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function UpdateTrapPlugin($dbconnection,$id,$name,$path,$parameter,$oid,$description,$priority)
	{
		$query= $dbconnection->prepare("UPDATE esia_plugins SET plugin_name=:name,plugin_path=:path,plugin_type='trap',plugin_trap_oid=:oid,
				plugin_default_parameter=:param, plugin_description=:description,plugin_priority=:priority
				WHERE plugin_id=:id");
		parent::executeStatement($query,array(':name' =>$name,
				':path'=>$path,
				':param'=>$parameter,
				':id'=>$id,
				':description'=>$description,
				':oid'=>$oid,
				':priority'=>$priority));
	}
	
	/**
	 * Mets à jour les paramètres d'un service
	 * @name UpdateService
	 * @param PDO $dbconnection
	 * @param integer $id id du service
	 * @param string $name Nom du service
	 * @param integer $plugin id du plugin
	 * @param integer $node id du noeud
	 * @param integer $interval Interval de test
	 * @param string $parameter paramètre du service
	 * @param integer $priority priorité du service
	 * @param integer $alert id du programme a exécuter en cas d'erreur
	 * @param string $alert_param  = paramètre à utiliser
	 * @param bool $unknown si = 1 exécuter le programme d'alerte en cas d'erreur de type inconnu
	 * @param bool $warning si = 1 exécuter le programme d'alerte en cas d'erreur de type warning
	 * @param bool $critical si = 1 exécuter le programme d'alerte en cas d'erreur de type critical
	 * @throws EsiaDbException
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function UpdateService($dbconnection,$id,$name,$plugin,$node,$interval,$parameter,$priority,$alert,$alert_param=null,$unknown=null,$warning=null,$critical=null,$ok=null)
	{
		if(is_numeric($node)&&is_numeric($plugin)&&is_numeric($interval))
		{
			//DEBUT GESA
			$gesa = Model_adm::getGesaLink($dbconnection,$node);
			if($gesa!=null)
			{
				if(gethostbyname($gesa->getName())==$gesa->getName())
				{
					$parameter = preg_replace('#\$NRPE_HOST#',$gesa->getIPv4Adress(),$parameter);
				}
				else
				{	
					$parameter = preg_replace('#\$NRPE_HOST#',$gesa->getName(),$parameter);
				}
			}
			//FIN GESA
			$query= $dbconnection->prepare("UPDATE esia_services
					SET service_plugin_id=:plugin_id, service_interval_time=:interval,
					service_parameter=:parameter, service_name=:name,service_priority=:priority
					WHERE service_id=:id;");
			$query = parent::executeStatement($query,array(':name'=>$name,
					':plugin_id'=>$plugin,
					':id'=>$id,
					':interval'=>$interval,
					':parameter'=>$parameter,
					':priority'=>$priority));
            if($alert !=-1)
            {
			    $query= $dbconnection->prepare("DELETE FROM esia_service_response_program WHERE response_service_id=:id;");
			    $query = parent::executeStatement($query,array(':id'=>$id));
			    //si on a spécifier un programme d'alerte
			    if($ok!=null && $warning!=null && $critical!=null && $unknown!=NULL && (!empty($alert)||$alert!=0) )
			    {
				    $query= $dbconnection->prepare("INSERT INTO esia_service_response_program(
						    response_service_id, response_program_id, response_parameter, response_warning, response_critical, response_unknown,response_ok)
						    VALUES (:service, :program, :parameter,:warning, :critical, :unknown, :ok);");
				    $query = parent::executeStatement($query,array(':service'=>$id,
						    ':program'=>$alert,
						    ':parameter'=>$alert_param,
						    ':warning'=>$warning,
						    ':critical'=>$critical,
						    ':ok'=>$ok,
						    ':unknown'=>$unknown));
			    }
            }
			Model_adm::reloadService($dbconnection,$node,$id);
		//	$query= $dbconnection->prepare("INSERT INTO esia_service_pending_execute(pending_service_id,pending_type)VALUES (:id,'UPDATE');");
		//	$query = parent::executeStatement($query,array(':id'=>$id));

		}
		else
		{
			throw new EsiaDbException('bad parameter: $plugin,$inteval and $node must be numeric');
		}
	}
	/**
	 * @brief reload les services demandé au niveau du démon de traitement
	 * @param PDO $dbconnection  variable de connexion à la DB
	 * @param integer $nodeid identifiant du noeud
	 * @param integer $serviceid identifiant du service (optionnel) si il n'est spécifé alors l'ensemble des services sont rechargés
	 * @throws EsiaDbException si une erreur survient
	 */
	static function reloadService($dbconnection,$nodeid,$serviceid=null)
	{
		try 
		{	
			if(is_numeric($nodeid)&&is_numeric($serviceid))
			{
				
				$query= $dbconnection->prepare("INSERT INTO esia_service_pending_execute(pending_service_id,pending_type)VALUES (:id,'UPDATE');");
				$query = parent::executeStatement($query,array(':id'=>$serviceid));
		
			}
			elseif(is_numeric($nodeid))
			{
				$query= $dbconnection->prepare("INSERT INTO esia_service_pending_execute(
												pending_service_id, pending_type)
												SELECT service_id,'UPDATE' FROM esia_services WHERE service_device_id=:id;");
				$query = parent::executeStatement($query,array(':id'=>$nodeid));						
			}
			else
			{
				throw new EsiaDbException('reload service bad parameter: $nodeid must be numeric');
			}
		}
		catch (EsiaDbException $e)
		{
			if(! preg_match('#pending_primary_key#', $e->getMessage()))
			{
				throw new EsiaDbException($e->getMessage());
			}
		}
	}
	
	/**
	 * Mets à jour le mot de passe de l'utilisateur
	 * @name UpdateUserPassword
	 * @param PDO $dbconnection
	 * @param User $usermod objet User contenant l'utilisateur
	 * @param string $password le nouveau mot de passe
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function UpdateUserPassword($dbconnection, User $usermod,$password)
	{
		$hashpassword = $usermod->generateHashPassword($usermod->getName(),$password,$usermod->getInscriptionTime());
		$query = $dbconnection->prepare('UPDATE esia_users SET user_password=:pass,user_change_password=1 WHERE user_id=:id;');
		$query = parent::executeStatement($query,array(':id'=>$usermod->getId(),
				':pass'=>$hashpassword));
	}
	
	/**
	 * Mets à jour les informations de l'utilisateurs
	 * @name UpdateUser
	 * @param PDO $dbconnection
	 * @param User $usermod objet User contenant l'utilisateur
	 * @param string $password (optionnel) le nouveau mot de passe
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function UpdateUser($dbconnection, User $usermod,$password=null)
	{
		//$hashpassword = $usermod->generateHashPassword($usermod->getName(),$password,$usermod->getInscriptionTime());
		$query = $dbconnection->prepare('UPDATE esia_users
				SET user_change_password=1,
				user_email = :mail,
				user_lang = :lang,
				home_page = :home_page
				WHERE user_id=:id;');
		$query = parent::executeStatement($query,array(':id'=>$usermod->getId(),
				':mail'=>$usermod->getEmail(),
				':home_page'=>$usermod->getHomePage(),
				':lang'=>$usermod->getLang()));
	}

	/**
	 * Mets à jour les droits d'accès d'un groupe, attention les anciens droits sont écrasés
	 * @name UpdateGroupGrantAccess
	 * @param PDO $dbconnection
	 * @param integer $groupid id du group
	 * @param array $access tableau contenant les identifiants des nouveaux droit d'accès
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function UpdateGroupGrantAccess($dbconnection, $groupid,$access)
	{
		$query = $dbconnection->prepare("DELETE FROM esia_privilege_grantedto WHERE grant_group_id=:group_id;");
		$query = parent::executeStatement($query,array(':group_id'=>$groupid));
		$query = $dbconnection->prepare("INSERT INTO esia_privilege_grantedto(grant_group_id, grant_privilege_id, grant_acces) VALUES ( :group_id , :access_id ,1);");
		foreach($access as $selectdroitId)
		{
			if($selectdroitId!=0&&is_numeric($selectdroitId))
			{
				$query = parent::executeStatement($query,array(':group_id'=>$groupid,
						':access_id'=>$selectdroitId,));
			}
		}
	}
	
	/**
	 *  Mets à jour les droits d'accès d'un groupe par rapport au noeud, attention les anciens droits sont écrasés
	 * @name UpdateGroupNodeAccess
	 * @param PDO $dbconnection
	 * @param integer $groupid id du group
	 * @param array $access tableau contenant les identifiants des noeuds où le groupe à le droit d'accès
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function UpdateGroupNodeAccess($dbconnection, $groupid,$access)
	{
		$query = $dbconnection->prepare("DELETE FROM esia_group_device_access WHERE access_group_id=:group_id;");
		$query = parent::executeStatement($query,array(':group_id'=>$groupid));
		$query = $dbconnection->prepare("INSERT INTO esia_group_device_access VALUES ( :group_id , :dev_id);");
		foreach($access as $selectdroitId)
		{
			if($selectdroitId!=0&&is_numeric($selectdroitId))
			{
				$query = parent::executeStatement($query,array(':group_id'=>$groupid,
						':dev_id'=>$selectdroitId,));
			}
		}
	}

	/**
	 * met à jour un groupe de noeud  à l'application
	 * @name updateGroupNode
	 * @param PDO $dbconnection
	 * @param integer $id id du groupe
	 * @param integer $group id du groupe
	 * @param integer $parent groupe parent
	 * @retval identifiant du groupe de noeud ajouté
	 * @exception EsiaDbException si une erreur survient
	 */
	static function updateGroupNode($dbconnection,$id,$group,$parent=null)
	{
		if(isset($parent))
		{
			$query = $dbconnection->prepare('UPDATE esia_device_group
											   SET device_group_name=:name, device_group_parent=:parent
											 WHERE device_group_id=:id;');
			$query = parent::executeStatement($query,array(':id'=>$id,':name'=>$group,':parent'=>$parent));
		}
		else
		{
			$query = $dbconnection->prepare('UPDATE esia_device_group
											   SET device_group_name=:name, device_group_parent=NULL
											 WHERE device_group_id=:id;');
			$query = parent::executeStatement($query,array(':id'=>$id,':name'=>$group));
		}
		return $id;
	}
	
	/**
	 * insére le pattern service dans la base de donnée
	 * @name updatePatternService
	 * @param PDO $dbconnection
	 * @param integer $id identifiant du pattern
	 * @param string $name Nom du paternService
	 * @param string $description description du pattern service
	 * @param array $services tableau contenant les informations de base des plugins du pattern
	 *		$array[]['id'] identifiant du services (included_id dans la table services_included)
	 *		$array[]['name'] nouveau nom du service
	 *		$array[]['plugin'] id du plugin 
	 *		$array[]['parameter] nouveau paramètre
	 *		$array[]['interval'] nouvel interval
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function updatePatternService($dbconnection,$id,$name,$description,$services=null)
	{
		//mise à jour des informations du pattern
		$query = $dbconnection->prepare('UPDATE esia_service_patterns SET 
											pattern_name=:name,
											pattern_description=:description
										WHERE pattern_id=:id;');
		$query = parent::executeStatement($query,array(':id'=>$id,
														':name'=>$name,
														':description'=>$description));
		//on récupère la liste des noeuds / services associé à ce pattern
		$query = $dbconnection->prepare("SELECT service_id,service_device_id,included_id
										  FROM esia_services
										  JOIN esia_service_pattern_included ON service_included_id=included_id
										  WHERE included_pattern_id=:id;");
		$query = parent::executeStatement($query,array(':id'=>$id));
		$arr= array();
		$node_arr= array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$arr[$row->included_id][]=array('service_id'=>$row->service_id,'device_id'=>$row->service_device_id);
			$node_arr[$row->service_device_id]=$row->service_device_id;
		}
		//préparation pour l'update des service inclus au pattern
		$query = $dbconnection->prepare("UPDATE esia_service_pattern_included SET 
											included_name = :name,
											included_parameter=:parameter,
											included_interval=:interval,
											included_program_id=:program, 
  											included_program_parameter=:alertparameter,
  											included_unknown=:unknown,
											included_warning=:warning,
											included_critical=:critical,
											included_ok=:ok,
											included_priority=:priority
										WHERE included_id = :id;");
		$qinsert = $dbconnection->prepare("INSERT INTO esia_service_pattern_included(included_id, included_name, included_pattern_id, included_plugin_id, included_parameter, included_interval,included_priority)
				VALUES (default, :name, :pattern,:plugin,:parameter,:interval,:priority) RETURNING included_id;");
		$qupdate = $dbconnection->prepare("UPDATE esia_services SET service_included_id=:patternid WHERE service_id = :serid");
		
		foreach($services as $ser)
		{
			if(!empty($ser['id']))
			{
				if(!empty($ser['alerting_software']) && $ser['alerting_software']!=0)
				{
					parent::executeStatement($query,array(':id'=>$ser['id'],
																	':name'=>$ser['name'],
																	':parameter'=>$ser['parameter'],
																	':interval'=>$ser['interval'],
																	':program'=>$ser['alerting_software'],
																	':alertparameter'=>	$ser['alerting_parameter'],
																	':unknown'=>$ser['unknown'],
																	':warning'=> $ser['warning'],
																	':critical'=>$ser['critical'],':ok'=>$ser['ok'],
																	':priority'=>$ser['priority'],));
				}
				else
				{
					parent::executeStatement($query,array(':id'=>$ser['id'],
							':name'=>$ser['name'],
							':parameter'=>$ser['parameter'],
							':interval'=>$ser['interval'],
							':program'=>null,
							':alertparameter'=>	null,
							':unknown'=>null,
							':warning'=>null,
							':critical'=>null,
							':ok'=>null,
							':priority'=>$ser['priority'],));
				}
				$list = $arr[$ser['id']];
				if(!empty($list))
				{
					foreach($list as $row)
					{
						try {
							//on update tout les services.
							Model_adm::UpdateService($dbconnection,$row['service_id'],$ser['name'],$ser['plugin'],$row['device_id'],
									$ser['interval'],$ser['parameter'],$ser['priority'],$ser['alerting_software'],$ser['alerting_parameter']
									,$ser['unknown'],$ser['warning'],$ser['critical'],$ser['critical'],$ser['ok']);							
						}
						catch (EsiaDbException $e)
						{

						}
					}
				}
			}
			else
			{
				$rinsert= parent::executeStatement($qinsert,array(':name'=>$ser['name'],
						':pattern'=>$id,
						':plugin'=>$ser['plugin'],
						':parameter'=>$ser['parameter'],
						':interval'=>$ser['interval'],':priority'=>$ser['priority']));
				$row=$rinsert->fetch(PDO::FETCH_OBJ);
				$partternid = $row->included_id;
				foreach($node_arr as $node)
				{
					try {
						$serid =Model_adm::addService($dbconnection, $ser['name'], $ser['plugin'], $node, $ser['interval'], $ser['parameter'],$ser['priority'],$ser['alerting_software'], $ser['alerting_parameter'],
																	 $ser['unknown'],$ser['warning'],$ser['critical'],$ser['ok']);
						parent::executeStatement($qupdate,array(':serid'=>$serid,':patternid'=>$partternid));
					}
					catch (EsiaDbException $e)
					{
					
					}
				}
			}
		}
	}
	
	/**
	 * supprime le noeud de l'application
	 * @name deleteNode
	 * @param PDO $dbconnection
	 * @param string|integer $nodeid id du device ou son nom ou un objet device
	 * @throws Exception
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function deleteNode($dbconnection,$nodeid)
	{
		if(@get_class($nodeid)=='Device')
		{
			$nodeid =  $nodeid->getId();
		}
		elseif(@get_class($nodeid)=='RemoteDevice')
		{
			$nodeid =  $nodeid->getId();
		}
		if(is_numeric($nodeid))
		{
			$query = $dbconnection->prepare("UPDATE esia_device SET device_delete=TRUE WHERE device_id =?;");
		}
		elseif(is_string($nodeid))
		{
		    $query = $dbconnection->prepare("UPDATE esia_device SET device_delete=TRUE WHERE device_name =?;");
		}
		else
		{
			throw new Exception('$nodeid must be numeric or equals device name');
		}
		global $lang;
		model_adm::addGDPRRegister($dbconnection,'DELETE','NODE',$lang['GDPR_DELETE_NODE']. $nodeid ,$lang['GDPR_DELETE_NODE_FINALITY']);
		$query = parent::executeStatement($query,array($nodeid));
	}
	
	/**
	 * supprime le plugin de l'application
	 * @name deletePlugin
	 * @param PDO $dbconnection
	 * @param integer $pluginid id du plugin
	 * @throws EsiaDbException
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function deletePlugin($dbconnection,$pluginid)
	{
		if(is_numeric($pluginid))
		{
			$query= $dbconnection->prepare("UPDATE esia_services SET service_delete=TRUE
					WHERE service_plugin_id=:id");
			$query = parent::executeStatement($query,array($pluginid));
			$query= $dbconnection->prepare("UPDATE esia_plugins SET plugin_delete=TRUE
					WHERE plugin_id=:id");
			$query = parent::executeStatement($query,array($pluginid));
		}
		else
		{
			throw new EsiaDbException('plugin\'s id is not numeric');
		}
	}
	
	/**
	 * supprime le service de l'application
	 * @name deleteService
	 * @param PDO $dbconnection
	 * @param integer $service id du service
	 * @throws EsiaDbException
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function deleteService($dbconnection,$service)
	{
		if(is_numeric($service))
		{
			$query= $dbconnection->prepare("DELETE FROM esia_error_happen
					WHERE happen_service_id=?");
			$query = parent::executeStatement($query,array($service));
			$query= $dbconnection->prepare("UPDATE esia_services SET service_delete=TRUE
					WHERE service_id=?");
			$query = parent::executeStatement($query,array($service));
			global $lang;
			model_adm::addGDPRRegister($dbconnection,'DELETE','SERVICE',$lang['GDPR_DELETE_SERVICE']. $service,$lang['GDPR_DELETE_SERVICE_FINALITY']);
			
		}
		else
		{
			throw new EsiaDbException('service is not numeric');
		}
	}
	/**
	 * supprime l'utilisateur de l'application
	 * @name deleteUser
	 * @param PDO $dbconnection
	 * @param integer $id id de l'utilisateur
	 * @throws EsiaDbException
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function deleteUser($dbconnection,$id)
	{
		$query = $dbconnection->prepare("SELECT member_user_id FROM esia_memberof WHERE member_user_id=?;");
		$query = parent::executeStatement($query,array($id));
		$nb_rows = $query->rowCount();
		//si la requête renvoi un nombre de ligne différent de 0 alors
		//on supprime la liaison de l'utilisateur avec le groupe
		if($nb_rows!=0)
		{
			$query = $dbconnection->prepare("DELETE FROM esia_memberof WHERE member_user_id=?;");
			$query = parent::executeStatement($query,array($id));
		}
		//suppression de l'utilisateur
		$query = $dbconnection->prepare("DELETE FROM esia_users WHERE user_id=?;");
		$query = parent::executeStatement($query,array($id));
		global $lang;
		model_adm::addGDPRRegister($dbconnection,'DELETE','USER',$lang['GDPR_DELETE_USER']. $id,$lang['GDPR_DELETE_USER_FINALITY']);
		
	}
	
	/**
	 * retire l'utilisateur d'un groupe donnée
	 * @name deleteUserFromGroup
	 * @param PDO $dbconnection
	 * @param integer $userid id ou le nom de l'utilisateur
	 * @param integer $groupid id ou le nom du group
	 * @throws EsiaDbException
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function deleteUserFromGroup($dbconnection,$userid,$groupid)
	{
		if(!is_numeric($userid))
		{
			$userid = Model_adm::InstanceUser($dbconnection, $userid);
			$userid = $userid->getId();
		}
		if(!is_numeric($groupid))
		{
			$query = $dbconnection->prepare("SELECT group_id FROM esia_groups WHERE group_name=?;");
			$query = parent::executeStatement($query,array( $groupid ));
			$row=$query->fetch(PDO::FETCH_OBJ);
			$groupid=$row->group_id;
		}
		$query = $dbconnection->prepare("DELETE FROM esia_memberof WHERE member_user_id=? AND member_group_id=?;");
		$query = parent::executeStatement($query,array($userid,$groupid));

	}

	/**
	 * supprime le groupe passé en paramètre
	 * @name deleteGroup
	 * @param PDO $dbconnection
	 * @param integer $id id du groups
	 * @throws EsiaDbException
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function deleteGroup($dbconnection,$id)
	{
		$query = $dbconnection->prepare("DELETE FROM esia_memberof WHERE member_group_id=?;");
		$query = parent::executeStatement($query,array($id));
		$query = $dbconnection->prepare("DELETE FROM esia_group_device_access WHERE access_group_id=?;");
		$query = parent::executeStatement($query,array($id));
		$query = $dbconnection->prepare("DELETE FROM esia_privilege_grantedto WHERE grant_group_id=?");
		$query = parent::executeStatement($query,array($id));
		$query = $dbconnection->prepare("DELETE FROM esia_groups WHERE group_id=?;");
		$query = parent::executeStatement($query,array($id));
	}

	/**
	 * supprime le service du pattern ainsi que tout les service actif qui si rapporte
	 * @name deleteServiceOfPattern
	 * @param PDO $dbconnection
	 * @param integer $id id du service (table included_id)
	 * @throws EsiaDbException
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function deleteServiceOfPattern($dbconnection,$id)
	{
		$query = $dbconnection->prepare("SELECT service_id FROM esia_services WHERE service_included_id=?;");
		$query = parent::executeStatement($query,array($id));
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			Model_adm::deleteService($dbconnection, $row->service_id);
		}
		$query =  $dbconnection->prepare("UPDATE esia_services SET service_included_id=NULL WHERE service_included_id=?;");
		$query = parent::executeStatement($query,array($id));
		$query =  $dbconnection->prepare("DELETE FROM esia_service_pattern_included WHERE included_id=?;");
		$query = parent::executeStatement($query,array($id));

	}
	
	/**
	 * supprime le pattern de service ainsi que tout les services actifs associés
	 * @name deletePatternService
	 * @param PDO $dbconnection
	 * @param integer $id id du pattern de service
	 * @throws EsiaDbException
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function deletePatternService($dbconnection,$id)
	{
		$query = $dbconnection->prepare("SELECT service_id,included_id
										FROM esia_services 
										JOIN esia_service_pattern_included ON included_id = service_included_id
										WHERE included_pattern_id=?;");
		$query = parent::executeStatement($query,array($id));
		$list = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			Model_adm::deleteService($dbconnection, $row->service_id);
			$list[$row->included_id]=$row->included_id;
		}
		$query =  $dbconnection->prepare("UPDATE esia_services SET service_included_id=NULL WHERE service_included_id=?;");
		foreach($list as $inc_id)
		{
			parent::executeStatement($query,array($inc_id));
		}
		$query =  $dbconnection->prepare("DELETE FROM esia_service_pattern_included WHERE included_pattern_id=?;");
		$query = parent::executeStatement($query,array($id));
		$query =  $dbconnection->prepare("DELETE FROM esia_service_patterns WHERE pattern_id=?;");
		$query = parent::executeStatement($query,array($id));	
	}

	/**
	 * supprime le groupe de noeud passé en paramètre et les groups enfant devienne des groupes membres de la racine principal
	 * @name deleteGroupNode
	 * @param PDO $dbconnection
	 * @param integer $id id du groups
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function deleteGroupNode($dbconnection,$id)
	{
		$query = $dbconnection->prepare('UPDATE esia_device_group SET device_group_parent=NULL
											 WHERE device_group_parent=?;');
		$query = parent::executeStatement($query,array($id));
		$query = $dbconnection->prepare("DELETE FROM esia_device_group_in WHERE group_in_device_group_id=?");
		$query = parent::executeStatement($query,array($id));
		$query = $dbconnection->prepare("DELETE FROM esia_device_group WHERE device_group_id=?;");
		$query = parent::executeStatement($query,array($id));
	}

	/**
	 * test et si il y a une gesa qui est liée elle retourne un objet device
	 * @name getGesaLink
	 * @param PDO $dbconnection
	 * @param integer $nodeid identifiant du noeud
	 * @retval NULL|GESA null si aucune gesa n'est lié sinon un objet device
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getGesaLink($dbconnection,$nodeid)
	{
		$query= $dbconnection->prepare("SELECT connected_gesa_id FROM gesa_connected WHERE connected_device_id=?;");
		$query = parent::executeStatement($query,array($nodeid));
		$gesa=null;
		if($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$gesa = new GESA();
			$gesa->constructDevice(Model_esia::InstanceNode($dbconnection,$row->connected_gesa_id));
		}
		return $gesa;
	}

	/**
	 * Lie la GESA au noeud correspondant
	 * @name linkGesaNode
	 * @param PDO $dbconnection
	 * @param integer $gesaid identifiant de la GESA
	 * @param integer $nodeid identifiant du noeud
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function linkGesaNode($dbconnection,$gesaid,$nodeid)
	{
		$query= $dbconnection->prepare("INSERT INTO gesa_connected(connected_gesa_id, connected_device_id) VALUES (?, ?);");
		$query = parent::executeStatement($query,array($gesaid,$nodeid));
	}
	
	/**
	 * Délie la GESA au noeud correspondant
	 * @name unlinkGesaNode
	 * @param PDO $dbconnection
	 * @param integer $gesaid identifiant de la GESA
	 * @param integer $nodeid identifiant du noeud
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function unlinkGesaNode($dbconnection,$gesaid,$nodeid)
	{
		$query= $dbconnection->prepare("DELETE FROM gesa_connected WHERE connected_gesa_id =? AND connected_device_id=?;");
		$query = parent::executeStatement($query,array($gesaid,$nodeid));
	}

	/**
	 * Dissocie un pattern de service du noeud donné supprime pas les services!!!!!!
	 * @name unassignPatternServiceNode
	 * @param PDO $dbconnection
	 * @param integer $patternid identifiant du pattern de service
	 * @param integer $node identifiant du noeud
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function unassignPatternServiceNode($dbconnection,$patternid,$node)
	{
		$query = $dbconnection->prepare("SELECT service_id
											FROM esia_services 
											JOIN esia_service_pattern_included ON service_included_id = included_id
											WHERE included_pattern_id=:id AND service_device_id=:node;");	
		$query = parent::executeStatement($query,array(':id'=>$patternid,':node'=>$node));	
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			Model_adm::deleteService($dbconnection, $row->service_id);
		}
	}
	/**
	 * Délie un pattern de service du noeud donné. Ne supprime pas les services!!!!!!
	 * @name unlinkPatternServiceNode
	 * @param PDO $dbconnection
	 * @param integer $patternid identifiant du pattern de service
	 * @param integer $node identifiant du noeud
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function unlinkPatternServiceNode($dbconnection,$patternid,$node)
	{
		$query = $dbconnection->prepare("UPDATE esia_services SET service_included_id = NULL WHERE service_id IN (SELECT service_id
											FROM esia_services 
											JOIN esia_service_pattern_included ON service_included_id = included_id
											WHERE included_pattern_id=:id AND service_device_id=:node);");	
		$query = parent::executeStatement($query,array(':id'=>$patternid,':node'=>$node));	
	}	
	/**
	 * recherche un utilisateur ou tout les utilisateur
	 * @name searchUser
	 * @param PDO $dbconnection
	 * @param string|integer $username (optionnel) l identifiant ou le nom ou une partie du nom de l'utilisateur (si pas préciser retourne tout les utilisateurs
	 * @retval un tableau associatif
	 *	$arr[<user id>]['id'] id de l'utilisateur
	 *	$arr[<user id>]['name'] nom de l'utilisateur
	 *	$arr[<user id>]['mail'] adresse mail de l'utilisateur
 	 *	$arr[<user id>]['group'] tableau contenant la liste des groupes auquel l'utilisateur appartient
	 * @exception EsiaDbException si une erreur survient
	 */
	static function searchUser($dbconnection,$username=null)
	{
		if(isset($username))
		{
			if(is_numeric($username))
			{
				$query = $dbconnection->prepare("SELECT user_id,user_name,user_email,group_name,group_id
						FROM esia_users
						LEFT JOIN esia_memberof ON user_id = member_user_id
						LEFT JOIN esia_groups ON group_id = member_group_id
						WHERE user_id = ?;");
				$query = parent::executeStatement($query,array($username));
			}
			else
			{
				$query = $dbconnection->prepare("SELECT user_id,user_name,user_email,group_name,group_id
						FROM esia_users
						LEFT JOIN esia_memberof ON user_id = member_user_id
						LEFT JOIN esia_groups ON group_id = member_group_id
						WHERE user_name LIKE ?;");
				$query = parent::executeStatement($query,array('%'.$username.'%'));
			}
		}
		else
		{
			$query = $dbconnection->prepare("SELECT user_id,user_name,user_email,group_name,group_id
					FROM esia_users
					LEFT JOIN esia_memberof ON user_id = member_user_id
					LEFT JOIN esia_groups ON group_id = member_group_id
					WHERE visibility_user = TRUE;");
			$query = parent::executeStatement($query);
		}
		$list_user=array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$list_user[$row->user_id]['id'] = $row->user_id;
			$list_user[$row->user_id]['name']=$row->user_name;
			$list_user[$row->user_id]['mail'] = $row->user_email;
			$list_user[$row->user_id]['group'][]=$row->group_name;
			$list_user[$row->user_id]['group_id'][]=$row->group_id;
		}
		return $list_user;
	}

	/**
	 * retourne les groupes dont l'utilisateur n'est pas membre
	 * @name getUserNotInGroup
	 * @param PDO $dbconnection
	 * @param string|integer $userid (optionnel) l'identifiant ou le nom de l'utilisateur
	 * @retval un tableau associatif
	 *	$arr[<group id>]['id'] id du groupe
	 *	$arr[<group id>]['name'] nom du groupe
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getUserNotInGroup($dbconnection,$userid)
	{
		$query = $dbconnection->prepare("SELECT group_id,group_name
				FROM esia_groups
				EXCEPT
				SELECT group_id,group_name
				FROM esia_groups
				JOIN esia_memberof ON group_id=member_group_id
				WHERE member_user_id=?");
		$query = parent::executeStatement($query,array( $userid ));
		$list_grp=array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$list_grp[$row->group_id]['id'] = $row->group_id;
			$list_grp[$row->group_id]['name']=$row->group_name;
		}
		return $list_grp;
	}

	/**
	 * met à jour la configuration
	 * @name updateConfiguration
	 * @param PDO $dbconnection
	 * @param array $arr tableau associatifs ayant comme structure $list_key[<clé>]=valeur
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function updateConfiguration($dbconnection,$arr)
	{
		$query = $dbconnection->prepare("UPDATE esia_configuration SET config_value=:value WHERE config_key=:key");
		foreach($arr as $key => $value)
		{
			$query = parent::executeStatement($query,array('value'=>$value,'key'=>$key));
		}
	}
	
	/**
	 * indique qu'il faut recharger la file de service
	 * @name mustReloadDaemon
	 * @param PDO $dbconnection
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function mustReloadDaemon($dbconnection)
	{
		$query= $dbconnection->prepare("UPDATE esia_configuration SET config_value=1 WHERE config_key='SYS_RELOAD';");
		$query = parent::executeStatement($query);
	}

	/**
	 * indique qu'il faut redémarrer le programme
	 * @name mustRestartDaemon
	 * @param PDO $dbconnection
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function mustRestartDaemon($dbconnection)
	{
		$query= $dbconnection->prepare("UPDATE esia_configuration SET config_value=2 WHERE config_key='SYS_RELOAD';");
		$query = parent::executeStatement($query);
	}
	/**
	 * @brief met à jour une tuile
	 * @param PDO $dbconnection
	 * @param integer $id  identifiant de la DB (table: dashboard_tile)
	 * @param integer $position la position par rapport au dashboard
	 * @param integer $height la hauteur de la tuile
	 * @param integer $width la largeur de la tuile
	 * @param integer $dashboardid identifiant du dashboard
	 * @exception: une esiaDBException
	 */
	static function updateTileDashboard($dbconnection,$id,$position,$height,$width,$dashboardid)
	{
		$query = $dbconnection->prepare("UPDATE dashboard_tile
										   SET tile_position=:position, tile_height=:height, 
												tile_width=:width, tile_dashboard_id=:dashid
										 WHERE tile_id=:id");
		$query = parent::executeStatement($query,array('id'=>$id,
														'position'=>$position,
														'height'=>$height,
														'width'=>$width,
														'dashid'=>$dashboardid));
	}
	/**
	 * @brief ajoute une nouvelle tuille
	 * @param PDO $dbconnection
	 * @param integer $position la position par rapport au dashboard
	 * @param integer $height la hauteur de la tuile
	 * @param integer $width la largeur de la tuile
	 * @param integer $dashboardid identifiant du dashboard
	 * @retval integer identifiant de la tuile en DB (table: dashboard_tile)
	 * @exception: une esiaDBException
	 */
	static function insertTileDashboard($dbconnection,$position,$height,$width,$dashboardid)
	{
		$query = $dbconnection->prepare("INSERT INTO dashboard_tile(
            tile_id, tile_position, tile_height, tile_width, tile_dashboard_id)
			    VALUES (DEFAULT,:position,:height,:width,:dashid) RETURNING tile_id;");
		$query = parent::executeStatement($query,array(	'position'=>$position,
				'height'=>$height,
				'width'=>$width,
				'dashid'=>$dashboardid));
		$row=$query->fetch(PDO::FETCH_OBJ);
		return $row->tile_id;
	}
	/**
	 * @brief récupère la liste des utilisateur a qui on a partagé le dashboard
	 * @param PDO $dbconnection
	 * @param integer $id identifiant du dashboard
	 * @exception: esiaDBException
	 */
	static function getUserShareDashboard($dbconnection,$id,$write=false)
	{
		if($write==false)
		{
			$query = $dbconnection->prepare("SELECT board_user_id
  											FROM dashboard_user_access
											WHERE board_user_share = true AND board_dashboard_id=?;");
		}
		else
		{
			$query = $dbconnection->prepare("SELECT board_user_id
  											FROM dashboard_user_access
											WHERE board_user_share = false AND board_dashboard_id=?;");
		}
		$query = parent::executeStatement($query,array($id));
		$list=array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$list[]=$row->board_user_id;
		}
		return $list;
	}
	/**
	 * @brief met à jour un dashboard 
	 * @param PDO $dbconnection
	 * @param integer $id identifiant du dashboard
	 * @param integer $name nom du dashboard
	 * @param integer $description  description du dashboard
	 * @exception: une esiaDBException
	 */
	static function updateDashboard($dbconnection,$id,$name,$description,$users=null,$write=null,$userid=null)
	{
		$query = $dbconnection->prepare("UPDATE dashboard
 				  SET dashboard_name=:name, dashboard_description=:description WHERE dashboard_id=:id;");
		$query = parent::executeStatement($query,array(	'id'=>$id,'name'=>$name,'description'=>$description));
	//	$row=$query->fetch(PDO::FETCH_OBJ);
		
		if(isset($write))
		{
			$queryDel = $dbconnection->prepare("DELETE FROM dashboard_user_access WHERE board_dashboard_id = ? AND board_user_id = ? AND board_user_share = true;");
			$query = $dbconnection->prepare("DELETE FROM dashboard_user_access WHERE board_dashboard_id = ? AND board_user_id <> ? AND board_user_share = FALSE;");
			$query = parent::executeStatement($query,array($id,$userid));
			$query = $dbconnection->prepare("INSERT INTO dashboard_user_access(board_user_id, board_dashboard_id, board_user_share) VALUES (?,?,FALSE);");
			foreach($write as $w)
			{
			    try {
			        $queryDel = parent::executeStatement($queryDel,array($id,$w));
			        $query = parent::executeStatement($query,array($w,$id));
			    } catch (Exception $e) {
			    }
			}
		}
		else
		{
			$query = $dbconnection->prepare("DELETE FROM dashboard_user_access WHERE board_dashboard_id = ? AND board_user_id <> ? AND board_user_share = FALSE;");
			$query = parent::executeStatement($query,array($id,$userid));
		}
		
		if(isset($users))
		{
			$query = $dbconnection->prepare("DELETE FROM dashboard_user_access WHERE board_dashboard_id = ? AND board_user_share = true;");
			$query = parent::executeStatement($query,array($id));
			$query = $dbconnection->prepare("INSERT INTO dashboard_user_access(board_user_id, board_dashboard_id, board_user_share) VALUES (?,?,TRUE);");
			foreach($users as $user)
			{
				$query = parent::executeStatement($query,array($user,$id));
			}						
		}
		else
		{
			$query = $dbconnection->prepare("DELETE FROM dashboard_user_access WHERE board_dashboard_id = ? AND board_user_share = true;");
			$query = parent::executeStatement($query,array($id));
		}
		//return $row->tile_id;
	}
	/**
	 * @brief ajoute un nouveau dashboard
	 * @param PDO $dbconnection
	 * @param integer $id identifiant du dashboard
	 * @param integer $nom du dashboard
	 * @param integer $description description
	 * @exception: une esiaDBException
     * @retval integer identifiant du dashboard en DB (table: dashboard)
	 */
	static function insertDashboard($dbconnection,$name,$description,$controller,$userid)
	{
		$query = $dbconnection->prepare("INSERT INTO dashboard(
            dashboard_id, dashboard_name, dashboard_controller, dashboard_description)
    								VALUES (DEFAULT,:name,:controller,:description)  RETURNING dashboard_id;");
		$query = parent::executeStatement($query,array('name'=>$name,'description'=>$description,'controller'=>$controller));
		$row=$query->fetch(PDO::FETCH_OBJ);
		
		$query = $dbconnection->prepare("INSERT INTO dashboard_user_access(board_user_id, board_dashboard_id, board_user_share)
										 VALUES (:userid, :dashid, FALSE);");
		$query = parent::executeStatement($query,array('userid'=>$userid,'dashid'=>$row->dashboard_id));

		return $row->dashboard_id;
	}
	/**
	 * @brief supprime un dashboard
	 * @param PDO $dbconnection
	 * @param integer $id identifiant du dashboard
	 * @exception: une esiaDBException
	 * @retval /
	 */
	static function deleteDashboard($dbconnection,$id)
	{
		$query = $dbconnection->prepare("DELETE FROM dashboard_widget WHERE widget_tile_id IN (SELECT tile_id FROM dashboard_tile WHERE tile_dashboard_id=?);");
		$query = parent::executeStatement($query,array($id));
		$query = $dbconnection->prepare("DELETE FROM dashboard_tile WHERE tile_dashboard_id = ?;");
		$query = parent::executeStatement($query,array($id));
		$query = $dbconnection->prepare("DELETE FROM dashboard_user_access WHERE board_dashboard_id = ?;");
		$query = parent::executeStatement($query,array($id));
		$query = $dbconnection->prepare("DELETE FROM dashboard WHERE dashboard_id = ?;");
		$query = parent::executeStatement($query,array($id));
	}
	/**
	 * @brief supprime une tuile du dashboard
	 * @param PDO $dbconnection
	 * @param integer $id identifiant de la tuile
	 * @exception: une esiaDBException
	 * @retval /
	 */
	static function deleteTileDashboard($dbconnection,$id)
	{
		$query = $dbconnection->prepare("DELETE FROM dashboard_widget WHERE widget_tile_id =?;");
		$query = parent::executeStatement($query,array($id));
		$query = $dbconnection->prepare("DELETE FROM dashboard_tile WHERE tile_id = ?;");
		$query = parent::executeStatement($query,array($id));
	}
	/**
	 * @brief supprime une widget
	 * @param PDO $dbconnection
	 * @param integer $id identifiant de la widget
	 * @exception: une esiaDBException
	 * @retval /
	 */
	static function deleteWidgetDashboard($dbconnection,$id)
	{
		$query = $dbconnection->prepare("DELETE FROM dashboard_widget WHERE widget_id =?;");
		$query = parent::executeStatement($query,array($id));
	}
	/**
	 * @brief ajoute une nouvelle tuille
	 * @param PDO $dbconnection
	 * @param integer $tileid identifiant de la tuile
	 * @param integer $viewid identifiant de la vue
	 * @retval integer identifiant de la widget en DB (table: dashboard_tile)
	 * @exception: une esiaDBException
	 */
	static function insertWidget($dbconnection,$tileid,$viewid,$name='new widget')
	{
		$query = $dbconnection->prepare("INSERT INTO dashboard_widget( widget_id, widget_name, widget_view_during,widget_option, widget_tile_id, widget_view_id)
   				 VALUES (DEFAULT,:name,
						(SELECT view_during FROM dashboard_view WHERE view_id=:viewid),
						(SELECT view_default_option FROM dashboard_view WHERE view_id=:viewid),:tileid,:viewid) RETURNING widget_id;");
		$query = parent::executeStatement($query,array(	'viewid'=>$viewid,'tileid'=>$tileid,'name'=>$name ));
		$row=$query->fetch(PDO::FETCH_OBJ);
		return $row->widget_id;
	}
	/**
	 * @brief ajoute une nouvelle tuille
	 * @param PDO $dbconnection
	 * @param integer $tileid identifiant de la tuile
	 * @param integer $viewid identifiant de la vue
	 * @retval integer identifiant de la widget en DB (table: dashboard_tile)
	 * @exception une esiaDBException
	 */
	static function updateWidgetOrder($dbconnection,$array)
	{
		$query = $dbconnection->prepare("UPDATE dashboard_widget SET widget_position=:position WHERE widget_id=:id;");
		foreach($array as $line)
		{
			$query = parent::executeStatement($query,array(	'id'=>$line['id'],'position'=>$line['position']));
		}
	}
    static function addServiceMaintenancePlan($dbconnection,$serviceid,$type, $start,$end,$during)
    {
        $query = $dbconnection->prepare("INSERT INTO esia_maintenance_planned(planned_id, planned_type, planned_start, planned_end, planned_during,planned_service_id)
                                        VALUES (DEFAULT,:type,to_timestamp(:start,'DD-MM-YYYY HH24:MI'),to_timestamp(:end,'DD-MM-YYYY HH24:MI'),:during,:service);");
        if(is_array($serviceid))
        {
            foreach($serviceid as $id)
            {
                if(is_numeric($id))
                {
                    $query = parent::executeStatement($query,array(	'type'=>$type,'service'=>$id,'start'=>$start->format('d-m-Y H:i'),'end'=>$end->format('d-m-Y H:i'),'during'=>$during ));
                }
            }

        }
        elseif(is_numeric($serviceid))
        {
            $query = parent::executeStatement($query,array(	'type'=>$type,'service'=>$serviceid,'start'=>$start->format('d-m-Y H:i'),'end'=>$end->format('d-m-Y H:i'),'during'=>$during ));
        }
        else
        {
            throw new EsiaDbException('$serviceid must be numeric or array');
        }
    }
    static function deleteServiceMaintenancePlan($dbconnection,$plannedid)
    {
        if(is_numeric($plannedid))
        {
            $query = $dbconnection->prepare("DELETE FROM esia_maintenance_planned WHERE planned_id =?");
            $query = parent::executeStatement($query,array($plannedid));
        }
        else
        {
             throw new EsiaDbException('$plannedid must be numeric');
        }
    }
    static function getNextMaintenancePlan($dbconnection,$serviceid=null,$node=null )
    {
        $sql ='SELECT planned_id, planned_type, planned_start, planned_end, planned_during,planned_service_id ,service_name,plugin_name,device_id,device_name
               FROM esia_maintenance_planned 
               JOIN esia_services ON service_id = planned_service_id 
               JOIN esia_plugins ON plugin_id = service_plugin_id 
               JOIN esia_device ON device_id = service_device_id ';
        if(is_numeric($serviceid))
        {
            $query = $dbconnection->prepare($sql."WHERE planned_service_id=? AND planned_start > NOW();");
            $query = parent::executeStatement($query,array($serviceid));
        }
        elseif(is_numeric($node))
        {
            $query = $dbconnection->prepare($sql."WHERE service_device_id=?  AND planned_start > NOW();");
                        $query = parent::executeStatement($query,array($node));
        }
        elseif(is_array($node))
        {
		    foreach($node as $id)
			{
				if(!is_numeric($id))
					throw new EsiaDbException('$node array must be numeric');
			}
            $arrids = implode(',', $node);
            $query = $dbconnection->prepare($sql."WHERE service_device_id IN ($arrids)   AND planned_start > NOW();");
                        $query = parent::executeStatement($query);
        }
        else
        {
            throw new EsiaDbException('$service or $node must be numeric');
        }    

        $arr = array();
  		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
            $arr[] = array('id'=>$row->planned_id,'type'=>$row->planned_type, 'start'=>$row->planned_start,'end'=>$row->planned_end,'during'=>$row->planned_during,'service'=>$row->planned_service_id, 
                    'service_name'=>$row->service_name,
                    'plugin_name'=>$row->plugin_name,
                    'device_name'=>$row->device_name,
                    'device_id'=>$row->device_id );
        }
        return $arr;
    }
    
    static function addCyclicMaintenance($dbconnection,$serviceid,$type, $start,$during,$interval,$endfirsterror = null )
    {
    	$sql ="INSERT INTO esia_maintenance_configuration( maintenance_conf_id, maintenance_conf_type, maintenance_conf_start_date,
    			maintenance_conf_create, maintenance_conf_service_id, maintenance_conf_during, maintenance_conf_recurring,maintenance_conf_during_firsterror )
    			VALUES (DEFAULT,:type,to_timestamp(:start,'DD-MM-YYYY HH24:MI'),NOW(), :service ,cast(:during as interval),cast(:interval as interval),cast(:firsterrorduring as interval) );";
    	$query = $dbconnection->prepare($sql);
    	if(is_array($serviceid))
    	{
    		foreach($serviceid as $id)
    		{
    			if(is_numeric($id))
    			{
    				$query = parent::executeStatement($query,array(	'type'=>$type,'service'=>$id,'start'=>$start->format('d-m-Y H:i'),'during'=>$during,':interval'=>$interval,':firsterrorduring'=>$endfirsterror ));
    			}
    		}
    		
    	}
    	elseif(is_numeric($serviceid))
    	{
    		$query = parent::executeStatement($query,array(	'type'=>$type,'service'=>$id,'start'=>$start->format('d-m-Y H:i'),'during'=>$during,':interval'=>$interval,':firsterrorduring'=>$endfirsterror ));
    	}
    	else
    	{
    		throw new EsiaDbException('$serviceid must be numeric or array');
    	}
    }        
    
    static function getCyclicMaintenancePlan($dbconnection,$serviceid=null,$node=null )
    {
        $sql ='SELECT maintenance_conf_id, maintenance_conf_type, maintenance_conf_start_date,maintenance_conf_create, maintenance_conf_service_id, 
                    maintenance_conf_during, maintenance_conf_recurring,service_name,plugin_name,device_id,device_name
               FROM esia_maintenance_configuration
               JOIN esia_services ON service_id = maintenance_conf_service_id 
               JOIN esia_plugins ON plugin_id = service_plugin_id
               JOIN esia_device ON device_id = service_device_id ';
        if(is_numeric($serviceid))
        {
            $query = $dbconnection->prepare($sql."WHERE planned_service_id=?");
            $query = parent::executeStatement($query,array($serviceid));
        }
        elseif(is_numeric($node))
        {
            $query = $dbconnection->prepare($sql."WHERE service_device_id=?");
                        $query = parent::executeStatement($query,array($node));
        }
        elseif(is_array($node))
        {
		    foreach($node as $id)
			{
				if(!is_numeric($id))
					throw new EsiaDbException('$node array must be numeric');
			}
            $arrids = implode(',', $node);
            $query = $dbconnection->prepare($sql."WHERE service_device_id IN ($arrids);");
                        $query = parent::executeStatement($query);
        }
        else
        {
            throw new EsiaDbException('$service or $node must be numeric');
        }    
        $arr = array();
  		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
            $arr[] = array('id'=>$row->maintenance_conf_id,'type'=>$row->maintenance_conf_type, 
                    'start'=>$row->maintenance_conf_start_date,'create'=>$row->maintenance_conf_create,'during'=>$row->maintenance_conf_during,'service'=>$row->maintenance_conf_service_id, 
                    'interval'=>$row->maintenance_conf_recurring,
                    'service_name'=>$row->service_name,
                    'plugin_name'=>$row->plugin_name,
                    'device_name'=>$row->device_name,
                    'device_id'=>$row->device_id  );
        }
        return $arr;
    }
    static function deleteCyclicMaintenance($dbconnection,$confid)
    {
        if(is_numeric($confid))
        {
            $query = $dbconnection->prepare("DELETE FROM esia_maintenance_configuration WHERE maintenance_conf_id=?");
            $query = parent::executeStatement($query,array($confid));
        }
        else
        {
            throw new EsiaDbException('$confid must be numeric');
        } 
      
    }
    static function passNextMaintenance($dbconnection,$confid)
    {
        if(is_numeric($confid))
        {
            $query = $dbconnection->prepare("UPDATE esia_maintenance_configuration SET maintenance_conf_start_date=maintenance_conf_start_date +maintenance_conf_recurring WHERE maintenance_conf_id=?");
            $query = parent::executeStatement($query,array($confid));
        }
        else
        {
            throw new EsiaDbException('$confid must be numeric');
        } 
      
    }
    static function addAdvanceNodeConfig($dbconnection,$nodeid,$config_type_id,$param)
    {
        $query = $dbconnection->prepare("INSERT INTO esia_device_advance_config(adv_id, adv_device_id, adv_config_type_id, adv_param)
                VALUES (DEFAULT,:nodeid,:configid,:param) RETURNING adv_id;");
        $query = parent::executeStatement($query,array(':nodeid'=>$nodeid,':configid'=>$config_type_id,':param'=>json_encode($param)));
        $row=$query->fetch(PDO::FETCH_OBJ);
        return $row->adv_id;
    }
    static function deleteAdvanceNodeConfig($dbconnection,$config_type_id)
    {
        $query = $dbconnection->prepare("DELETE FROM esia_device_advance_config WHERE adv_id = ?");
        $query = parent::executeStatement($query,array($config_type_id));
    }
    static function getAdvanceNodeConfig($dbconnection,$nodeid=null,$configid=null,$config_type_id=null)
    {
        $sql = "SELECT adv_id, adv_device_id, adv_config_type_id, adv_param, adv_authfile_crypt,adv_plugin_keyword_value, device_config_type_name,device_config_type_controller,web_plugin_apache_path,web_id
                FROM esia_device_advance_config
                JOIN esia_device_advance_config_type ON adv_config_type_id = device_config_type_id
                LEFT JOIN esia_web_plugin ON device_config_type_webplugin_id = web_id";
        $params = array();
        if(is_numeric($nodeid))
        {
            $sql .=" WHERE adv_device_id = :id ";
            $params[':id']=$nodeid;
        }
        elseif(is_numeric($configid))
        {
            $sql .=" WHERE adv_id = :id ";
            $params[':id']=$configid;
        }
        elseif(is_numeric($config_type_id))
        {
            $sql .=" WHERE adv_config_type_id = :id ";
            $params[':id']=$configid;
        }
        $query = $dbconnection->prepare($sql);
        $query = parent::executeStatement($query,$params);
        $arr = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
            $arr[]= array(  'id'=> $row->adv_id,
                            'node_id'=> $row->adv_device_id,
                            'name'=> $row->device_config_type_name,
                            'controller'=> $row->device_config_type_controller,
                            'parameter'=> json_decode($row->adv_param,TRUE),
                            'config_type_id'=> $row->adv_config_type_id,
                            'web_plugin_id'=> $row->web_id,
                            'web_plugin_path'=> $row->web_plugin_apache_path,
                        );
        }
        return $arr;
    }
    
    /**
     * @brief updatePlan : update un plan
     * @param PDO $dbconnection  l'objet de connexion PDO
     * @param integer $id identifiant du plan à modifier
     * @param string $name le nom du plan
     * @param integer $type l'identifiant du type de plan
     * @param string $params paramètres du plan au format JSON
     */
    static function updateAdvanceNodeConfig($dbconnection,$id,$params)
    {
    	$query= $dbconnection->prepare("UPDATE esia_device_advance_config SET adv_param=:params
										WHERE adv_id = :id
										RETURNING adv_id;");
    
    	$query = parent::executeStatement($query,array(':id' =>$id,':params'=>$params));
    	$row=$query->fetch(PDO::FETCH_OBJ);
    	return $row->adv_id;
    }
    
    static function getAdvanceNodeConfigType($dbconnection,$id=null,$webplugin=null)
    {
        $sql = "SELECT device_config_type_id, device_config_type_name, device_config_type_controller,device_config_type_default_parameter::text, web_id, web_plugin_apache_path,web_activate
                FROM esia_device_advance_config_type
                LEFT JOIN esia_web_plugin ON device_config_type_webplugin_id = web_id";
        $params = array();
        if(is_numeric($id))
        {
            $sql .=" WHERE device_config_type_id = :id ";
            $params[':id']=$id;
        }
        elseif(is_string($id))
        {
            $sql .=" WHERE device_config_type_name = :name ";
            $params[':name']=$id;
        } 
        elseif(is_numeric($webplugin))
        {
            $sql .=" WHERE web_id = :id ";
            $params[':id']=$webplugin;
        }
        elseif(is_string($webplugin))
        {
            $sql .=" WHERE web_name = :name ";
            $params[':name']=$webplugin;
        }   
        $query = $dbconnection->prepare($sql);
        $query = parent::executeStatement($query,$params);
        $arr = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
            $arr[]= array(  'id'=> $row->device_config_type_id,
                            'name'=> $row->device_config_type_name,
                            'controller'=> $row->device_config_type_controller,
                            'parameter'=> json_decode($row->device_config_type_default_parameter,TRUE),
                            'web_plugin_id'=> $row->web_id,
                            'web_plugin_path'=> $row->web_plugin_apache_path,
                            'web_plugin_activate'=> $row->web_activate
                        );
        }
        return $arr;
    }
    static function updateServiceProgramAlert($dbconnection,$serviceid,$progid=null,$param=null,$ok=null,$warning=null,$critical=null,$unknown=null)
    {
    	if(is_array($serviceid))
    	{
    		$arrids = array();
    		//si on charge un tableau associatif complets
    		foreach($serviceid as $line)
    		{
    			if(!is_numeric($line['serviceid']))
    				throw new EsiaDbException('$node array must be numeric');
    			else 
    				$arrids[]=$line['serviceid'];
    		}
    		$arrids = implode(',',$arrids);
    		$query = $dbconnection->prepare("DELETE FROM esia_service_response_program WHERE response_service_id IN ($arrids);");
    		$query = parent::executeStatement($query);
    		$query = $dbconnection->prepare("INSERT INTO esia_service_response_program (response_service_id, response_program_id, response_parameter, 
            												response_ok, response_warning, response_critical, response_unknown)
    										VALUES (:serviceid,:program,:parameter,:ok,:warning,:critical,:unknown);");
    		foreach($serviceid as $line)
    		{
    			if($line['progid'] != 0)
    			{
	    			$query = parent::executeStatement($query,array(':serviceid'=>$line['serviceid'],
	    					':program'=>$line['progid'],
	    					':parameter'=>$line['parameter'],
	    					':warning'=>$line['warning'],
	    					':critical'=>$line['critical'],
	    					':ok'=>$line['ok'],
	    					':unknown'=>$line['unknown']));
					Model_adm::reloadService($dbconnection,0,$line['serviceid']);
    			}
    		}
    	}
    	else 
    	{
    		$query = $dbconnection->prepare("DELETE FROM esia_service_response_program WHERE response_service_id =?);");
    		$query = parent::executeStatement($query,array($serviceid));
    		$query = $dbconnection->prepare("INSERT INTO esia_service_response_program(response_service_id, response_program_id, response_parameter,
            												response_ok, response_warning, response_critical, response_unknown)
    										VALUES (:serviceid,:program,:parameter,:ok,:warning,:critical,:unknown);");
    		$query = parent::executeStatement($query,array(':serviceid'=>$serviceid,
    				':program'=>$progid,
    				':parameter'=>$param,
    				':warning'=>$warning,
    				':critical'=>$critical,
    				':ok'=>$ok,
    				':unknown'=>$unknown));
    		Model_adm::reloadService($dbconnection,0,$serviceid);
    		
    	}
    }
    static function getGDPRRegister($dbconnection)
    {
    	$query = $dbconnection->prepare("SELECT register_id,register_user_name,to_char(register_date,'DD/MM/YYYY HH24:MI:SS') as register_date,register_action,register_type,register_data,register_finality FROM gdpr_register;");
    	$query = parent::executeStatement($query,array());
    	$arr = array();
    	while($row=$query->fetch(PDO::FETCH_OBJ))
    	{
    		$arr[] = array('id'=>$row->register_id,
    				'user'=>$row->register_user_name,
    				'date'=>$row->register_date,
    				'action'=>$row->register_action,
    				'type'=>$row->register_type,
    				'data'=>$row->register_data,
    				'finality'=>$row->register_finality);
    	}
    	return $arr;
    }
    static function addGDPRRegister($dbconnection,$action,$type,$data,$finality)
    {
    	$query = $dbconnection->prepare("INSERT INTO gdpr_register (register_id,register_user_name,register_date,register_action,register_type,register_data,register_finality) VALUES (DEFAULT,?,NOW(),?,?,?,?);");
    	global $user;
    	$query = parent::executeStatement($query,array($user->getName(),$action,$type,$data,$finality ));
    	
    }
}
?>
