<?php 
/**
 * @name esia_mobile
 * @description page qui permet aux applications mobiles de se connecter pour récupérer des informations
 * @author George Julien <j.george@esia-sa.com>
 * @contributor
 * @copyright Nicolas Biersart 2012
 * @license
 * @esia 1.00
 * @version 0.1.1
 * @date 08/12/2014
 */


/*@NAME: digestConnect
 *@IN: $username login de l'utilisateur
 * 	$digest du mot de passe de l'utilisateur
 *@OUT: /
 *@TREAT: Vérifie si l'utilisateur existe et si il est authentifié
 *@EXCEPTION: 	UserInvalidePasswordException, si l'utilisateur n'existe pas ou si le mot de passe est invalide
 */
function digestConnect($username,$digest,$dbconnection)
{
  $_SESSION['session_id']['time']=time();
  $u =  Model_esia::InstanceUser($dbconnection,$username);
  if ($digest==$u->getHashPassword())
    {
      return $u;
    }
  else
    {
      throw new UserInvalidePasswordException("Invalide login/password");
    }
}


/**
 * @name Cell
 * @description Classe stockant les paramètre d'une cellule, génére l'élément XML "Case" via sa méthode getXml permettant la création d'un tableau sur l'application mobile
 * @author George Julien <j.george@esia-sa.com>
 * @contributor
 * @copyright Nicolas Biersart 2012
 * @license
 * @esia 1.00
 * @version 1.0.0
 * @date 08/12/2014
 */
class Cell{
  private $text="";
  private $color=null;
  private $text_color=null;
  private $link=null;
  private $font=null;
  private $excludeSort=null;
	
  /*@NAME: Cell
   *@IN: 	$text 		: Le texte qui sera affiché dans la case, "" pour une case vide. Si le texte commence par un @, l'application mobile prendra le texte comme une Ressource de type String défini dans dans son strings.xml
   *	$color		: (optionnel) La couleur d'arrière plan du texte. Si null, couleur par défaut. L'application mobile utilise Color.parseColor(String ). Le format peut donc être #RRGGBB, #AARRGGBB, 'red', 'blue', ...
   *	$text_color	: (optionnel) La couleur du texte lui-même Si null, couleur par défaut. L'application mobile utilise Color.parseColor(String ). Le format peut donc être #RRGGBB, #AARRGGBB, 'red', 'blue', ...
   *	$link		: (optionnel) Si défini, un click sur la case lancera le navigateur de l'application mobile avec l'url correspondant à '$url_serveur.$link'. $link commence donc par / 
   *	$font		: (optionnel) Si défini, doit contenir "bold', 'bold_italic' ou 'italic'
   *	$excludeSort	: (optionnel) Si défini, avec 'y' comme convention mais fonctionne avec toute valeurs, la ligne contenant cette case ne sera jamais triée.
   *@OUT: /
   *@TREAT: Constructeur par défaut de la classe User.*/
  public function Cell($text,$color=null,$text_color=null,$link=null,$font=null,$excludeSort=null){
    $this->text=$text;					
    $this->color=$color;
    $this->text_color=$text_color;
    $this->link=$link;
    $this->font=$font;
    $this->excludeSort=$excludeSort;	
  }
	
  /*@NAME: getXml
   *@IN: $xml: Objet de type DOMDocument	
   *@OUT: Objet de type Element XML
   *@TREAT: Génére un élément XML de type CASE avec ses paramètres d'affichages. Ne contient pas de paramètres de localisation.
   *@EXCEPTION: /
   */
  public function getXml($xml){
    $elm=$xml->createElement('case',$this->text);
    if(isset($this->color))$elm->setAttribute('co',$this->color);
    if(isset($this->text_color))$elm->setAttribute('tc',$this->text_color);
    if(isset($this->link))$elm->setAttribute('ln',$this->link);
    if(isset($this->font))$elm->setAttribute('fn',$this->font);
    if(isset($this->excludeSort))$elm->setAttribute('es',$this->excludeSort);
    return $elm;
  }
}

/**
 * @name Line
 * @description Classe contenant un ensemble d'objets de type Cell contenant donc l'ensemble d'une case d'une ligne.
 * @author George Julien <j.george@esia-sa.com>
 * @contributor
 * @copyright Nicolas Biersart 2012
 * @license
 * @esia 1.00
 * @version 1.0.0
 * @date 08/12/2014
 */
class Line{
  private $cells;

  public function Line(){}

  /*@NAME: add
   *@IN: 	$cell	: Objet de type Cell qui coorespond à une case du tableau
   *	$column : (Optionnel) permet de spécifier la position de la cellule ajoutée à cette ligne. Par défaut ajoute à la première place libre.	
   *@OUT: /
   *@TREAT: Ajoute une case à cette ligne.
   *@EXCEPTION: /
   */
  public function add($cell,$column=null){
    if(isset($column))$this->cells[$column]=$cell;
    else $this->cells[count($this->cells)]=$cell;
  } 


  /*@NAME: makeXml
   *@IN: 	$xml: Objet de type DOMDocument	
   *	$root: Elément parent de tous les éléments du document xml généré.
   *@OUT: /
   *@TREAT: Ajoute une ligne entière au document XML et ajoute les paramètres de localisation aux éléments CASE de la ligne.
   *@EXCEPTION: /
   */
  public function makeXml($root,$xml,$id=0){
    foreach($this->cells as $key => $cell){
      $elm=$cell->getXml($xml);
      $elm->setAttribute('cl',$key);
      $elm->setAttribute('li',$id);
      $root->appendChild($elm);
    }
  }
}

/**
 * @name Table
 * @description Classe contenant un ensemble d'objets Line et permettant d'ajouter un tableau complet à un document XML
 * @author George Julien <j.george@esia-sa.com>
 * @contributor
 * @copyright Nicolas Biersart 2012
 * @license
 * @esia 1.00
 * @version 1.0.0
 * @date 08/12/2014
 */
class Table{
  private $lines;
	
  public function Table(){}

  /*@NAME: add
   *@IN: 	$line	: Objet de type Line qui correspond à une ligne du tableau
   *	$pos : (Optionnel) permet de spécifier la position de la ligne ajoutée à ce tableau. Par défaut ajoute à la première place libre.	
   *@OUT: /
   *@TREAT: Ajoute une ligne entière au tableau
   *@EXCEPTION: /
   */
  public function addLine($line,$pos=null){ 
    if(isset($pos))$this->lines[$pos]=$line;
    else $this->lines[count($this->lines)]=$line;
  }

  public function addCell($cell){
    if(empty($this->lines)) $this->newLine();
    $this->lines[count($this->lines)-1]->add($cell);
  }

  public function newLine(){
    $this->addLine(new Line());
  }

  /*@NAME: makeXml
   *@IN: 	$xml: Objet de type DOMDocument	
   *	$root: Elément parent de tous les éléments du document xml généré.
   *@OUT: /
   *@TREAT: Génére, grâce à sa variable qui est un tableau de lignes, un tableau au format XML et l'ajoute dans le DOMDocument 
   *@EXCEPTION: /
   */
  public function makeXml($root,$xml){
    foreach($this->lines as $key => $line){
      $line->makeXml($root,$xml,$key);
    }
  }
}

/*@NAME: getRequetsParam
 *@IN: 	$name: Nom du paramètre HTTP
 *@OUT:  Contenu de la variable
 *@TREAT: Récupére un paramètre HTTP sans se préocupper de savoir si elle est envoyée via un requête GET ou un POST 
 *@EXCEPTION: /
 */
function getRequetsParam($name){
  if(isset($_POST[$name])) return $_POST[$name];
  if(isset($_GET[$name])) return $_GET[$name];
  return;
}

$path = rtrim ($_SERVER['DOCUMENT_ROOT'],'/');
include $path.'/esia.conf.php';
include_once USR_CLASS;
include_once LIB_LOG;


//Login de l'utilisateur, obligatoire
$p_login=getRequetsParam('login');
 
//mot de passe haché sous la forme de sha1($login.$password.$user_inscription_date_time). Si null, la réponse sera : <CODE>$HANDSHAKE</CODE><MESSAGE>$user_inscription_date_time</MESSAGE>	
$p_pass=getRequetsParam('pass'); 	

//type de requête : 
//'test'=>La réponse ne contiendra que le CODE $OK sans tableau ou alors un CODE d'erreur suivi de son MESSAGE. Utilisé par EsiaDroid pour contrôler si la connection est fonctionnelle (Hôte accessible et paramètres authentification OK)
//'widget'=>Envoi les données de la widget
//'activity'=>Envoi le tableau avec les données de la page d'acceuil de l'application EsiaDroid. 
//'board'=>Envoi le tableau avec les données du tableau de board de l'application EsiaDroid. 
$p_type=getRequetsParam('type');	




$HANDSHAKE=-1;  	//Réponse OK quand le paramètre 'pass' de la réponse HTTP est vide.
$OK=0;			//Réponse OK à toute requête contenant un paramètre 'pass' et un paramètre 'type'
$ERROR_PROTOCOL=1;	//Erreur dans les paramètre de la requête.
$ERROR_DB=2;		//Erreur de type SQL
$ERROR_USER=3;		//Erreur d'authentificatin

header('Content-Type: text/xml; charset=utf-8');
$xml=new DOMDocument('1.0', 'utf-8');
$root=$xml->createElement('response');


if(isset($p_login)&&isset($p_pass)&&isset($p_type)) //Requêtes nécessitant authentification
  {
    try
      {
	//Connection à la DB et authentification de l'utilisateur
	$dbconnection = new PDO('pgsql:host='.DB_HOST.';port='.DB_PORT.';dbname='.DB_NAME, DB_USERNAME, DB_PWD);
	$dbconnection->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);
	
	$user = digestConnect($p_login,$p_pass,$dbconnection); 
	$user->generateSessionID();
		
	$esialog = new EsiaLog(FILE_LOG,$user->getName());
	//		$esialog->write('ESIA', 'mobile connection successfull');		
		
	
		
	if($p_type=='widget'){ //////////////////Requête de type widget////////////////////
				
	  //Calcul du nombre de service "actifs" (tableau en haut à droite)
	  $arr_error = array();
	  $arr_error['OK']=array('nbr'=>0,'pourcent'=>0);
	  $arr_error["warning"]= array('nbr'=>0,'pourcent'=>0);
	  $arr_error["critical"]= array('nbr'=>0,'pourcent'=>0);
	  $arr_error["unknown"]= array('nbr'=>0,'pourcent'=>0);
		
	  $li=Model_esia::countServiceAllState($dbconnection,$user->getId());
	  $totser = 0;
	  foreach($li as $k => $v)
	    {
	      $totser= $totser + $v;
	    }
	  //si il n'y a aucun service qui tourne évite une erreur 'division par 0'
	  if($totser!=0)
	    {
	      foreach($li as $k => $v)
		{
		  $pourcent = round((($v/$totser)*100));				
		  $arr_error[$k]=array('nbr'=>$v,'pourcent'=>$pourcent);
		}
	    }	

	  $elm  = $xml->createElement('code',$OK);
	  $root->appendChild($elm);

	  $table=new Table();
	  $table->addCell(new Cell($arr_error['OK']['nbr']));	
	  $table->addCell(new Cell($arr_error['warning']['nbr']));
	  $table->addCell(new Cell($arr_error['critical']['nbr']));	
	  $table->addCell(new Cell($arr_error['unknown']['nbr']));				
	  $table->makeXml($root,$xml);
	}
	elseif($p_type=="test"){ //////////////////Requête de type test////////////////////
	  $elm  = $xml->createElement('code',$OK);
	  $root->appendChild($elm);
	}
	elseif($p_type=='activity'){  //////////////////Requête de type activity////////////////////
	  $arr = Model_esia::countNodeType($dbconnection,$user->getId(), Model_esia::NODE_COUNT_SERVICE_STATE);

	  $elm  = $xml->createElement('code',$OK);
	  $root->appendChild($elm);
		
	  $table=new Table();
		
	  //HEADER
	  $color='#263238';
	  $text_color='white';
	  $font="bold";
	  $table->addCell(new Cell('@activity_type',$color,$text_color,null,$font));
	  $table->addCell(new Cell('@activity_nodes',$color,$text_color,null,$font));
	  $table->addCell(new Cell('@activity_services',$color,$text_color,null,$font));
	  $table->addCell(new Cell('@activity_ok',$color,$text_color,null,$font));
	  $table->addCell(new Cell('@activity_warn',$color,$text_color,null,$font));
	  $table->addCell(new Cell('@activity_crit',$color,$text_color,null,$font));
	  $table->addCell(new Cell('@activity_unknown',$color,$text_color,null,$font));

	  $i=1;
	  $tnode=0;
	  $tservice=0;
	  $tok=0;
	  $twarn=0;
	  $tcrit=0;
	  $tunk=0;
	  foreach($arr as $key => $val){
	    $tnode+=$val['nbr_device'];
	    $service=$val['ok']+$val['warning']+$val['critical']+$val['unknown'];			
	    $tservice+=$service;
	    $tok+=$val['ok'];
	    $twarn+=$val['warning'];
	    $tcrit+=$val['critical'];
	    $tunk+=$val['unknown'];
			
	    //BODY
	    $table->newLine();
	    $table->addCell(new Cell($key,'#455a64','white'));
	    $table->addCell(new Cell($val['nbr_device']));
	    $table->addCell(new Cell($service));
	    $table->addCell(new Cell($val['ok'],'green'));
	    if($val['warning']>0) $table->addCell(new Cell($val['warning'],'yellow'));  
	    else $table->addCell(new Cell($val['warning']));
	    if($val['critical']>0) $table->addCell(new Cell($val['critical'],'red'));  
	    else $table->addCell(new Cell($val['critical']));
	    if($val['unknown']>0) $table->addCell(new Cell($val['unknown'],'gray'));  
	    else $table->addCell(new Cell($val['unknown']));

	    $i++;
	  }

	  //TRAILER
	  $table->newLine();

	  $color='#607d8b';
	  $text_color='white';

	  $table->addCell(new Cell('@activity_total',$color,$text_color,null,null,'y'));
	  $table->addCell(new Cell($tnode,$color,$text_color,null,null,'y'));
	  $table->addCell(new Cell($tservice,$color,$text_color,null,null,'y'));
	  $table->addCell(new Cell($tok,$color,$text_color,null,null,'y'));
	  $table->addCell(new Cell($twarn,$color,$text_color,null,null,'y'));
	  $table->addCell(new Cell($tcrit,$color,$text_color,null,null,'y'));		
	  $table->addCell(new Cell($tunk,$color,$text_color,null,null,'y'));

	  $table->makeXml($root,$xml);

	}
	elseif($p_type=='board'){ //////////////////Requête de type board////////////////////
                

	  $list_node = Model_esia::getError($dbconnection,$user->getId());

	  $nodes_fin =array();        
	  foreach($list_node as $node)
	    {
	      if(array_key_exists($node['name'],$nodes_fin))
		{
		  //echo $node['name']." ".$nodes_fin[$node['name']]['type_error']."==".$node['type_error']." <br />";
		  $type_error= $nodes_fin[$node['name']]['type_error'];
		  if($type_error!='critical' && $node['type_error']=='critical')
		    {
		      $type_error = 'critical';
		    }
		  elseif($type_error=='unknown' && $node['type_error']=='warning')
		    {
		      $type_error = 'warning';
		    }
		  $nodes_fin[$node['name']]=array('name'=>$node['name'], 
						  'group'=>$node['group'],
						  'page_control'=>$node['page_control'],
						  'id'=>$node['id'],
						  'type_device'=>$node['type_device'], 
						  'type_error'=>$type_error, 
						  'error'=>$node['error'],                                                 
						  'time'=>$node['time'],
						  'nbre' => $nodes_fin[$node['name']]['nbre']+1 );
		}
	      else
		{
		  $nodes_fin[$node['name']]=array('name'=>$node['name'], 
						  'group'=>$node['group'],
						  'page_control'=>$node['page_control'],
						  'id'=>$node['id'],
						  'type_device'=>$node['type_device'], 
						  'type_error'=>$node['type_error'], 
						  'error'=>$node['error'],                                          
						  'time'=>$node['time'],
						  'nbre' => 1);
		}
	    }		

	  $elm  = $xml->createElement('code',$OK);
	  $root->appendChild($elm);

	  $table=new Table();
	  $color='#263238';
	  $text_color='white';
	  $font="bold";

	  //HEADER
	  $table->addCell(new Cell('@activity_nodes',$color,$text_color,null,$font));
	  $table->addCell(new Cell('@activity_group',$color,$text_color,null,$font));
	  $table->addCell(new Cell('@activity_number',$color,$text_color,null,$font));

	  //BODY
	  foreach($nodes_fin as $val){
	    if($val['type_error']=='critical') $color='red';
	    elseif($val['type_error']=='warning') $color='yellow';
	    elseif($val['type_error']=='unknown') $color='gray';
	    else $color='green';

	    $table->newLine();
	    $table->addCell(new Cell($val['name'],null,null,$val['page_control']));
	    $table->addCell(new Cell($val['group']));
	    $table->addCell(new Cell($val['nbre'],$color));
	  }	
							
	  $table->makeXml($root,$xml);
			
	}
	$user->disconnect();
      }
    catch(UserException $e)
      {
	$elm  = $xml->createElement('code',$ERROR_USER);
	$root->appendChild($elm);
		
	$elm  = $xml->createElement('message',$e->getMessage());
	$root->appendChild($elm);
      }
    catch(Exception $e)
      {
	$elm  = $xml->createElement('code',$ERROR_DB);
	$root->appendChild($elm);
		
	$elm  = $xml->createElement('sqlerr',$e->getCode());
	$root->appendChild($elm);
		
	$elm  = $xml->createElement('message',$e->getMessage());
	$root->appendChild($elm);
      }
  }
else if(isset($p_login)&&empty($p_pass)&&empty($p_type)){ //////////////////Requête de type HANDSHAKE////////////////////
  $elm  = $xml->createElement('code',$HANDSHAKE);
  $root->appendChild($elm);

  $dbconnection = new PDO('pgsql:host='.DB_HOST.';port='.DB_PORT.';dbname='.DB_NAME, DB_USERNAME, DB_PWD);
  $u =  Model_esia::InstanceUser($dbconnection,$p_login);
  $elm  = $xml->createElement('message',$u->getInscriptionTime());
  $root->appendChild($elm);	
}

else
  {
    $elm  = $xml->createElement('code',$ERROR_PROTOCOL);
    $root->appendChild($elm);
  }

$xml->appendChild($root);
echo $xml->saveXML();
?>