<?php
/**
 * @name Device
 * @description Classe principal de la  gestion des noeuds
 * @author Nicolas Biersart <n.biersart@esia-network.com>
 * @contributor
 * @copyright Nicolas Biersart 2012
 * @license
 * @esia 1.00
 * @version 1.0.0
 * @date 16/11/2012
 */
include_once SNMP_CLASS;
include_once LIB_SNMP;
include_once TELNET_CLASS;
include_once SSH_CLASS;
include_once GESA_CLASS;

define('PLUGINS_PATH','/usr/local/esia/plugins');

class RemoteDevice extends Device
{
	const CHECK_GESA='check_gesa.php';
	protected $gesa;
	protected $sysDescr;
	protected $sysContact;
	protected $sysLocation;
	protected $sysUptime;
	protected $sysName;
	protected $sysObjectId;
	const GESA_MIB_SYSTEM = 'get_snmp_system_mib';
	
	/*@NAME: __construct
	 *@IN: (Optionnel) si on passe un objet 'Device', le nouvelle objet sera une copie du précédent
	*@OUT: /
	*@TREAT: Constructeur par défaut de la classe Device.
	*@EXCEPTION: 	DeviceException si l'objet $dev n'est pas un objet device
	*/
	public function __construct ($dev=NULL,$gesa=NULL)
	{
		if(is_object($dev))
		{
			if(get_class($dev)=='Device')
			{
				$this->constructDevice($dev);
				$this->sysDescr=NULL;
				$this->sysContact=NULL;
				$this->sysLocation=NULL;
				$this->sysUptime=NULL;
				$this->sysName=NULL;
				$this->sysObjectId=array();
			}
			else
			{
				throw DeviceException('$dev is not a Device object');
			}
		}
		else
		{
			$this->snmpReq = new SnmpRequest();
			$this->ip_version=4;
			$this->sysDescr=NULL;
			$this->sysContact=NULL;
			$this->sysLocation=NULL;
			$this->sysUptime=NULL;
			$this->sysName=NULL;
			$this->sysObjectId=array();
		}
		if(is_object($gesa))
		{
			if(get_class($gesa)=='GESA')
			{
				$this->gesa=$gesa;
			}	
		}	
		else
		{
			$this->gesa = new GESA();
		}
	}
	/*@NAME: __clone
	 *@IN:/
	*@OUT: /
	*@TREAT: Clone l'objet device
	*@EXCEPTION: /
	*/
	public function __clone()
	{
		$this->constructDevice($self);
	}
	/*@NAME: constructDevice
	 *@IN: $dev un objet Device
	*@OUT: /
	*@TREAT: constructeur de "copie"
	*@EXCEPTION: 	/
	*/
	public function constructDevice(Device $dev)
	{
		$this->id = $dev->getId();
		$this->name = $dev->getName();
		$this->description = $dev->getDescription();
		$this->type = $dev->getType();
		$this->ip = $dev->getIPv4Adress();
		$this->ip_version = $dev->getIPVersion();
		$this->login = $dev->getLogin();
		$this->pwd = $dev->getPassword();
		$this->typeConnection = $dev->getTypeConnection();
		$this->group = $dev->getGroup();
		$this->snmpReq = $dev->getSNMPRequest();
		$this->groupid = $dev->getGroupId();
		$this->list_group = $dev->getGroupList();
	}
	/*@setGESA
	 *@IN: un objet GESA
	*@OUT: /
	*@TREAT: set une GESA
	*@EXCEPTION: /
	*/
	public function setGESA($g)
	{
		if(@get_class($g)=='GESA')
		{
			$this->gesa = $g;
		}
	}
	/*@getGESA
	 *@IN: /
	*@OUT: l'objet GESA
	*@TREAT: Retourne une Gesa
	*@EXCEPTION: /
	*/
	public function getGESA()
	{
		return $this->gesa;
	}
	
	public function detectSNMPVersion($community){
		$address = $this->getIPv4Adress();
		
		$sn = $this->gesa->getSerialNumber();
		$command = 'check_snmp';
		$arguments1 = "-H $address -C $community -P 1 -o .1.3.6.1.2.1.1.2.0 -t 1 -e 0";
		$arguments2 = "-H $address -C $community -P 2c -o .1.3.6.1.2.1.1.2.0 -t 1 -e 0";
		
		$desc1 = "-H $address -C $community -P 1 -o .1.3.6.1.2.1.1.1.0 -t 1 -e 0";
		$desc2 = "-H $address -C $community -P 2c -o .1.3.6.1.2.1.1.1.0 -t 1 -e 0";
		$timeout = 3;
		$pri=3;
		
		exec(PLUGINS_PATH.'/'.self::CHECK_GESA." -G $sn -C $command -A '$arguments2' -t $timeout -p $pri",$output,$returnexec);
		
		if( $returnexec != 0 ){
			unset($output);
			exec(PLUGINS_PATH.'/'.self::CHECK_GESA." -G $sn -C $command -A '$arguments1' -t $timeout -p $pri",$output,$returnexec);
			if( $returnexec != 0 ) return null;
			else $version = SnmpRequest::VERSION_1;
		}
		else $version = SnmpRequest::VERSION_2C;
		
		if(preg_match ( '/=\s+OID:\s+(\S*)/' , $output[0], $regs )){
			$this->sysObjectId = array($regs[1]);
		}
		
		if($version == SnmpRequest::VERSION_1 ) $desc = $desc1;
		else $desc = $desc2;
		
		unset($output);
		exec(PLUGINS_PATH.'/'.self::CHECK_GESA." -G $sn -C $command -A '$desc' -t $timeout -p $pri",$output,$returnexec);
		if(preg_match ( '/-\s+"([^"]*)"/' , $output[0], $regs )){
			$this->sysDescr = $regs[1];
		}
		else 
		{
			$this->sysDescr = "";
		}
		
		return $version;
	}
	
	/*@NAME:getHostAliveBySNMP
	 *@IN: /
	*@OUT: TRUE Si sa le test reussi sinon une Exception est levée
	*@TREAT: Effectue une requête SNMP sur le noeud (oid du sysUptime) si le noeud répond TRUE est retourné sinon  une exception est levée
	*@EXCEPTION: DeviceNotAliveSNMPException si une erreur survient
	*/
	public function getHostAliveBySNMP()
	{
		try
		{
			$uptime =$this->snmpget(".1.3.6.1.2.1.1.3.0");
		}
		catch (DeviceSNMPException $e)
		{
			throw new DeviceNotAliveSNMPException('Device SNMP not respond',1,$e);
		}
		return true;
	}
	/*@NAME:getUpTime
	 *@IN: /
	*@OUT: un tableau associatif contenant les clés 'day','hours','minut' et 'second'
	*@TREAT: Effectue une requête SNMP via la GESA sur le noeud (oid du sysUptime  => .1.3.6.1.2.1.1.3.0)
	*@EXCEPTION: DeviceException si une erreur survient
	*/
	public function getUpTime()
	{
		if(!isset($this->sysUpTime)&&empty($this->sysUpTime))
		{
			$this->getSystemMib();
		}
		return $this->sysUpTime;
	}
	/*@NAME:getSysDescr
	 *@IN: /
	*@OUT: le sysDescr du noeud
	*@TREAT: Effectue une requête SNMP via la GESA sur le noeud (oid du sysDescription => .1.3.6.1.2.1.1.1.0)
	*@EXCEPTION: DeviceException si une erreur survient
	*/
	public function getSysDescr()
	{
		if(!isset($this->sysDescr)&&empty($this->sysDescr))
		{
			$this->getSystemMib();
		}
		return $this->sysDescr;
	}
	/*@NAME:getSysLocation
	 *@IN: /
	*@OUT: la localisation du noeud
	*@TREAT: Effectue une requête SNMP via la GESA sur le noeud (oid du sysLocation => .1.3.6.1.2.1.1.6.0)
	*@EXCEPTION: DeviceException si une erreur survient
	*/
	public function getSysLocation()
	{
		if(!isset($this->sysLocation)&&empty($this->sysLocation))
		{
			$this->getSystemMib();
		}
		return $this->sysLocation;
	}
	/*@NAME:getSysContact
	 *@IN: /
	*@OUT: le contact du noeud
	*@TREAT: Effectue une requête SNMP via la GESA sur le noeud (oid du sysContact => .1.3.6.1.2.1.1.4.0)
	*@EXCEPTION: DeviceException si une erreur survient
	*/
	public function getSysContact()
	{
		if(!isset($this->sysContact)&&empty($this->sysContact))
		{
			$this->getSystemMib();
		}
		return $this->sysContact;
	}
	/*@getsysName
	 *@IN: /
	*@OUT: le nom du noeud
	*@TREAT: Effectue une requête SNMP via la GESA sur le noeud (oid du sysContact => .1.3.6.1.2.1.1.4.0)
	*@EXCEPTION: DeviceException si une erreur survient
	*/
	public function getSysName()
	{
		if(!isset($this->sysName)&&empty($this->sysName))
		{
			$this->getSystemMib();
		}
		return $this->sysName;
	}
	/*@getSysObjectId
	 *@IN: /
	*@OUT: le nom du noeud
	*@TREAT: Effectue une requête SNMP via la GESA sur le noeud (oid du sysContact => .1.3.6.1.2.1.1.4.0)
	*@EXCEPTION: DeviceException si une erreur survient
	*/
	public function getSysObjectId()
	{
		if(!isset($this->sysObjectId)&&empty($this->sysObjectId))
		{
			$this->getSystemMib();
		}
		return $this->sysObjectId ;
	}
	/*@NAME:snmpget
	 *@IN: $oid à interroger
	*@OUT: le résultat de la reqête
	*@TREAT: Effectue une requête SNMP GET sur le noeud
	*@EXCEPTION: DeviceSNMPException si une erreur survient
	*/
	public function snmpget($oid)
	{
		try
		{
			return $this->snmpReq->snmpget($this->ip,$oid);
		}
		catch(SnmpException $e)
		{
			throw new DeviceSNMPException($e->getMessage(),1,$e);
		}
	}
	/*@NAME:snmpwalk
	 *@IN: $oid à interroger
	*@OUT: le résultat de la reqête
	*@TREAT: Effectue une requête SNMP WALK sur le noeud
	*@EXCEPTION: DeviceSNMPException si une erreur survient
	*/
	public function snmpwalk($oid)
	{
		try
		{
			return $this->snmpReq->snmpwalk($this->ip,$oid);
		}
		catch(SnmpException $e)
		{
			throw new DeviceSNMPException($e->getMessage(),1,$e);
		}
	}
	/*@NAME:snmpwalkoid
	 *@IN: $oid à interroger
	*@OUT: le résultat de la reqête
	*@TREAT: Effectue une requête SNMP WALK sur le noeud
	*@EXCEPTION: DeviceSNMPException si une erreur survient
	*/
	public function snmpwalkoid($oid)
	{
		try
		{
			return $this->snmpReq->snmpwalkoid($this->ip,$oid);
		}
		catch(SnmpException $e)
		{
			throw new DeviceSNMPException($e->getMessage(),1,$e);
		}
	}
	/*@NAME:getSNMPObject
	 *@IN: /
	*@OUT: l'OBJET SNMPREquest
	*@TREAT: retourne l'objet SNMP Request du noeud
	*@EXCEPTION: /
	*/
	public function getSNMPObject()
	{
		return $this->snmpReq;
	}
	/*@NAME:checkConnection
	 *@IN: /
	*@OUT: /
	*@TREAT: test la connexion du noeud
	*@EXCEPTION: DeviceException si une erreur survient
	*/
	public function checkConnection($lien='')
	{
		if($this->typeConnection==Device::TELNET)
		{
			try
			{
				$t = new Telnet($this->ip,23,1);
				$t->login($this->login,$this->pwd);
				$t->close();
			}
			catch(TelnetException $e)
			{
				throw new DeviceException($e->getMessage(),2,$e);
			}
		}
		elseif($this->typeConnection==Device::SSH)
		{
			try
			{
				$t = new SSH($this->ip,22);
				$t->login($this->login,$this->pwd);
				$t->close();
			}
			catch(SSHException $e)
			{
				throw new DeviceException($e->getMessage(),2,$e);
			}
		}
		else
		{
			throw new DeviceException('Not connection protocol specified');
		}
	}
	protected function getSystemMib()
	{
		try
		{
			if( $this->getSNMPVersion() != 3 ){
				$param = $this->getSNMPCommunity();
				$sysinfo = $this->gesa->check_nrpe($this,RemoteDevice::GESA_MIB_SYSTEM,$param);
			}
			else{
				$snmpReq = $this->getSNMPRequest();
				$level = $snmpReq->getSNMP3SecLevel();
				if( $level == 'authPriv' ){
					$priv_prot = $snmpReq->getPrivacyProtocol();
					$priv_pwd = $snmpReq->getPrivacyKey();
					$trailer = " -x $priv_prot -X $priv_pwd";
				}
				else $trailer='';
				$user = $snmpReq->getSNMP3User();
				$auth_prot = $snmpReq->getSNMP3AuthProtocol();
				$auth_pwd = $snmpReq->getSNMP3AuthKey();
				$sysinfo = $this->gesa->check_ecatp($this,RemoteDevice::GESA_MIB_SYSTEM.'.pl',"-v3 -u $user -a $auth_prot -A $auth_pwd -L $level".$trailer);
			}
			$sysinfo = $this->ThreadGESAError($sysinfo, RemoteDevice::GESA_MIB_SYSTEM);
			foreach ($sysinfo as $line)
			{
				switch ($line[0])
				{
					case 'sysDescr': //$this->sysDescr = $line[1]; break;
						$this->sysDescr = $line[1];
						if($this->sysDescr == NULL) $this->sysDescr = "";
						break;						
					case 'sysUpTime': $this->sysUpTime = $line[1]; break;
					case 'sysContact': $this->sysContact = $line[1]; break;
					case 'sysName': $this->sysName = $line[1]; break;
					case 'sysLocation': $this->sysLocation = $line[1]; break;
					case 'sysObjectID': 
						$this->sysObjectId[] = $line[1];
						if(isset($line[2]))
							$this->sysObjectId[] = $line[2]; 
						break;
				}
			}
		}
		catch(GesaConnectionFailed $e)
		{
			throw new DeviceException($e->getMessage());
		}
	}
	protected function ThreadGESAError($info,$function)
	{
		if($info['status']=='END')
		{
			return $info['data'];
		}
		else
		{
			throw new DeviceException("Bad fonction: ".$function);
		}		
	}
	protected function getInterfaceMib()
	{
		//INTERFACE A FAIRE 
	}
}
?>
