<?php
/**
 * @name MapModel
 * @description Classe modèle du plugin de géolocalisation d'Esia hérite de Model_esia
 *		Toutes les méthodes sont déclarée en 'static' ainsi on garde les Objets PDO pour les programmeurs extérieurs et pour faire des modifications, ils ne sont pas /
 *		obligé de connaitre la hierachie de classe d'ESIA
 * @author Nicolas Biersart <n.biersart@esia-network.com>
 * @contributor
 * @copyright Nicolas Biersart 2012
 * @license
 * @esia 1.00
 * @version 1.0.0
 * @date 23/06/2013
 */
include_once MVCMODEL;

class MapModel extends Model_esia
{
	/*@NAME: GetMapConfiguration
	 *@IN: $dbconnection l'objet de connexion PDO
	*@OUT: un tableau associatif les clés sont:
	*  $tab['CENTER_LATITUDE']: la latitude du centre de la carte par défaut
	*  $tab['CENTER_LONGITUDE']: la longitude du centre de la carte par défaut
	*  $tab['TYPE']: le type de la carte par défaut
	*  $tab['ZOOM']: le niveau de zoom par défaut
	*  $tab['ZOOM_CHANGED']: le niveau de zoom du changement bâtiment/zone par défaut
	*  $tab['IMAGE_MAX_SIZE']: taille maximun d'une image en Mo
	*  $tab['IMG_DIRECTORY']: repertoire de destination des images (apache)
	*@TREAT:  Effectue une requête sur la base de donnée pour récupérer les valeurs par défaut de la carte. (situé dans esia_configuration => clé MAP_*)
	*@EXCEPTION: 	EsiaDbException si une erreur survient
	*/
	static function GetMapConfiguration($dbconnection)
	{
		$conf = array();
		$query = $dbconnection->prepare("SELECT config_key,config_value
				FROM esia_configuration
				WHERE config_key LIKE 'MAP_%'");
		$query = Model_esia::executeStatement($query);
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			if($row->config_key=='MAP_CENTER_LATITUDE')
				$conf['CENTER_LATITUDE'] = $row->config_value;
			elseif($row->config_key=='MAP_CENTER_LONGITUDE')
				$conf['CENTER_LONGITUDE'] = $row->config_value;
			elseif($row->config_key=='MAP_TYPE')
				$conf['TYPE'] = $row->config_value;
			elseif($row->config_key=='MAP_ZOOM')
				$conf['ZOOM'] = $row->config_value;
			elseif($row->config_key=='MAP_ZOOM_CHANGED')
				$conf['ZOOM_CHANGED'] = $row->config_value;
			elseif($row->config_key=='MAP_IMG_TAILLE_MAX')
				$conf['IMAGE_MAX_SIZE'] = $row->config_value;
			elseif($row->config_key=='MAP_IMG_DIR')
				$conf['IMG_DIRECTORY'] = $row->config_value;
		}
		return $conf;
	}
	/*@NAME: getZoneState
	 *@IN: $dbconnection l'objet de connexion PDO
	*	$zoneid (optionnel) id d'une zone
	*	$user	(optionnel) nom d'utilisateur
	*@OUT: un tableau associatif les clés sont:
	*  $tab[<id_zone>]:id de la zone
	*  $tab[<id_zone>]['name']: nom de la zone
	*  $tab[<id_zone>]['ok']: nombre de service en OK
	*  $tab[<id_zone>]['warning']: nombre de service en warning
	*  $tab[<id_zone>]['critical']: nombre de service en critique
	*  $tab[<id_zone>]['unknown']: nombre de service en inconnu
	*  $tab[<id_zone>]['latitude']: tableau contenant les coordonnées de latitude délimitant la zone
	*  $tab[<id_zone>]['longitude']: tableau contenant les coordonnées de longitude délimitant la zone
	*@TREAT: Recherche la zone donnée ou toutes les zones, si on spécifie un droit d'utilisateur, on vérifiera le droit device_view
	*@EXCEPTION: 	EsiaDbException si une erreur survient
	*/
	static function getZoneState($dbconnection,$zoneid=null,$user = null)
	{
		$sql = "SELECT zone_id,zone_name,zone_latitude,zone_longitude,error_type,count(DISTINCT happen_id)as nbr_error
				FROM esia_zone
				JOIN esia_building ON zone_id = building_zone_id
				JOIN esia_floor ON floor_building_id = building_id		
				JOIN (
					SELECT localization_floor_id,error_type,device_id,happen_id
					FROM esia_localization
					JOIN esia_device ON localization_device_id = device_id
					JOIN esia_error_happen ON device_id = happen_device_id
					JOIN esia_errors ON error_id = happen_error_id
					WHERE happen_acquitte=0
					UNION 
					SELECT localization_floor_id,error_type,device_id,happen_id
					FROM esia_localization
					JOIN esia_container ON localization_container_id = container_id
					JOIN esia_device_stored_container ON  container_id = stored_container_id
					JOIN esia_device ON stored_device_id = device_id
					JOIN esia_error_happen ON device_id = happen_device_id
					JOIN esia_errors ON error_id = happen_error_id
					WHERE happen_acquitte=0
				) AS l1 ON l1.localization_floor_id = floor_id
				WHERE 1=1";
		//requete pour les zone en OK 
		$sql2 = " UNION
		SELECT zone_id,zone_name,zone_latitude,zone_longitude,'ok',count(DISTINCT service_id)as nbr_error
		FROM esia_zone
		JOIN esia_building ON zone_id = building_zone_id
		JOIN esia_floor ON floor_building_id = building_id
		JOIN (
			SELECT localization_floor_id,device_id,service_id
			FROM esia_localization
			JOIN esia_device ON localization_device_id = device_id 
			JOIN esia_services ON service_device_id = device_id 
			WHERE service_previous_state=0 AND service_delete=FALSE
			UNION 
			SELECT localization_floor_id,device_id,service_id
			FROM esia_localization
			JOIN esia_container ON localization_container_id = container_id
			JOIN esia_device_stored_container ON  container_id = stored_container_id
			JOIN esia_device ON stored_device_id = device_id
			JOIN esia_services ON service_device_id = device_id 
			WHERE service_previous_state=0 AND service_delete=FALSE
		) AS l1 ON l1.localization_floor_id = floor_id 
			WHERE 1=1";
		/*$sql3=" UNION
		SELECT zone_id,zone_name,zone_latitude,zone_longitude,'ok',0
		FROM esia_zone";*/
		$sql3='';
		$param=array();
		if(is_numeric($zoneid))
		{
			$sql .= "AND zone_id= :zone ";
			$sql2.= "AND zone_id= :zone ";
			$param['zone'] = $zoneid;
		}
		if($user!=null)
		{
			$sql .= "AND device_id IN ( SELECT * FROM getUserAccessNode(:user))";
			$sql2.= "AND device_id IN ( SELECT * FROM getUserAccessNode(:user))";
			$param['user'] = $user;
		}
		$sql .= " GROUP BY  zone_id,zone_name,zone_latitude,zone_longitude, error_type";
		$sql2.= " GROUP BY  zone_id,zone_name,zone_latitude,zone_longitude";
		$sql2.= " ORDER BY zone_name";
		//echo "<pre>".$sql.$sql2."</pre>";
		$query= $dbconnection->prepare($sql.$sql2.$sql3);
		$query = Model_esia::executeStatement($query,$param);
		$arr=array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$arr[$row->zone_id]['name']=$row->zone_name;
			$arr[$row->zone_id][$row->error_type]=$row->nbr_error;
			$arr[$row->zone_id]['latitude'] = MapModel::LatLngToArray($row->zone_latitude);
			$arr[$row->zone_id]['longitude'] = MapModel::LatLngToArray($row->zone_longitude);
			if(!array_key_exists("ok", $arr[$row->zone_id]))
			{
				$arr[$row->zone_id]['ok']=0;
			}
			if(!array_key_exists("warning", $arr[$row->zone_id]))
			{
				$arr[$row->zone_id]['warning']=0;
			}
			if(!array_key_exists("critical", $arr[$row->zone_id]))
			{
				$arr[$row->zone_id]['critical']=0;
			}
			if(!array_key_exists("unknown", $arr[$row->zone_id]))
			{
				$arr[$row->zone_id]['unknown']=0;
			}
		}
		return $arr;
	}
	/*@NAME: getBuildingState
	 *@IN: $dbconnection l'objet de connexion PDO
	*	$zoneid (optionnel) id d'un bâtiment
	*	$user	(optionnel) nom d'utilisateur
	*@OUT: un tableau associatif les clés sont:
	*  $tab[<id_building>]:id de la zone
	*  $tab[<id_building>]['name']: nom de la zone
	*  $tab[<id_building>]['ok']: nombre de service en OK
	*  $tab[<id_building>]['warning']: nombre de service en warning
	*  $tab[<id_building>]['critical']: nombre de service en critique
	*  $tab[<id_building>]['unknown']: nombre de service en inconnu
	*  $tab[<id_building>]['latitude']: coordonnées de latitude
	*  $tab[<id_building>]['longitude']: coordonnées de longitude
	*@TREAT: Recherche le bâtiment donné ou tout les bâtiment, si on spécifie un utilisateur, on vérifiera le droit device_view
	*@EXCEPTION: 	EsiaDbException si une erreur survient
	*/
	static function getBuildingState($dbconnection,$id=null,$user = null)
	{
		$sql = "SELECT building_id,building_name,building_latitude,building_longitude,error_type,count(DISTINCT happen_id)as nbr_error
		FROM esia_building
		JOIN esia_floor ON floor_building_id = building_id
		JOIN (
					SELECT localization_floor_id,error_type,device_id,happen_id
					FROM esia_localization
					JOIN esia_device ON localization_device_id = device_id
					JOIN esia_error_happen ON device_id = happen_device_id
					JOIN esia_errors ON error_id = happen_error_id
					WHERE happen_acquitte=0
					UNION 
					SELECT localization_floor_id,error_type,device_id,happen_id
					FROM esia_localization
					JOIN esia_container ON localization_container_id = container_id
					JOIN esia_device_stored_container ON  container_id = stored_container_id
					JOIN esia_device ON stored_device_id = device_id
					JOIN esia_error_happen ON device_id = happen_device_id
					JOIN esia_errors ON error_id = happen_error_id
					WHERE happen_acquitte=0
				) AS l1 ON l1.localization_floor_id = floor_id
		WHERE 1=1 ";
		$sql2 = " UNION
		SELECT building_id,building_name,building_latitude,building_longitude,'ok',count(DISTINCT service_id)as nbr_error
		FROM esia_building
		JOIN esia_floor ON floor_building_id = building_id
		JOIN (
			SELECT localization_floor_id,device_id,service_id
			FROM esia_localization
			JOIN esia_device ON localization_device_id = device_id 
			JOIN esia_services ON service_device_id = device_id 
			WHERE service_previous_state=0 AND service_delete=FALSE
			UNION 
			SELECT localization_floor_id,device_id,service_id
			FROM esia_localization
			JOIN esia_container ON localization_container_id = container_id
			JOIN esia_device_stored_container ON  container_id = stored_container_id
			JOIN esia_device ON stored_device_id = device_id
			JOIN esia_services ON service_device_id = device_id 
			WHERE service_previous_state=0 AND service_delete=FALSE
		) AS l1 ON l1.localization_floor_id = floor_id 
		WHERE 1=1 ";
		$sql3="";
	/*	$sql3=" UNION
		SELECT  building_id,building_name,building_latitude,building_longitude,'ok',0
		FROM esia_building";*/
		$param=array();
		if(is_numeric($id))
		{
			$sql .= "AND building_id= :building ";
			$sql2.= "AND building_id= :building ";
			$param['building'] = $id;
		}
		if($user!=null)
		{
			$sql .= "AND device_id IN ( SELECT * FROM getUserAccessNode(:user))";
			$sql2.= "AND device_id IN ( SELECT * FROM getUserAccessNode(:user))";
			$param['user'] = $user;
		}
		$sql .= " GROUP BY  building_id,building_name,building_latitude,building_longitude, error_type";
		$sql2.= " GROUP BY  building_id,building_name,building_latitude,building_longitude";
		$sql2.= " ORDER BY building_name";
		$query= $dbconnection->prepare($sql.$sql2.$sql3);
		$query = Model_esia::executeStatement($query,$param);
		$arr=array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$arr[$row->building_id]['name']=$row->building_name;
			$arr[$row->building_id][$row->error_type]=$row->nbr_error;
			$arr[$row->building_id]['latitude'] = $row->building_latitude;
			$arr[$row->building_id]['longitude'] = $row->building_longitude;
			if(!array_key_exists("ok", $arr[$row->building_id]))
			{
				$arr[$row->building_id]['ok']=0;
			}
			if(!array_key_exists("warning", $arr[$row->building_id]))
			{
				$arr[$row->building_id]['warning']=0;
			}
			if(!array_key_exists("critical", $arr[$row->building_id]))
			{
				$arr[$row->building_id]['critical']=0;
			}
			if(!array_key_exists("unknown", $arr[$row->building_id]))
			{
				$arr[$row->building_id]['unknown']=0;
			}
		}
		return $arr;
	}
	/*@NAME: GetZone
	 *@IN: $dbconnection l'objet de connexion PDO
	*	$id l'id de la zone (optionnel) doit être numérique
	*	$building=true si on veut récupérer les bâtiments en plus de la zone
	*@OUT: un tableau associatif les clés sont:
	*  $tab[<id_zone>]['id']:id de la zone
	*  $tab[<id_zone>]['name']: nom de la zone
	*  $tab[<id_zone>]['abbreviation']: nombre de service en OK
	*  $tab[<id_zone>]['latitude']: tableau contenant les coordonnées de latitude délimitant la zone
	*  $tab[<id_zone>]['longitude']: tableau contenant les coordonnées de longitude délimitant la zone
	*  SI spécifié par le paramètre 'Building' les information suivantes seront disponible(il peut-être vide si rien n'est spécifié
			*  $tab[<id_zone>]['building'][<id_building>]['id'] id du bâtiment
			*  $tab[<id_zone>]['building'][<id_building>]['name'] nom du bâtiment
			*  $tab[<id_zone>]['building'][<id_building>]['abbreviation'] abréviation du bâtiments
			*  $tab[<id_zone>]['building'][<id_building>]['description'] description ,
			*  $tab[<id_zone>]['building'][<id_building>]['latitude'] latitude,
			*  $tab[<id_zone>]['building'][<id_building>]['longitude'] longitude,
			*  $tab[<id_zone>]['building'][<id_building>]['floor'][<id_floor>]['id'] id de l'étage
			*  $tab[<id_zone>]['building'][<id_building>]['floor'][<id_floor>]['name'] nom de l'étage
			*@TREAT: Effectue une requête sur la base de donnée pour récupérer les informations sur les/la zone(s) ainsi que leur bâtiments
			*@EXCEPTION: 	EsiaDbException si une erreur de connexion survient ou si l'id n'est pas numérique et supérieur à 0
			*/
	static function GetZone($dbconnection,$id=null,$building=true)
	{
		$list=array();
		if(empty($id))
		{
			$query = $dbconnection->prepare('SELECT zone_id,zone_name,zone_abreviation,zone_latitude,zone_longitude
					FROM esia_zone ORDER BY zone_name');
			$query = Model_esia::executeStatement($query);
		}
		elseif(is_numeric($id)&&$id>0)
		{
			$query = $dbconnection->prepare('SELECT zone_id,zone_name,zone_abreviation,zone_latitude,zone_longitude
					FROM esia_zone
					WHERE zone_id=?
					ORDER BY zone_name');
			$query = Model_esia::executeStatement($query,array($id));
		}
		else
		{
			throw new EsiaDbException('Error parameter: id must be numeric');
		}
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$list[$row->zone_id]=array('id'=>$row->zone_id,
					'name'=>$row->zone_name,
					'abbreviation'=>$row->zone_abreviation,
					'latitude'=>MapModel::LatLngToArray($row->zone_latitude),
					'longitude'=>MapModel::LatLngToArray($row->zone_longitude));
		}
		if($building===true)
		{
			foreach($list  as $id => $row)
			{
			 $list[$id]['building']=MapModel::GetBuildingByZone($dbconnection,$id);
			}
		}
		return $list;
	}
	/*@NAME: GetBuildingByZone
	 *@IN: $dbconnection l'objet de connexion PDO
	*	$id l'id de la zone (optionnel) doit être numérique
	*@OUT: un tableau associatif les clés sont:
	*  $tab[<id_building>]['id'] id du bâtiment
	*  $tab[<id_building>]['name'] nom du bâtiment
	*  $tab[<id_building>]['abbreviation'] abréviation du bâtiments
	*  $tab[<id_building>]['description'] description ,
	*  $tab[<id_building>]['latitude'] latitude,
	*  $tab[<id_building>]['longitude'] longitude,
	*  $tab[<id_building>]['floor'][<id_floor>]['id'] id de l'étage
	*  $tab[<id_building>]['floor'][<id_floor>]['name'] nom de l'étage
	*@TREAT: Effectue une requête sur la base de donnée pour récupérer les informations sur les bâtiments composant la zone
	*@EXCEPTION: 	EsiaDbException si une erreur de connexion survient ou si l'id n'est pas numérique et supérieur à 0
	*/
	static function GetBuildingByZone($dbconnection,$id)
	{
		$list_build=array();
		if(is_numeric($id)&&$id>0)
		{
			$query = $dbconnection->prepare('SELECT building_id,building_name,building_abreviation,building_description,building_latitude,building_longitude,floor_id, floor_name
					FROM esia_building
					LEFT JOIN esia_floor ON floor_building_id= building_id
					WHERE building_zone_id=? ORDER BY building_name');
			$query = Model_esia::executeStatement($query,array($id));
		}
		elseif(is_string($id))
		{
			$query = $dbconnection->prepare('SELECT building_id,building_name,building_abreviation,building_description,building_latitude,building_longitude,floor_id, floor_name
					FROM esia_zone
					JOIN esia_building on zone_id = building_zone_id
					LEFT JOIN esia_floor ON floor_building_id= building_id
					WHERE zone_name=? ORDER BY building_name');
			$query = Model_esia::executeStatement($query,array($id));
		}
		else
		{
			throw new EsiaDbException('Error parameter: id must be numeric or the name of building');
		}
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			if(!array_key_exists($row->building_id,$list_build))
			{
				$list_build[$row->building_id]=array('id'=>$row->building_id,
						'name'=>$row->building_name,
						'abbreviation'=>$row->building_abreviation,
						'description'=>$row->building_description,
						'latitude'=>$row->building_latitude,
						'longitude'=>$row->building_longitude,
				);
			}
			$list_build[$row->building_id]['floor'][$row->floor_id]=array('id'=>$row->floor_id,'name'=>$row->floor_name);
		}
		return $list_build;
	}
	/*@NAME: GetBuilding
	 *@IN: $dbconnection l'objet de connexion PDO
	*	$id l'id du bâtiment (optionnel) doit être numérique
	*@OUT: un tableau associatif les clés sont:
	*    $tab[<building_id>]['zone_id']: l'id de la zone où se trouve le bâtiment
	*    $tab[<building_id>]['zone']: le nom de la zone où se trouve le bâtiment
	*    $tab[<building_id>]['id']: id du batîment
	*    $tab[<building_id>]['name']: le nom du batîment
	*    $tab[<building_id>]['abbreviation']:  l'abréviation du batîment
	*    $tab[<building_id>]['description']:  la description du batîment
	*    $tab[<building_id>]['latitude']:  la latitude du batîment
	*    $tab[<building_id>]['longitude']: la longitude du batîment
	*    $tab[<building_id>]['floor']: tableau contenant les infoprmations sur les différents étages
	*@TREAT: Effectue une requête sur la base de donnée pour récupérer les informations sur le(s) bâtiment(s)
	*@EXCEPTION: 	EsiaDbException si une erreur de connexion survient ou si l'id n'est pas numérique et supérieur à 0 ou si l'id n'est pas un string
	*/
	static function GetBuilding($dbconnection,$id=null)
	{
		$ret=false;
		if(empty($id))
		{
			$query = $dbconnection->prepare('SELECT zone_id, zone_name,building_id,building_name,building_abreviation,building_description,building_latitude,building_longitude,floor_id, floor_name,floor_map
					FROM esia_building
					LEFT JOIN esia_zone on zone_id = building_zone_id
					LEFT JOIN esia_floor ON floor_building_id= building_id ORDER BY building_name');
			$query = Model_esia::executeStatement($query);
		}
		elseif(is_numeric($id)&&$id>0)
		{
			$query = $dbconnection->prepare('SELECT zone_id, zone_name,building_id,building_name,building_abreviation,building_description,building_latitude,building_longitude,floor_id, floor_name,floor_map
					FROM esia_building
					LEFT JOIN esia_zone on zone_id = building_zone_id
					LEFT JOIN esia_floor ON floor_building_id= building_id
					WHERE building_id=? ORDER BY building_name');
			$query = Model_esia::executeStatement($query,array($id));
		}
		elseif(is_string($id))
		{
			$query = $dbconnection->prepare('SELECT zone_id, zone_name,building_id,building_name,building_abreviation,building_description,building_latitude,building_longitude,floor_id, floor_name,floor_map
					FROM esia_building
					LEFT JOIN esia_zone on zone_id = building_zone_id
					LEFT JOIN esia_floor ON floor_building_id= building_id
					WHERE building_name=? ORDER BY building_name');
			$query = Model_esia::executeStatement($query,array($id));
		}
		else
		{
			throw new EsiaDbException('Error parameter: id must be numeric or the name of building');
		}
		$list_build=array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			if(!array_key_exists($row->building_id,$list_build))
			{
				$list_build[$row->building_id]=array('zone_id'=>$row->zone_id,
						'zone'=>$row->zone_name,
						'id'=>$row->building_id,
						'name'=>$row->building_name,
						'abbreviation'=>$row->building_abreviation,
						'description'=>$row->building_description,
						'latitude'=>$row->building_latitude,
						'longitude'=>$row->building_longitude,
				);
			}
			if(!empty($row->floor_id))
			{
				$map='';
				if(preg_match('#'.$_SERVER['DOCUMENT_ROOT'].'([a-zA-Z0-9\.\\-/\_]*)#',$row->floor_map,$regs))
				{
					$map='/'.$regs[1];
				}
				$list_build[$row->building_id]['floor'][$row->floor_id]=array('id'=>$row->floor_id,'name'=>$row->floor_name,'map'=>$map);
			}
		}
		//echo'<pre>'; print_r($list_build);echo'</pre>';
		return $list_build;
	}
	/*@NAME: GetFloors
	 *@IN: $dbconnection l'objet de connexion PDO
	*	$id l'id de l'étage (optionnel) ou son nom
	*@OUT: un tableau associatif les clés sont:
	*    $tab[<floor_id>]['id']: id de l'étage
	*    $tab[<floor_id>]['name']: le nom de l'étage
	*    $tab[<floor_id>]['map']:  lien vers l'image de fond
	*    $tab[<floor_id>]['building_id']:  id du bâtiment
	*@TREAT: Effectue une requête sur la base de donnée pour récupérer les informations sur le(s) étage(s)
	*@EXCEPTION: 	EsiaDbException si une erreur de connexion survient ou si l'id n'est pas numérique et supérieur à 0 ou si l'id n'est pas un string
	*/
	static function GetFloors($dbconnection,$id=null)
	{
		$ret=false;
		if(empty($id))
		{
			$query = $dbconnection->prepare('SELECT floor_id, floor_name, floor_map, floor_building_id FROM esia_floor ORDER BY floor_name;');
			$query = Model_esia::executeStatement($query);
		}
		elseif(is_numeric($id)&&$id>0)
		{
			$query = $dbconnection->prepare('SELECT floor_id, floor_name, floor_map, floor_building_id FROM esia_floor WHERE floor_id=? ORDER BY floor_name;');
			$query = Model_esia::executeStatement($query,array($id));
		}
		elseif(is_string($id))
		{
			$query = $dbconnection->prepare('SELECT floor_id, floor_name, floor_map, floor_building_id FROM esia_floor WHERE floor_name=? ORDER BY floor_name;');
			$query = Model_esia::executeStatement($query,array($id));
		}
		else
		{
			throw new EsiaDbException('Error parameter: id must be numeric or the name of map');
		}
		$list_floors=array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			if(preg_match('#'.$_SERVER['DOCUMENT_ROOT'].'([a-zA-Z0-9\.\\-/\_]*)#',$row->floor_map,$regs))
			{
				$map='/'.$regs[1];
				$size = getimagesize($row->floor_map);
				$width=$size[0];
				$height=$size[1];
			}
			$list_floors[$row->floor_id]=array(
					'id'=>$row->floor_id,
					'name'=>$row->floor_name,
					'map'=>$map,
					'height'=>$height,
					'width'=>$width,
					'building_id'=>$row->floor_building_id);
		}
		//echo'<pre>'; print_r($list_build);echo'</pre>';
		return $list_floors;
	}
	/*@NAME: getMapFloorInformation
	 *@IN: $dbconnection l'objet de connexion PDO
	*	$id de l'étage
	*@OUT: un tableau associatif les clés sont:
	*		$floor['zone_name'] nom de la zone
	*		$floor['building_name'] nom du bâtiment
	*		$floor['floor_name'] nom de l'étage
	* 		$floor['floor_map'] chemin d'accès de fichier d'arrière plan (image)
	*		$floor['map_width'] longueur de l'image
	*		$floor['map_height'] hauteur de l'image
	*@TREAT: renvoi les informations sur un étage donné
	*@EXCEPTION: 	EsiaDbException si une erreur de connexion survient ou si l'id n'est pas numérique et supérieur à 0
	*/
	static function getMapFloorInformation($dbconnection, $id=null)
	{
		$sql = 'SELECT zone_id,zone_name,building_id,building_name,floor_id,floor_name,floor_map
				FROM esia_floor
				JOIN esia_building ON building_id= floor_building_id
				JOIN esia_zone ON zone_id= building_zone_id';
		$param=array();
		if(isset($id))
		{
			if(!is_numeric($id))
			{
				throw new EsiaDbException('Error parameter: id must be numeric');
			}
			$sql.=' WHERE floor_id=?';
			$params[]=$id;
		}
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);
		$list=array();
		while($r_floor=$query->fetch(PDO::FETCH_OBJ))  //prend la zone,l batiment et l'étages
		{
			$floor = array();
			//Information sur le plan (taille et chemin d'accès)
			$map='';
			$height='';
			$width='';
			if(preg_match('#'.$_SERVER['DOCUMENT_ROOT'].'([a-zA-Z0-9\.\\-/\_]*)#',$r_floor->floor_map,$regs))
			{
				$map='/'.$regs[1];
				$size = getimagesize($r_floor->floor_map);
				$width=$size[0];
				$height=$size[1];
			}
			$floor['floor_id'] =$r_floor->floor_id;
			$floor['zone_name'] =$r_floor->zone_name;
			$floor['building_name'] =$r_floor->building_name;
			$floor['floor_name'] =$r_floor->floor_name;
			$floor['floor_map'] = $map;
			$floor['map_width'] = $width;
			$floor['map_height'] = $height;
			$list[]=$floor;
		}
		if(isset($id))
		{
			$floor = array_pop($list);
		}
		else 
		{
			$floor = $list;
		}
		return $floor;
	}
	/*@NAME: getImage
	 *@IN: $dbconnection l'objet de connexion PDO
	*@OUT: un tableau associatif les clés sont:
	* 	$list_img[<id>]['id'] id de l'image
	*	$list_img[<id>]['name'] nom de l'image
	*	$list_img[<id>]['description'] description de l'image
	*	$list_img[<id>]['img_ok'] image quand tout va bien
	*	$list_img[<id>]['img_warning'] image quand il y a une alerte
	*	$list_img[<id>]['img_critical'] image quand il y a une erreur critique
	*	$list_img[<id>]['img_unknown'] image quand il y a une erreur inconnue
	*@TREAT: renvoi les informations sur les type de noeud et les image associée
	*@EXCEPTION: 	EsiaDbException si une erreur de connexion survient
	*/
	static function getImage($dbconnection)
	{
		$query = $dbconnection->prepare('SELECT image_id, image_name, image_description, image_path_ok, image_path_warning,image_path_critical, image_path_unknown
				FROM esia_localization_images;');
		$list_img=array();
		$query = Model_esia::executeStatement($query);
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$list_img[$row->image_id]=array(
					'id'=>$row->image_id,
					'name'=>$row->image_name,
					'description'=>$row->image_description,
					'img_ok'=>$row->image_path_ok,
					'img_warning'=>$row->image_path_warning,
					'img_critical'=>$row->image_path_critical,
					'img_unknown'=>$row->image_path_unknown);
		}
		return $list_img;
	}
	/*@NAME: getNodesLocalizationByFloor
	 *@IN: $dbconnection l'objet de connexion PDO
	*	$id= id de l'étage
	*@OUT: un tableau associatif les clés sont:
	* 	$list[<node_id>]['id'] id du noeud
	*	$list[<node_id>]['name'] nom du noeud
	*	$list[<node_id>]['type'] type du noeud
	*	$list[<node_id>]['description'] description du noeud
	*	$list[<node_id>]['x'] position X sur l'image
	*	$list[<node_id>]['y'] position Y sur l'image
	*	$list[<node_id>]['height'] hauteur de l'image
	*	$list[<node_id>]['width'] largeur de l'image
	*	$list[<node_id>]['image_id'] id de l'image
	*@TREAT: renvoi les informations sur le noeud ainsi que sa position/image dans le plan
	*@EXCEPTION: 	EsiaDbException si une erreur de connexion survient
	*/
	static function getNodesLocalizationByFloor($dbconnection,$id)
	{
		$query = $dbconnection->prepare('SELECT device_id,device_name,device_type,device_description,localization_x,
				localization_y,localization_width,localization_height,localization_image_id
				FROM esia_localization
				JOIN esia_device ON device_id= localization_device_id
				WHERE localization_floor_id=?;');
		$list = array();
		$query = Model_esia::executeStatement($query,array($id));
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$list[$row->device_id] = array('id'=>$row->device_id,
					'name'=>$row->device_name,
					'type'=>$row->device_type,
					'description'=>$row->device_description,
					'x'=>$row->localization_x,
					'y'=>$row->localization_y,
					'height'=>$row->localization_height,
					'width'=>$row->localization_width,
					'image_id'=>$row->localization_image_id
			);

		}
		return $list;
	}
	/*@NAME: getContainerLocalizationByFloor
	 *@IN: $dbconnection l'objet de connexion PDO
	*	$id= id de l'étage
	*@OUT: un tableau associatif les clés sont:
	* 	$list[<container_id>]['id'] id du container
	*	$list[<container_id>]['name'] nom du container
	*	$list[<container_id>]['type'] type du container
	*	$list[<container_id>]['description'] description du container
	*	$list[<container_id>]['x'] position X sur l'image
	*	$list[<container_id>]['y'] position Y sur l'image
	*	$list[<container_id>]['height'] hauteur de l'image
	*	$list[<container_id>]['width'] largeur de l'image
	*	$list[<container_id>]['image_id'] id de l'image
	*@TREAT: renvoi les informations sur les containers ainsi que sa position/image dans le plan
	*@EXCEPTION: 	EsiaDbException si une erreur de connexion survient
	*/
	static function getContainerLocalizationByFloor($dbconnection,$id)
	{
		$query = $dbconnection->prepare('SELECT container_id,container_name,container_type_name,container_description,localization_x,
				localization_y,localization_width,localization_height,localization_image_id
				FROM esia_localization
				JOIN esia_container ON container_id= localization_container_id
				JOIN esia_container_type ON container_type_id= container_type
				WHERE localization_floor_id=?;');
		$list = array();
		$query = Model_esia::executeStatement($query,array($id));
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$list[$row->container_id] = array('id'=>$row->container_id,
					'name'=>$row->container_name,
					'type'=>$row->container_type_name,
					'description'=>$row->container_description,
					'x'=>$row->localization_x,
					'y'=>$row->localization_y,
					'height'=>$row->localization_height,
					'width'=>$row->localization_width,
					'image_id'=>$row->localization_image_id
			);
	
		}
		return $list;
	}
	/*@NAME: getGroupNode
	 *@IN: $dbconnection l'objet de connexion PDO
	*	$u (optionnel) le nom ou l'id de l'utilisateur
	*@OUT: un tableau contenant les noms des groupes de noeud
	*@TREAT: retourne le nom de groupe auquelle l'utilisateur à le droits sur au moins un noeud (droit: device_view) si aucun utilisateur n'est spécifié il retourne tout
	*	les noms de groupes de noeud
	*@EXCEPTION: 	EsiaDbException si une erreur survient
	*/
	static function getGroupNode($dbconnection,$u=null)
	{
		return Model_esia::getGroupDevice($dbconnection,$u);
	}
	/*@NAME: getContainerType
	 *@IN: $dbconnection l'objet de connexion PDO
	*	$id (optionnel) le nom ou l'id du type de container
	*@OUT: un tableau contenant 
	*	array[<container_type_id]['id'] = identifiant du type du container
	*	array[<container_type_id]['name'] = nom du type du container
	*@TREAT: retourne les différents type de container de la base de donnée
	*/
	static function getContainerType($dbconnection,$id=null)
	{
		$list_type_container=array();
		if(isset($id)&&!empty($id))
		{
			if(is_numeric($id))
			{
				$query = $dbconnection->prepare('SELECT container_type_id,container_type_name
						FROM esia_container_type WHERE container_type_id=? ORDER BY container_type_name;');
			}
			else
			{
				$query = $dbconnection->prepare('SELECT container_type_id,container_type_name
						FROM esia_container_type WHERE container_type_name=? ORDER BY container_type_name;');
			}
			parent::executeStatement($query,array($id));
		}
		else
		{
			$query = $dbconnection->prepare('SELECT container_type_id,container_type_name
					FROM esia_container_type ORDER BY container_type_name;');
			parent::executeStatement($query);
		}
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$list_type_container[$row->container_type_id]=array('id'=>$row->container_type_id,
					'name'=>$row->container_type_name);
		}
		return $list_type_container;
	}
	/*@NAME: getContainerType
	 *@IN: $dbconnection l'objet de connexion PDO
	*	$id (optionnel) l'id du container
	*@OUT: un tableau contenant
	*	array[<container_id]['id'] = identifiant du container
	*	array[<container_id]['name'] = nom du container
	*	array[<container_id]['description'] = description du container
	*	array[<container_id]['image'] = lien vers la photo
	* 	array[<container_id]['type_id'] = id du type de container
	* 	array[<container_id]['type_name'] = nom du type de container
	* 	array[<container_id]['nodes'][<node_id>]['id'] = id du noeud
	* 	array[<container_id]['nodes'][<node_id>]['name'] = nom du noeud
	*  	array[<container_id]['nodes'][<node_id>]['type'] = type noeud
	* 	array[<container_id]['nodes'][<node_id>]['group'] = groupe du noeud
	*@TREAT: retourne les différents containers dans la base de donnée
	*/
	static function getContainers($dbconnection,$id=null)
	{
		$list_container=array();
		if(isset($id)&&!empty($id))
		{
			if(is_numeric($id))
			{
				$query = $dbconnection->prepare('SELECT container_id,container_name,container_description,container_photo,container_type_id,container_type_name
					FROM esia_container
					JOIN esia_container_type ON container_type=container_type_id
					WHERE container_id=? ORDER BY container_name;');
			}
			parent::executeStatement($query,array($id));
		}
		else
		{
			$query = $dbconnection->prepare('SELECT container_id,container_name,container_description,container_photo,container_type_id,container_type_name
					FROM esia_container
					JOIN esia_container_type ON container_type=container_type_id ORDER BY container_name;');
			parent::executeStatement($query);
		}
		$q = $dbconnection->prepare('SELECT device_id,device_name,device_type,device_group_name
				FROM esia_device
				JOIN esia_device_stored_container ON stored_device_id = device_id
				wHERE stored_container_id=? ORDER BY device_name');
	
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$list_node=array();
			parent::executeStatement($q,array($row->container_id));
			while($node=$q->fetch(PDO::FETCH_OBJ))
			{
				$list_node[$node->device_id]=array('id'=>$node->device_id,
													'name'=>$node->device_name,
													'type'=>$node->device_type,
													'group'=>$node->device_group_name);
			}
			$list_container[$row->container_id]=array('id'=>$row->container_id,
														'name'=>$row->container_name,
														'description'=>$row->container_description,
														'image'=>$row->container_photo,
														'type_id'=>$row->container_type_id,
														'type_name'=>$row->container_type_name,
														'nodes'=>$list_node);
		}
		return $list_container;
	}
	/*@NAME: UpdateZone
	 *@IN: $dbconnection l'objet de connexion PDO
	*	$lat coordonnée de la latitude
	*@OUT: /
	*@TREAT: met à jour les coordonnées latitude du centre de la carte
	*@EXCEPTION: 	EsiaDbException si une erreur de connexion survient
	*/
	static function UpdateZone($dbconnection,$id,$name,$abbreviation,$latitude=null,$longitude=null)
	{
		if(!empty($latitude)&&!empty($longitude))
		{
			$sql='UPDATE esia_zone SET zone_name=:name , zone_abreviation=:abr ';
			$sql.=', zone_latitude = '.MapModel::ArrayToLatLng($latitude);
			$sql.=', zone_longitude = '.MapModel::ArrayToLatLng($longitude);
			$sql.='WHERE zone_id=:id ;';
		}
		else
		{
			$sql='UPDATE esia_zone SET zone_name=:name , zone_abreviation=:abr WHERE zone_id=:id ;';
		}
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,array(':id'=>$id,':name'=>$name,':abr'=>$abbreviation));
		return $id;
	}
	/*@NAME: UpdateBuilding
	 *@IN: $dbconnection l'objet de connexion PDO
	*	$lat coordonnée de la latitude
	*@OUT: /
	*@TREAT: met à jour les coordonnées latitude du centre de la carte
	*@EXCEPTION: 	EsiaDbException si une erreur de connexion survient
	*/
	static function UpdateBuilding($dbconnection,$id,$name,$abbreviation,$description,$zoneid,$latitude=null,$longitude=null)
	{
		$query = $dbconnection->prepare('UPDATE esia_building SET building_name=:name,
				building_abreviation=:abr,
				building_zone_id=:zone_id,
				building_latitude=:latitude,
				building_longitude=:longitude,
				building_description=:description
				WHERE building_id = :id');
		if(isset($latitude)&&isset($longitude))
		{
			$query = Model_esia::executeStatement($query,array(':name'=>$name,
					':abr'=>$abbreviation,
					':zone_id'=>$zoneid,
					':latitude'=>$latitude,
					':longitude'=>$longitude,
					':description'=>$description,':id'=>$id));
		}
		else
		{
			$query = Model_esia::executeStatement($query,array(':name'=>$name,
					':abr'=>$abbreviation,
					':zone_id'=>$zoneid,
					':latitude'=>$longitude,
					':longitude'=>$latitude,
					':description'=>$description,':id'=>$id));
		}
		return $id;
	}
	/*@NAME: UpdateCenterLatitude
	 *@IN: $dbconnection l'objet de connexion PDO
	*	$lat coordonnée de la latitude
	*@OUT: /
	*@TREAT: met à jour les coordonnées latitude du centre de la carte
	*@EXCEPTION: 	EsiaDbException si une erreur de connexion survient
	*/
	static function UpdateCenterLatitude($dbconnection,$lat)
	{
		$query = $dbconnection->prepare("UPDATE esia_configuration SET config_value=? WHERE config_key='MAP_CENTER_LATITUDE';");
		$query = Model_esia::executeStatement($query,array($lat));
	}
	/*@NAME: UpdateCenterLongitude
	 *@IN: $dbconnection l'objet de connexion PDO
	*	$lng coordonnée de la longitude
	*@OUT: /
	*@TREAT: met à jour les coordonnées longitude du centre de la carte
	*@EXCEPTION: 	EsiaDbException si une erreur de connexion survient
	*/
	static function UpdateCenterLongitude($dbconnection,$lng)
	{
		$query = $dbconnection->prepare("UPDATE esia_configuration SET config_value=? WHERE config_key='MAP_CENTER_LONGITUDE';");
		$query = Model_esia::executeStatement($query,array($lng));
	}
	/*@NAME: UpdateMapType
	 *@IN: $dbconnection l'objet de connexion PDO
	*	$type type de carte
	*@OUT: /
	*@TREAT: met à jour le type de carte (voir doc google map: hybrid,roamap,etc)
	*@EXCEPTION: 	EsiaDbException si une erreur de connexion survient
	*/
	static function UpdateMapType($dbconnection,$type)
	{
		$query = $dbconnection->prepare("UPDATE esia_configuration SET config_value=? WHERE config_key='MAP_TYPE';");
		$query = Model_esia::executeStatement($query,array($type));
	}
	/*@NAME: UpdateZoom
	 *@IN: $dbconnection l'objet de connexion PDO
	*	$zoom Niveau de zoom par défaut
	*@OUT: /
	*@TREAT: met à jour le niveau de zoom par défaut
	*@EXCEPTION: 	EsiaDbException si une erreur de connexion survient
	*/
	static function UpdateZoom($dbconnection,$zoom)
	{
		$query = $dbconnection->prepare("UPDATE esia_configuration SET config_value=? WHERE config_key='MAP_ZOOM';");
		$query = Model_esia::executeStatement($query,array($zoom));
	}
	/*@NAME: UpdateZoomChanged
	 *@IN: $dbconnection l'objet de connexion PDO
	*	$zoom Niveau de zoom pour le passage zone > batiment
	*@OUT: /
	*@TREAT: met à jour le niveau de zoom pour le passage zone > batiment
	*@EXCEPTION: 	EsiaDbException si une erreur de connexion survient
	*/
	static function UpdateZoomChanged($dbconnection,$zoom)
	{
		$query = $dbconnection->prepare("UPDATE esia_configuration SET config_value=? WHERE config_key='MAP_ZOOM_CHANGED';");
		$query = Model_esia::executeStatement($query,array($zoom));
	}
	/*@NAME: UpdateContainer
	 *@IN: $dbconnection l'objet de connexion PDO
	 *	$id du container à modifier
	*	$name nom du container
	*	$description  description du container
	*	$type type du container (id de la table esia_container_type
	*	$image lien vers l'image
	*	$listnode array contenant les id des noeuds relié à ce container
	*@OUT: /
	*@TREAT: met à jour un container dans la base de donnée
	*@EXCEPTION: 	EsiaDbException si une erreur de connexion survient
	*/
	static function UpdateContainer($dbconnection,$id,$name,$description,$type,$image,$listnode)
	{
		$param = array(':name'=>$name,
					':description'=>$description,
					':type'=>$type,':id'=>$id);
		//on vérifie si il y a une nouvelle image
		if(isset($image)&&!empty($image))
		{
			$query = $dbconnection->prepare('UPDATE esia_container SET container_name=:name,container_description=:description,container_type=:type,container_photo=:photo WHERE container_id=:id;');
			$param[':photo']=$image;
		}
		else
		{
			$query = $dbconnection->prepare('UPDATE esia_container SET container_name=:name,container_description=:description,container_type=:type WHERE container_id=:id;');
		}
		parent::executeStatement($query,$param);
		//on supprime les noeuds relié à ce container
		$query = $dbconnection->prepare('DELETE FROM esia_device_stored_container WHERE stored_container_id=?;');
		parent::executeStatement($query,array($id));
		
		//on lie les noeuds à ce container
		$query = $dbconnection->prepare('INSERT INTO esia_device_stored_container(
				stored_device_id, stored_container_id)
				VALUES (:nodeid, :containerid);');
		//Model_esia::printHTMLarray($listnode);
		foreach($listnode as $node)
		{
			parent::executeStatement($query,array(':nodeid'=>$node,':containerid'=>$id));
		}
	}
	/*@NAME: DeleteBuilding
	 *@IN: $dbconnection l'objet de connexion PDO
	*	$id identifiant du bâtiment
	*@OUT: /
	*@TREAT: Supprime un bâtiment, ainsi que tout ses étages
	*@EXCEPTION: 	EsiaDbException si une erreur de connexion survient
	*/
	static function DeleteBuilding($dbconnection,$id)
	{
		$query = $dbconnection->prepare('SELECT floor_id
				FROM esia_floor
				WHERE floor_building_id=?');
		$q_delete = $dbconnection->prepare('DELETE FROM esia_localization WHERE localization_floor_id=?;');
		$query = Model_esia::executeStatement($query,array($id));
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$q_delete = Model_esia::executeStatement($q_delete,array($row->floor_id));
		}
		$query = $dbconnection->prepare('DELETE FROM esia_floor WHERE floor_building_id=?;');
		$query = Model_esia::executeStatement($query,array($id));
		$query = $dbconnection->prepare('DELETE FROM esia_building WHERE building_id=?;');
		$query = Model_esia::executeStatement($query,array($id));
	}
	/*@NAME: DeleteZone
	 *@IN: $dbconnection l'objet de connexion PDO
	*	$id identifiant de la zone
	*@OUT: /
	*@TREAT: Supprime une zone, ainsi que tout ses étages bâtiments et étages
	*@EXCEPTION: 	EsiaDbException si une erreur de connexion survient
	*/
	static function DeleteZone($dbconnection,$id)
	{
		$query = $dbconnection->prepare('SELECT building_id
				FROM esia_building
				WHERE building_zone_id=?');
		$query = Model_esia::executeStatement($query,array($id));
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			MapModel::DeleteBuilding($dbconnection,$row->building_id);
		}
		$query = $dbconnection->prepare('DELETE FROM esia_zone WHERE zone_id=?;');
		$query = Model_esia::executeStatement($query,array($id));
	}
	/*@NAME: DeleteFloor
	 *@IN: $dbconnection l'objet de connexion PDO
	*	$id identifiant du plan
	*@OUT: /
	*@TREAT: Supprime un un plan ainsi que toute ses informations
	*@EXCEPTION: 	EsiaDbException si une erreur de connexion survient
	*/
	static function DeleteFloor($dbconnection,$id)
	{
		$query = $dbconnection->prepare('SELECT floor_map
				FROM esia_floor
				WHERE floor_id=?');
		$query = Model_esia::executeStatement($query,array($id));
		$row=$query->fetch(PDO::FETCH_OBJ);
		if(!empty($row->floor_map))
		{
			if(@unlink($row->floor_map)==false)
				throw new EsiaDbException('Can\'t delete '.$row->floor_map);
		}
		$query = $dbconnection->prepare('DELETE FROM esia_localization WHERE localization_floor_id=?;');
		$query = Model_esia::executeStatement($query,array($id));
		$query = $dbconnection->prepare('DELETE FROM esia_floor WHERE floor_id=?;');
		$query = Model_esia::executeStatement($query,array($id));
	}
	/*@NAME: DeleteAllObjectInMap
	 *@IN: $dbconnection l'objet de connexion PDO
	*	$id identifiant du plan
	*@OUT: /
	*@TREAT: Supprime un un plan ainsi que toute ses informations
	*@EXCEPTION: 	EsiaDbException si une erreur de connexion survient
	*/
	static function DeleteAllObjectInMap($dbconnection,$id)
	{
		$query = $dbconnection->prepare('DELETE FROM esia_localization WHERE localization_floor_id =? ');
		$query = Model_esia::executeStatement($query,array($id));
	}
	/*@NAME: CreateZone
	 *@IN: $dbconnection l'objet de connexion PDO
	*	$name nom de la zone
	*	$abbreviation abréviation de la zone
	*	$latitude array contenant les latitudes
	*	$longitude array contenant les longitude
	*@OUT: id de la nouvelle zone
	*@TREAT: Insère une nouvelle zone dans la BD
	*@EXCEPTION: 	EsiaDbException si une erreur de connexion survient
	*/
	static function CreateZone($dbconnection,$name,$abbreviation,$latitude=null,$longitude=null)
	{
		if(!empty($latitude)&&!empty($longitude))
		{
			$sql='INSERT INTO esia_zone (zone_id, zone_name, zone_abreviation,zone_latitude,zone_longitude) VALUES(DEFAULT,:name,:abr,';
			$i=0;
			$sql.=MapModel::ArrayToLatLng($latitude);
			$sql.=',';
			$sql.=MapModel::ArrayToLatLng($longitude);
			$sql.=') RETURNING zone_id;';
		}
		else
		{
			$sql='INSERT INTO esia_zone (zone_id, zone_name, zone_abreviation) VALUES(DEFAULT,:name,:abr) RETURNING zone_id;';
		}
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,array(':name'=>$name,':abr'=>$abbreviation));
		$row=$query->fetch(PDO::FETCH_OBJ);
		return $row->zone_id;
	}
	/*@NAME: CreateBuilding
	 *@IN: $dbconnection l'objet de connexion PDO
	*	$zoneid identifiant de la zone
	*	$name nom  du bâtiment
	*	$abbreviation abréviation du bâtiment
	*	$description description du bâtiment
	*	$latitude latitude  du bâtiment
	*	$longitude longitude  du bâtiment
	*@OUT: id du bâtiment
	*@TREAT: Insère un nouveau bâtiment dans la BD
	*@EXCEPTION: 	EsiaDbException si une erreur de connexion survient
	*/
	static function CreateBuilding($dbconnection,$zoneid,$name,$abbreviation,$description,$latitude=null,$longitude=null)
	{
		$error='';
		if(isset($latitude)&&isset($longitude))
		{
			$query = $dbconnection->prepare('INSERT INTO esia_building (building_id,building_name,building_abreviation,building_zone_id,building_latitude,building_longitude,building_description)
					VALUES (default,:name,:abr,:zone_id,:latitude,:longitude,:description) RETURNING building_id;');
			$query = Model_esia::executeStatement($query,array(':name'=>$name,
					':abr'=>$abbreviation,
					':zone_id'=>$zoneid,
					':latitude'=>$latitude,
					':longitude'=>$longitude,
					':description'=>$description
			));
		}
		else
		{
			$query = $dbconnection->prepare('INSERT INTO esia_building (building_id,building_name,building_abreviation,building_zone_id,building_description)
					VALUES (default,:name,:abr,:zone_id,:description) RETURNING building_id;');
			$query = Model_esia::executeStatement($query,array(':name'=>$name,
					':abr'=>$abbreviation,
					':zone_id'=>$zoneid,
					':description'=>$description
			));
		}
		$row=$query->fetch(PDO::FETCH_OBJ);
		return $row->building_id;

	}
	/*@NAME: CreateContainer
	 *@IN: $dbconnection l'objet de connexion PDO
	*	$name nom du container
	*	$description  description du container
	*	$type type du container (id de la table esia_container_type
	*	$image lien vers l'image
	*	$listnode array contenant les id des noeuds relié à ce container
	*@OUT: id du nouveau container
	*@TREAT: Crée un container dans la base de donnée
	*@EXCEPTION: 	EsiaDbException si une erreur de connexion survient
	*/
	static function CreateContainer($dbconnection,$name,$description,$type,$image,$listnode)
	{
		$query = $dbconnection->prepare('INSERT INTO esia_container (container_id,container_name,container_description,container_photo,container_type )
				VALUES (DEFAULT,:name,:description,:photo,:type) RETURNING container_id; ');
		parent::executeStatement($query,array(':name'=>$name,
											':description'=>$description,
											':photo'=>$image,
											':type'=>$type ));
		$row=$query->fetch(PDO::FETCH_OBJ);
		$query = $dbconnection->prepare('INSERT INTO esia_device_stored_container(
				stored_device_id, stored_container_id)
				VALUES (:nodeid, :containerid);');
		
		foreach($listnode as $node)
		{
			parent::executeStatement($query,array(':nodeid'=>$node,':containerid'=>$row->container_id));
		}
		return $row->container_id;
	}
	/*@NAME: insertInMap
	 *@IN: $dbconnection l'objet de connexion PDO
	 *	$floor identifiant de l'étage
	*	$nodeid identifiant du noeud
	*	$imageid id de l'image
	*	$x nombre de pixel sur l'axe des x
	*	$y nombre de pixel sur l'axe des y
	*	$width largeur de l'image en px
	*	$height hauteur de l'image en px
	*	$node(optionnel) le type ajouter si c'est un 'node' ou un 'container' 
	*@OUT: /
	*@TREAT: ajoute l'image dans le plan
	*@EXCEPTION: 	EsiaDbException si une erreur de connexion survient
	*/
	static function insertInMap($dbconnection,$floor,$nodeid,$imageid,$x,$y,$width,$height,$type='node')
	{
		if($type=='container')
		{
			$query = $dbconnection->prepare('DELETE FROM esia_localization WHERE localization_container_id =? AND localization_floor_id =? ');
			$query = Model_esia::executeStatement($query,array($nodeid,$floor));
			
			$query = $dbconnection->prepare('INSERT INTO esia_localization(
					localization_id, localization_floor_id, localization_container_id,
					localization_x, localization_y, localization_width,
					localization_height, localization_image_id)
					VALUES (DEFAULT, :floor, :container,:x, :y, :width, :height,:img);');
			$query = Model_esia::executeStatement($query,array(':floor'=>$floor,
					':container'=>$nodeid,
					':x'=>$x,
					':y'=>$y,
					':width'=>$width,
					':height'=>$height,
					':img'=>$imageid));			
		}
		else
		{
			$query = $dbconnection->prepare('DELETE FROM esia_localization WHERE localization_device_id =? AND localization_floor_id =? ');
			$query = Model_esia::executeStatement($query,array($nodeid,$floor));
			
			$query = $dbconnection->prepare('INSERT INTO esia_localization(
					localization_id, localization_floor_id, localization_device_id,
					localization_x, localization_y, localization_width,
					localization_height, localization_image_id)
					VALUES (DEFAULT, :floor, :node,:x, :y, :width, :height,:img);');
			$query = Model_esia::executeStatement($query,array(':floor'=>$floor,
					':node'=>$nodeid,
					':x'=>$x,
					':y'=>$y,
					':width'=>$width,
					':height'=>$height,
					':img'=>$imageid));
		}
	}
	/*@NAME: PrepareCreateFloor
	 *@IN: $dbconnection l'objet de connexion PDO
	*	$id (optionnel) identifiant du bâtiment
	*@OUT: PDOStatement
	*@TREAT: prépare la requête SQL de création d'étage
	*@EXCEPTION: 	EsiaDbException si une erreur de connexion survient
	*/
	static function PrepareCreateFloor($dbconnection,$id=null)
	{
		if(isset($id)&&is_numeric($id))
		{
			$sql='INSERT INTO esia_floor(floor_id, floor_name, floor_map, floor_building_id)  VALUES (default,:name,:map,'.$id.');';
		}
		else
		{
			$sql='INSERT INTO esia_floor(floor_id, floor_name, floor_map, floor_building_id)  VALUES (default,:name,:map,:building_id);';
		}
		return $dbconnection->prepare($sql);
	}
	/*@NAME: PrepareGetFloorError
	 *@IN: $dbconnection l'objet de connexion PDO
	*	$userid id de l'utilisateur
	*@OUT: PDOStatement
	*@TREAT: prépare la requête pour récupérer l'état d'un étage + ses erreurs
	*@EXCEPTION: 	EsiaDbException si une erreur de connexion survient
	*/
	static function PrepareGetFloorError($dbconnection,$userid)
	{
		$sql = "SELECT error_type,error_name,happen_message,device_id,device_name,device_type,service_name,to_char(happen_timestamp,'DD/MM/YYYY HH24:MI:SS') as  timestamp
		FROM esia_localization
		JOIN esia_device ON localization_device_id = device_id
		JOIN esia_error_happen ON happen_device_id = device_id
		JOIN esia_errors ON happen_error_id = error_id
		JOIN esia_group_device_access ON  device_id = access_device_id
		JOIN esia_groups ON access_group_id = group_id
		JOIN esia_memberof ON member_group_id = group_id
		LEFT JOIN esia_services ON happen_service_id = service_id
		WHERE happen_acquitte =0 AND localization_floor_id=:id AND member_user_id = :user
		UNION
		SELECT error_type,error_name,happen_message,device_id,device_name,device_type,service_name,to_char(happen_timestamp,'DD/MM/YYYY HH24:MI:SS') as  timestamp
		FROM  esia_localization
		JOIN esia_container ON container_id = localization_container_id
		JOIN esia_device_stored_container ON stored_container_id = container_id
		JOIN esia_device ON stored_device_id = device_id
		JOIN esia_error_happen ON happen_device_id = device_id
		JOIN esia_errors ON happen_error_id = error_id
		JOIN esia_group_device_access ON  device_id = access_device_id
		JOIN esia_groups ON access_group_id = group_id
		JOIN esia_memberof ON member_group_id = group_id
		LEFT JOIN esia_services ON happen_service_id = service_id
		WHERE happen_acquitte =0 AND localization_floor_id=:id AND member_user_id = :user;";
		$query = $dbconnection->prepare($sql);
		$query->bindValue(':user', $userid, PDO::PARAM_INT);
		return $query;
	}
	/*@NAME: ExecuteGetFloorState
	 *@IN: $dbconnection l'objet de connexion PDO
	*	$query Pdostatement contenant la requete préparée par la fonction  MapModel::PrepareGetFloorError
	*	$id id de l'étage
	*@OUT: un tableau associatif
	*	$floor['id'] id de l'étage,
	*	$floor['state'] état de l'étage (ok,warning,critical,unknown)
	*	$floor['error']	tableau contenant toute les erreurs
	*	$floor['error'][]['type'] type de l'erreur
	*      $floor['error'][]['name'] nom de l'erreur
	*      $floor['error'][]['message'] message
	*      $floor['error'][]['node'] nom du noeud
	*      $floor['error'][]['node_type'] type du noeud
	*   	$floor['error'][]['control_page'] page de controle du noeud
	*      $floor['error'][]['service'] nom du service
	*      $floor['error'][]['time'] date à laquelle c'est produite l'erreur
	*@TREAT: Exécute la requête 'PrepareGetFloorError' pour récupérer l'état d'un étage + ses erreurs
	*@EXCEPTION: 	EsiaDbException si une erreur de connexion survient
	*/
	static function ExecuteGetFloorState($dbconnection,$query,$id)
	{
		$query->bindValue(':id',$id, PDO::PARAM_INT);
		$query = Model_esia::executeStatement($query);
		$devType = Model_esia::getDeviceType($dbconnection);
		$errors= array();
		$state = 'OK';
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			//on test le type d'erreur et on le compare à l'état actuelle de l'étage
			if($row->error_type=='critical')
			{
				$state='critical';
			}
			elseif($row->error_type=='warning'&&$state!='critical')
			{
				$state='warning';
			}
			elseif($row->error_type=='unknown'&&$state!='critical'&&$state!='warning')
			{
				$state='unknown';
			}
			//pour la redirection on prend la page de controle du noeud suivant sont type
			$page_control='/node_view.php?name='.$row->id;
			//page de contrôle
			if(!empty($devType[$row->device_type]['web_plugin_control_page']))
			{
				$page_control = Model_esia::replacePageControlVariable($devType[$row->device_type]['web_plugin_control_page'], $row);
				$page_control = $devType[$row->device_type]['web_plugin_apache_path'].$page_control;
			}
			$errors[]=array('type'=>$row->error_type,
					'name'=>$row->error_name,
					'message'=>$row->happen_message,
					'node'=>$row->device_name,
					'node_type'=>$row->device_type,
					'control_page'=>$page_control,
					'service'=>$row->service_name,
					'time'=>$row->timestamp,
			);
		}
		//tableau des étages
		$floor=array('id'=>$id,
				'state'=>$state,
				'error'=>$errors);
		return $floor;
	}
	/*@NAME: LatLngToArray
	 *@IN: $arr un array au format postgresql
	*@OUT: un array au format PHP
	*@TREAT: transforme un tableau SQL en tableau PHP
	*@EXCEPTION: /
	*/
	private static function LatLngToArray($arr)
	{
		if(!empty($arr))
		{
			return explode(',', trim($arr, '{}'));

		}
		else
		{
			return array();
		}
	}
	/*@NAME: ArrayToLatLng
	 *@IN: $arr un array au format php
	*@OUT: un array au format SQL (string)
	*@TREAT: transforme un tableau PHP en tableau SQL
	*@EXCEPTION: /
	*/
	private static function ArrayToLatLng($arr)
	{
		if(is_array($arr))
		{
			$i=0;
			$sql = '\'{';
			foreach ($arr as $l)
			{
				if($i!=0)
					$sql.=',';
				$sql.='"'.$l.'"';
				$i++;
			}
			$sql.='}\'';
			return $sql;
		}
		else
		{
			return '';
		}
	}
}
?>
