<?php
/**
 * @name RemoteServer
* @description Classe de base gérant les serveurs du plugin "Server"
* @author Nicolas Biersart <n.biersart@esia-network.com>
* @contributor
* @copyright Nicolas Biersart 2012
* @license
* @esia 1.00
* @version 1.0.0
* @date 07/11/2012
*/
include_once SNMP_CLASS;
include_once LIB_SNMP;
include_once DEVICE_CLASS;
include_once GESA_CLASS;
class RemoteServer extends RemoteDevice
{
	protected $memorySize;
	protected $cpuInfos;
	protected $processes;
	protected $hostSystemInfo;
	protected $storage;
	protected $interfaces;
	protected $ipv4addr;
	protected $tcpStats;
	protected $tcpTable;
	protected $udpStats;
	protected $udpTable;
	protected $ipStats;
	
	/*@NAME: __construct
	 *@IN: $dev un objet Device
	*@OUT: /
	*@TREAT: permet de construire un objet server à partir d'un objet "Device"
	*@EXCEPTION: 	/
	*/
	public function __construct($dev=null,$gesa=NULL)
	{
		if(get_class($dev)=='Device')
		{
			parent::constructDevice($dev);
		}
		elseif(get_class($dev)=='RemoteDevice')
		{
			parent::constructDevice($dev);
			$this->gesa = $gesa;
		}
		else
		{
			parent::__construct();
		}
	}
	/*@NAME: getCpuDescr
	 *@IN: $index=0 optionnel index du CPU
	*@OUT: Un tableau associatif ('index'=> index du cpu, 'description'=> description du cpu)
	*@TREAT: effectue des requêtes SNMP afin de retourner la description du processeur
	*	 Si index est égale à 0, la fonction effectuera un WALK et retournera la charge de tout les CPU dans le cas contraire, il retournera seulement la charge du
	*	processeur demandé
	*@EXCEPTION: 	DeviceSNMPException si une erreur SNMP survient
	*/
	public function getCpuDescr($index=0)
	{
		$cpu = array();
		if($index==0)
		{
			$l =parent::snmpwalkoid(".1.3.6.1.2.1.25.3.2.1.2");
			for (reset($l); $i = key($l); next($l))
			{
				if(preg_match('#HOST-RESOURCES-TYPES::hrDeviceProcessor|iso.3.6.1.2.1.25.3.1.3#',$l[$i]))
				{
					if(preg_match('#\.([0-9]*)$#',$i,$regs))
					{
						$load =parent::snmpget(".1.3.6.1.2.1.25.3.2.1.3.".$regs[1]);
						$cpu[]=array('index'=>$regs[1],'description'=>formatString($load));
					}
				}
			}
		}
		else
		{
			$load =parent::snmpget(".1.3.6.1.2.1.25.3.2.1.3.".$index);
			$cpu[]=array('index'=>$index,'description'=>formatString($load));
		}
		return $cpu;
	}
	/*@NAME: getCpuLoad
	 *@IN: $index=0 optionnel index du CPU
	*@OUT: Un tableau associatif ('index'=> index du cpu, 'load'=> charge du CPU)
	*@TREAT: effectue des requêtes SNMP afin de retourner la charge CPU du serveur de la dernière minute
	*	 Si index est égale à 0, la fonction effectuera un WALK et retournera la charge de tout les CPU dans le cas contraire, il retournera seulement la charge du
	*	processeur demandé
	*@EXCEPTION: 	DeviceSNMPException si une erreur SNMP survient
	*/
	public function getCpuLoad($index=0)
	{
		$cpu = array();
		if($index==0)
		{
			$l =parent::snmpwalkoid(".1.3.6.1.2.1.25.3.2.1.2");
			for (reset($l); $i = key($l); next($l))
			{
				if(preg_match('#HOST-RESOURCES-TYPES::hrDeviceProcessor|iso.3.6.1.2.1.25.3.1.3#',$l[$i]))
				{
					if(preg_match('#\.([0-9]*)$#',$i,$regs))
					{
						$load =parent::snmpget(".1.3.6.1.2.1.25.3.3.1.2.".$regs[1]);
						$cpu[]=array('index'=>$regs[1],'load'=>formatInteger($load));
					}
				}
			}
		}
		else
		{
			$load =parent::snmpget(".1.3.6.1.2.1.25.3.3.1.2.".$index);
			$cpu[]=array('index'=>$index,'load'=>formatInteger($load));
		}
		return $cpu;
	}
	/*@NAME: getMemorySize
	 *@IN: /
	*@OUT: un integer contenant la taille de la mémoire en octet
	*@TREAT: effectue des requêtes SNMP afin de retourner la quantité de mémoire ram disponnible
	*@EXCEPTION: 	DeviceSNMPException si une erreur SNMP survient
	*/
	public function getMemorySize()
	{
		if($this->memorySize==NULL)
		{
			$objectid = $this->getSysObjectId();
			$srv_logo=null;
			foreach($objectid as $line)
			{
				if(preg_match('#1\.3\.6\.1\.4\.1\.8072\..*#', $line[1]))
				{
					$this->memorySize=$this->getGESA()->check_nrpe($this,"check_snmp_mem",$this->getSNMPCommunity()."  80,10 90,30")["data"][0];
				}
				elseif(preg_match('#1\.3\.6\.1\.4\.1\.311\..*#', $line[1]))
				{
					$this->memorySize=$this->getGESA()->check_nrpe($this,"check_snmp_storage",$this->getSNMPCommunity().' ^Physical Memory$ 80 90')["data"][0];
				}
			}
		}
		return $this->memorySize;
	}
	
	public function getCpuInfos(){
		
		if($this->cpuInfos==NULL) $this->cpuInfos=GESA::data_to_arrays($this->nrpe_all("get_snmp_cpu"));
		return $this->cpuInfos;
	}
	
	public function getProcesses(){
		if($this->processes==NULL) $this->processes=GESA::data_to_arrays($this->nrpe_all("get_snmp_process"));
		return $this->processes;
	}
	
	public function getHostSystemInfo(){
		if($this->hostSystemInfo==NULL) $this->hostSystemInfo=GESA::data_to_arrays($this->nrpe_all("get_snmp_host_hrSystem"));
		return $this->hostSystemInfo;
	}
	
	public function getStorage(){
		if($this->storage==NULL) $this->storage=GESA::data_to_arrays($this->nrpe_all("get_snmp_storage"));
		return $this->storage;
	}
	
	public function getInterfaces(){
		if($this->interfaces==NULL) $this->interfaces=GESA::data_to_arrays($this->nrpe_all("get_snmp_interface"));
		return $this->interfaces;
	}
	
	public function getIPv4Adresses(){
		if($this->ipv4addr==NULL) $this->ipv4addr=GESA::data_to_arrays($this->nrpe_all("get_snmp_ipaddr"));
		return $this->ipv4addr;
	}
	
	public function getTCPStats(){
		if($this->tcpStats==NULL) $this->tcpStats=GESA::data_to_arrays($this->nrpe_all("get_snmp_tcpstats"));
		return $this->tcpStats;
	}
	public function getTCPTable(){
		if($this->tcpTable==NULL) $this->tcpTable=GESA::data_to_arrays($this->nrpe_all("get_snmp_tcpconn"));
		return $this->tcpTable;
	}
	public function getUDPStats(){
		if($this->udpStats==NULL) $this->udpStats=GESA::data_to_arrays($this->nrpe_all("get_snmp_udpstats"));
		return $this->udpStats;
	}
	public function getUDPTable(){
		if($this->udpTable==NULL) $this->udpTable=GESA::data_to_arrays($this->nrpe_all("get_snmp_udpconn"));
		return $this->udpTable;
	}
	
	public function getIPStats(){
		if($this->ipStats==NULL) $this->ipStats=GESA::data_to_arrays($this->nrpe_all("get_snmp_ipstats"));
		return $this->ipStats;
	}
	
	private function nrpe_all($cmd){
		$min_line=0;
		$data=array();
		if( $this->getSNMPVersion() != 3 ){
			$community=$this->getSNMPCommunity();
			do{
				$arr=$this->getGESA()->check_nrpe($this,$cmd,$community." ".$min_line);
				$data=array_merge($data,$arr["data"]);
				$min_line = count($data);
			}while($arr["status"]=="MORE");		
		}
		else{
			$snmpReq = $this->getSNMPRequest();
			$level = $snmpReq->getSNMP3SecLevel();
                        if( $level == 'authPriv' ){
                        	$priv_prot = $snmpReq->getPrivacyProtocol();
                                $priv_pwd = $snmpReq->getPrivacyKey();
                                $trailer = " -x $priv_prot -X $priv_pwd";
			}
                        else $trailer='';
                        $user = $snmpReq->getSNMP3User();
                        $auth_prot = $snmpReq->getSNMP3AuthProtocol();
                        $auth_pwd = $snmpReq->getSNMP3AuthKey();
			do{
                                $arr=$this->getGESA()->check_ecatp($this,$cmd.'.pl',"-l $min_line -v3 -u $user -a $auth_prot -A $auth_pwd -L $level".$trailer);
                                $data=array_merge($data,$arr["data"]);
                                $min_line = count($data);
                        }while($arr["status"]=="MORE");
		}
		return $data;
	}
	
}
?>
