<?php
/**
 * @name RemoteServer
* @description Classe de base gérant les serveurs du plugin "Server"
* @author Nicolas Biersart <n.biersart@esia-network.com>
* @contributor
* @copyright Nicolas Biersart 2012
* @license
* @esia 1.00
* @version 1.0.0
* @date 07/11/2012
*/
include_once SNMP_CLASS;
include_once LIB_SNMP;
include_once DEVICE_CLASS;
include_once GESA_CLASS;
class RemoteWmi extends RemoteDevice
{
	protected $cpuInfos;
	protected $mbInfos;
	protected $ramInfos;
	protected $netInfos;
	protected $diskInfos;
	protected $osInfos;
	protected $updatesInfos;
	protected $dateInfos;
	
	protected $security;
	
	protected $osDetails;
	protected $servicesDetails;
	protected $featureDetails;
	protected $softwareDetails;

	protected $driveDetails;
	protected $logicalDetails;
	protected $partDetails;
	
	protected $updatesDetails;
	protected $netConfigDetails;
	protected $netAdapterDetails;
	protected $printerDetails;
	
	/*@NAME: __construct
	 *@IN: $dev un objet Device
	*@OUT: /
	*@TREAT: permet de construire un objet server à partir d'un objet "Device"
	*@EXCEPTION: 	/
	*/
	public function __construct($dev=null,$gesa=NULL)
	{
		if(get_class($dev)=='Device')
		{
			parent::constructDevice($dev);
		}
		elseif(get_class($dev)=='RemoteDevice')
		{
			parent::constructDevice($dev);
			$this->gesa = $gesa;
		}
		else
		{
			parent::__construct();
		}
	}
	
	function nrpe_wmic($mode,$min_line,$max_line,$filter,$columns=NULL){
		$gesa=$this->gesa;
		$node=$this;
		
		if($columns == null ) $columns='null';
		
		$trealer_args=$filter;
		$trealer_args = $trealer_args . " " .$columns;
	
		$data=array();
		do{
			$args=$mode . " " . "$min_line " . $trealer_args;
			//	return $args;
			$arr=$gesa->check_nrpe($node,"get_wmi_info",$args);
			$data=array_merge($data,$arr["data"]);
			$min_line = count($data);
			if($max_line != -1 && $min_line<$max_line) break;
		}while($arr["status"]=="MORE");
	
		if(  $max_line == -1 ) return $data;
		else return array_slice($data,0,$max_line);
	}
	
	function generic_security_parser($data,$type_security,$other=null){
		$state=array();
		$state[262144]=array("def"=>"Updated","protection"=>"Disabled");
		$state[262160]=array("def"=>"Outdated","protection"=>"Disabled");
		$state[266240]=array("def"=>"Updated","protection"=>"Enabled");
		$state[266256]=array("def"=>"Outdated","protection"=>"Enabled");
		$state[393216]=array("def"=>"Updated","protection"=>"Disabled");
		$state[393232]=array("def"=>"Outdated","protection"=>"Disabled");
		$state[393488]=array("def"=>"Outdated","protection"=>"Disabled");
		$state[397312]=array("def"=>"Updated","protection"=>"Enabled");
		$state[397328]=array("def"=>"Outdated","protection"=>"Enabled");
		$state[397584]=array("def"=>"Outdated","protection"=>"Enabled");
	
		if($other==null) $other=array();
		$i=count($other);
		$max=$i+count($data);
		for($j=0;$i<$max;$i++,$j++){
			$other[$i]=$data[$j];
	
			if(empty($state[$other[$i]["productState"]])){
				$other[$i]["Definitions"]="Unknown";
				$other[$i]["Protection"]="Unknown";
			}
			else{
				$other[$i]["Definitions"]=$state[$other[$i]["productState"]]["def"];
				$other[$i]["Protection"]=$state[$other[$i]["productState"]]["protection"];
			}
			unset($other[$i]["productState"]);
			$other[$i]["Type"]=$type_security;
		}
		return $other;
	}
	
	public function getCpuInfos(){
		if($this->cpuInfos==NULL){
			$result = $this->nrpe_wmic("cpu",0,-1,"Name=%%","1,2,3");
			$data=array();
			for($i=1;$i<count($result);$i++){
				$vals=GESA::line_to_array($result[0], $result[$i]);
			
				$usage=$this->gesa->check_nrpe($this,"check_wmic_cpu",'cpu '.$vals["DeviceID"]);
				preg_match('#(\d*)%$#',$usage["data"][0],$matches);
			
				addpercent($data,$vals["DeviceID"], $matches[1],$vals["Name"]);
			}
			$this->cpuInfos=$data;
		}
		return $this->cpuInfos;
	}

	public function getMotherBoardInfos(){
		if($this->mbInfos==NULL) $this->mbInfos=GESA::data_to_arrays($this->nrpe_wmic("mb",0,2,"Caption=%%",null));
		return $this->mbInfos;
	}
	
	public function getRamInfos(){
		if($this->ramInfos==NULL) $this->ramInfos=$this->gesa->check_nrpe($this,"check_wmic","ram");
		return $this->ramInfos;
	}
	
	public function getNetInfos(){
		if($this->netInfos==NULL) $this->netInfos=$this->nrpe_wmic("net_config",1,-1,"MACAddress=%:%:%:%:%:%","0");
		return $this->netInfos;
	}
	
	public function getDiskInfos(){
		if($this->diskInfos==NULL) $this->diskInfos=$this->nrpe_wmic("disk_logical",0,-1,"Name=%%",null);
		return $this->diskInfos;
	}
	
	public function getOsInfos(){
		if($this->osInfos==NULL) $this->osInfos=$this->nrpe_wmic("os",0,-1,"Name=%%","1,6,10");
		return $this->osInfos;
	}
	
	public function getUpdatesInfos(){
		if($this->updatesInfos==NULL) $this->updatesInfos=$this->nrpe_wmic("updates",1,-1,"Caption=%%","4");
		return $this->updatesInfos;
	}
	
	public function getSecurity($type=null){
		$columns="1,3,4,5";
		if($this->security==NULL){
			$antivirus=GESA::data_to_arrays($this->nrpe_wmic( "antivirus",0,-1,"displayName=%%",$columns));
			$firewall=GESA::data_to_arrays($this->nrpe_wmic( "firewall",0,-1,"displayName=%%",$columns));
			$spyware=GESA::data_to_arrays($this->nrpe_wmic( "spyware",0,-1,"displayName=%%",$columns));
				
			$this->security=$this->generic_security_parser($antivirus,"antivirus");
			$this->security=$this->generic_security_parser($firewall,"firewall",$this->security);
			$this->security=$this->generic_security_parser($spyware,"spyware",$this->security);
		}
		$ret=array();
		if($type!=null){
			foreach($this->security as $value){
				if($value["Type"]==$type){
					array_push($ret,$value);
				}
			}
			return $ret;
		}
		return $this->security;
	}
	
	public function getDateInfos(){
		if($this->dateInfos==NULL) $this->dateInfos=$this->nrpe_wmic("os",1,1,"Name=%%","4");
		return $this->dateInfos;
 	}
 	
 	public function getOsDetails(){
 		if($this->osDetails==NULL) $this->osDetails=GESA::data_to_arrays($this->nrpe_wmic( "os",0,-1,"Name=%%"));
 		return $this->osDetails;
 	}
 	
 	public function getServicesDetails(){
 		if($this->servicesDetails==NULL) $this->servicesDetails=GESA::data_to_arrays($this->nrpe_wmic( "services",0,-1,"Name=%%","1,2,3,4,5,6,7"));
 		return $this->servicesDetails;
 	}
 	
 	public function getFeatureDetails(){
 		if($this->featureDetails==NULL) $this->featureDetails=GESA::data_to_arrays($this->nrpe_wmic( "feature",0,-1,"Name=%%","1,4,5"));
 		return $this->featureDetails;
 	}
 	
 	public function getSoftwareDetails(){
 		if($this->softwareDetails==NULL) $this->softwareDetails=GESA::data_to_arrays($this->nrpe_wmic( "software",0,-1,"Name=%%","4,5,6,7,8,9,10,11"));
 		return $this->softwareDetails;
 	}
 	
 	public function getDriveDetails(){
 		if($this->driveDetails==NULL) $this->driveDetails=GESA::data_to_arrays($this->nrpe_wmic( "disk_drive",0,-1,"Name=%%","1,3,8,9,11,12,13"));
 		return $this->driveDetails;
 	}
 	
 	public function getLogicalDetails(){
 		if($this->logicalDetails==NULL) $this->logicalDetails=GESA::data_to_arrays($this->nrpe_wmic( "disk_logical",0,-1,"Name=%%","9,11,15,16,21,25,31,38,39,40"));
 		return $this->logicalDetails;
 	} 					
 	
 	public function getPartDetails(){
 		if($this->partDetails==NULL) $this->partDetails=GESA::data_to_arrays($this->nrpe_wmic( "disk_part",0,-1,"Name=%%","4,5,12,17,21,24,25,27,28,29,34"));
 		return $this->partDetails;
 	}
 	
	public function getUpdatesDetails(){
		if($this->updatesDetails==NULL) $this->updatesDetails=GESA::data_to_arrays($this->nrpe_wmic( "updates",0,-1,"Caption=%%","1,2,3,4"));
		return $this->updatesDetails;
	}
	
	public function getNetConfigDetails(){
		if($this->netConfigDetails==NULL) $this->netConfigDetails=GESA::data_to_arrays($this->nrpe_wmic( "net_config",0,-1,"MACAddress=%:%:%:%:%:%",null));
		return $this->netConfigDetails;
	}
	
	public function getNetAdapterDetails(){
		if($this->netAdapterDetails==NULL) $this->netAdapterDetails=GESA::data_to_arrays($this->nrpe_wmic( "net_adapter",0,-1,"MACAddress=%:%:%:%:%:%",null));
		return $this->netAdapterDetails;
	}
	
	public function getPrinterDetails(){
		if($this->printerDetails==NULL) $this->printerDetails=GESA::data_to_arrays($this->nrpe_wmic( "printer",0,-1,"Name=%%","1,2,3"));
		return $this->printerDetails;
	}
}
?>
