<?php
/**
 * @name RemoteServer
* @description Classe de base gérant les serveurs du plugin "Server"
* @author Nicolas Biersart <n.biersart@esia-network.com>
* @contributor
* @copyright Nicolas Biersart 2012
* @license
* @esia 1.00
* @version 1.0.0
* @date 07/11/2012
*/
include_once SNMP_CLASS;
include_once LIB_SNMP;
include_once DEVICE_CLASS;
include_once GESA_CLASS;
$path = $_SERVER['DOCUMENT_ROOT'];
include_once $path.'common.php';

class Wmi extends Device
{
	protected $cpuInfos;
	protected $mbInfos;
	protected $ramInfos;
	protected $netInfos;
	protected $diskInfos;
	protected $osInfos;
	protected $updatesInfos;
	protected $dateInfos;
	
	protected $security;
	
	protected $osDetails;
	protected $servicesDetails;
	protected $featureDetails;
	protected $softwareDetails;

	protected $driveDetails;
	protected $logicalDetails;
	protected $partDetails;
	
	protected $updatesDetails;
	protected $netConfigDetails;
	protected $netAdapterDetails;
	protected $printerDetails;
	
	/*@NAME: __construct
	 *@IN: $dev un objet Device
	*@OUT: /
	*@TREAT: permet de construire un objet server à partir d'un objet "Device"
	*@EXCEPTION: 	/
	*/
	public function __construct($dev=null)
	{
		if(get_class($dev)=='Device')
		{
			parent::constructDevice($dev);
		}
		else
		{
			parent::__construct();
		}
	}
	
	function exec_check($command,$args){
		exec(PLUGINS_PATH.'/'.$command.' '.$args,$output,$returnexec);
					
		if(empty($output[0]))  throw new Exception($lang['WMI_ERROR_Execution']);
		if(preg_match('#UNKNOWN\s+ERROR#',$output[0]) || $returnexec==3 ) throw new Exception($output[0]);
		if(preg_match("#NRPE: Command $command not defined#",$output[0])) throw new Exception($output[0]);
		
		preg_match("#<(\w*)>#",end($output),$matches);
		if(empty($matches[1]))  return array("status"=>"CHECK",count($output),"data"=>$output);
		
		array_pop($output);
		
		$data=array();
		foreach($output as $key => $value){
			$data[$key]=str_getcsv ( $value , ",", '"','"');
		}
		
		return array("status"=>$matches[1],"size"=>count($output),"data"=>$data);
	}
	
	function wmic($mode,$min_line,$max_line,$filter,$columns=NULL){
		$node=$this->getIPv4Adress();
		
		$trealer_args=" -f ".$filter;
		if (isset($columns)) {
			$trealer_args = $trealer_args . " -c " .$columns;
		}
	
		$data=array();
		do{
			$args=$auth." -H ".$node." -m ". $mode . " -l " . "$min_line " . $trealer_args;

			
			$arr=$this->exec_check("get_wmic.pl",$args);
			$data=array_merge($data,$arr["data"]);
			$min_line = count($data);
			if($max_line != -1 && $min_line<$max_line) break;
		}while($arr["status"]=="MORE");
	
		if(  $max_line == -1 ) return $data;
		else return array_slice($data,0,$max_line);
	}
	
	function generic_security_parser($data,$type_security,$other=null){
		$state=array();
		$state[262144]=array("def"=>"Updated","protection"=>"Disabled");
		$state[262160]=array("def"=>"Outdated","protection"=>"Disabled");
		$state[266240]=array("def"=>"Updated","protection"=>"Enabled");
		$state[266256]=array("def"=>"Outdated","protection"=>"Enabled");
		$state[393216]=array("def"=>"Updated","protection"=>"Disabled");
		$state[393232]=array("def"=>"Outdated","protection"=>"Disabled");
		$state[393488]=array("def"=>"Outdated","protection"=>"Disabled");
		$state[397312]=array("def"=>"Updated","protection"=>"Enabled");
		$state[397328]=array("def"=>"Outdated","protection"=>"Enabled");
		$state[397584]=array("def"=>"Outdated","protection"=>"Enabled");
	
		if($other==null) $other=array();
		$i=count($other);
		$max=$i+count($data);
		for($j=0;$i<$max;$i++,$j++){
			$other[$i]=$data[$j];
	
			if(empty($state[$other[$i]["productState"]])){
				$other[$i]["Definitions"]="Unknown";
				$other[$i]["Protection"]="Unknown";
			}
			else{
				$other[$i]["Definitions"]=$state[$other[$i]["productState"]]["def"];
				$other[$i]["Protection"]=$state[$other[$i]["productState"]]["protection"];
			}
			unset($other[$i]["productState"]);
			$other[$i]["Type"]=$type_security;
		}
		return $other;
	}
	
	public function getCpuInfos(){
		if($this->cpuInfos==NULL){
			$result = $this->wmic("cpu",0,-1,"Name=%%","1,2,3");
			$data=array();
			for($i=1;$i<count($result);$i++){
				$vals=GESA::line_to_array($result[0], $result[$i]);

				$usage=$this->exec_check("check_wmic.pl"," -H ".$this->getIPv4Adress()." -m cpu -n ".$vals["DeviceID"]);
				preg_match('#(\d*)%$#',$usage["data"][0],$matches);
			
				addpercent($data,$vals["DeviceID"], $matches[1],$vals["Name"]);
			}
			$this->cpuInfos=$data;
		}
		return $this->cpuInfos;
	}

	public function getMotherBoardInfos(){
		if($this->mbInfos==NULL) $this->mbInfos=GESA::data_to_arrays($this->wmic("mb",0,2,"Caption=%%",null));
		return $this->mbInfos;
	}
	
	public function getRamInfos(){
		if($this->ramInfos==NULL) $this->ramInfos=$this->exec_check("check_wmic.pl"," -H ".$this->getIPv4Adress()." -m ram");
		return $this->ramInfos;
	}
	
	public function getNetInfos(){
		if($this->netInfos==NULL) $this->netInfos=$this->wmic("net_config",1,-1,"MACAddress=%:%:%:%:%:%","0");
		return $this->netInfos;
	}
	
	public function getDiskInfos(){
		if($this->diskInfos==NULL) $this->diskInfos=$this->wmic("disk_logical",0,-1,"Name=%%",null);
		return $this->diskInfos;
	}
	
	public function getOsInfos(){
		if($this->osInfos==NULL) $this->osInfos=$this->wmic("os",0,-1,"Name=%%","1,6,10");
		return $this->osInfos;
	}
	
	public function getUpdatesInfos(){
		if($this->updatesInfos==NULL) $this->updatesInfos=$this->wmic("updates",1,-1,"Caption=%%","4");
		return $this->updatesInfos;
	}
	
	public function getSecurity($type=null){
		$columns="1,3,4,5";
		if($this->security==NULL){
			$antivirus=GESA::data_to_arrays($this->wmic( "antivirus",0,-1,"displayName=%%",$columns));
			$firewall=GESA::data_to_arrays($this->wmic( "firewall",0,-1,"displayName=%%",$columns));
			$spyware=GESA::data_to_arrays($this->wmic( "spyware",0,-1,"displayName=%%",$columns));
				
			$this->security=$this->generic_security_parser($antivirus,"antivirus");
			$this->security=$this->generic_security_parser($firewall,"firewall",$this->security);
			$this->security=$this->generic_security_parser($spyware,"spyware",$this->security);
		}
		$ret=array();
		if($type!=null){
			foreach($this->security as $value){
				if($value["Type"]==$type){
					array_push($ret,$value);
				}
			}
			return $ret;
		}
		return $this->security;
	}
	
	public function getDateInfos(){
		if($this->dateInfos==NULL) $this->dateInfos=$this->wmic("os",1,1,"Name=%%","4");
		return $this->dateInfos;
 	}
 	
 	public function getOsDetails(){
 		if($this->osDetails==NULL) $this->osDetails=GESA::data_to_arrays($this->wmic( "os",0,-1,"Name=%%"));
 		return $this->osDetails;
 	}
 	
 	public function getServicesDetails(){
 		if($this->servicesDetails==NULL) $this->servicesDetails=GESA::data_to_arrays($this->wmic( "services",0,-1,"Name=%%","1,2,3,4,5,6,7"));
 		return $this->servicesDetails;
 	}
 	
 	public function getFeatureDetails(){
 		if($this->featureDetails==NULL) $this->featureDetails=GESA::data_to_arrays($this->wmic( "feature",0,-1,"Name=%%","1,4,5"));
 		return $this->featureDetails;
 	}
 	
 	public function getSoftwareDetails(){
 		if($this->softwareDetails==NULL) $this->softwareDetails=GESA::data_to_arrays($this->wmic( "software",0,-1,"Name=%%","4,5,6,7,8,9,10,11"));
 		return $this->softwareDetails;
 	}
 	
 	public function getDriveDetails(){
 		if($this->driveDetails==NULL) $this->driveDetails=GESA::data_to_arrays($this->wmic( "disk_drive",0,-1,"Name=%%","1,3,8,9,11,12,13"));
 		return $this->driveDetails;
 	}
 	
 	public function getLogicalDetails(){
 		if($this->logicalDetails==NULL) $this->logicalDetails=GESA::data_to_arrays($this->wmic( "disk_logical",0,-1,"Name=%%","9,11,15,16,21,25,31,38,39,40"));
 		return $this->logicalDetails;
 	} 					
 	
 	public function getPartDetails(){
 		if($this->partDetails==NULL) $this->partDetails=GESA::data_to_arrays($this->wmic( "disk_part",0,-1,"Name=%%","4,5,12,17,21,24,25,27,28,29,34"));
 		return $this->partDetails;
 	}
 	
	public function getUpdatesDetails(){
		if($this->updatesDetails==NULL) $this->updatesDetails=GESA::data_to_arrays($this->wmic( "updates",0,-1,"Caption=%%","1,2,3,4"));
		return $this->updatesDetails;
	}
	
	public function getNetConfigDetails(){
		if($this->netConfigDetails==NULL) $this->netConfigDetails=GESA::data_to_arrays($this->wmic( "net_config",0,-1,"MACAddress=%:%:%:%:%:%",null));
		return $this->netConfigDetails;
	}
	
	public function getNetAdapterDetails(){
		if($this->netAdapterDetails==NULL) $this->netAdapterDetails=GESA::data_to_arrays($this->wmic( "net_adapter",0,-1,"MACAddress=%:%:%:%:%:%",null));
		return $this->netAdapterDetails;
	}
	
	public function getPrinterDetails(){
		if($this->printerDetails==NULL) $this->printerDetails=GESA::data_to_arrays($this->wmic( "printer",0,-1,"Name=%%","1,2,3"));
		return $this->printerDetails;
	}
}
?>
